/* vsiptest_blockbind_i.c Version 0.000 April 7, 1997
// 
// This file contains the vsiptest_blockbind_i test functionality.
//
// Functional Description:
//   vsip_blockbind_i binds to a public data array with at least N
//   vsip_scalar_i real elements.
//
// Copyright May 1997, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Thomas Egolf
//         egolf@ece.gatech.edu
//         Dan Campbell [adapted to _i]
//         dan.campbell@gtri.gatech.edu
//
// Test description: This test suite will create real blocks of data with
//      various sizes using the malloc function call and then bind the
//      public block to a private block using this function. This test is
//      similar to the block create functionality however in this case the
//      block is created by the application developer.
//
//      Check some of the parameters of this function
//        1) Check the first and last array elements after binding
//        2) Check the size of the blocks
//        3) Check for all possible hints
//        4) Check for binding for negative input lengths (error conditions)
//
// VSIP functions called: vsip_blockbind_i, vsip_vbind_i, vsip_vget_i,
//                        vsip_vgetlength_i, vsip_vdestroy_i
//
// Revision History:
// May   6, 1997  Added copyright notice.             
// May   6, 1997  Executed tests 1-4 without error and block sizes up to 5000 
// April 7, 1999  Adapted to _i. hard to test some things
*/

#include<stdio.h>
#include<stdlib.h>  /* For malloc functionality */
#include "vsiptest_block.h"

int main ()
{
  size_t             block_size, i, j;
  vsip_scalar_i     *public_block;
  vsip_memory_hint   hint = 0;
  vsip_block_i      *bind_block;
  vsip_vview_i      *blockview;
  vsip_length        length;
  vsip_vattr_i       attrib;

  int ret = 0;

  vsip_init ((void *)0);
  public_block = NULL;

  /* API Checks */
  bind_block = vsip_blockbind_i (public_block, (vsip_length)(1), hint);
  if (bind_block == NULL)
  {
    vtprintf("Error in vsip_blockbind_i(); (1) \n"
             "block bind to NULL pointer returns failure\n");
    ret |= 1<<1;
  }

  /* Check various public block sizes from 2 to block_length_max */
  for (j=0; j<elements(VSIPTEST_BLOCKBIND_SIZE_ARRAY); j++)
  {
    block_size=VSIPTEST_BLOCKBIND_SIZE_ARRAY[j];

    public_block = (vsip_scalar_i *)malloc(block_size*sizeof(vsip_scalar_i));

    /* Fill the block with data values equal to its index value */
    for(i=0; i<block_size; i++) 
    {
      *(public_block + i) = (float)i;
    }

    /********************* VSIP_BLOCKBIND_D **************************
      Bind to the public block with at least block_size real
      elements and look at the contents of the binded block. */

    bind_block = vsip_blockbind_i(public_block, block_size, hint);
    vsip_blockadmit_i (bind_block, 1);

    /****************************************************************/

    /********************* VSIP_VBIND_D *****************************
      Create a view on the bound block */

    blockview = vsip_vbind_i(bind_block, 0, 1, (vsip_length)block_size);

    /* Check some of the parameters of this function
    //    1) Check the first and last array elements
    //    2) Check the size of the block */

    /********************* VSIP_VGET_D *******************************
      Check the minimum and maximum values of the view on the block */
#if 0
    index1 = 0; index2 = block_size-1;
    minval = vsip_vget_i(blockview, index1);
    maxval = vsip_vget_i(blockview, index2);

    if(minval != (vsip_scalar_i)0.0) 
    {
          vtprintf("Error in vsip_blockbind_i(); (2) \n"
       "First value assignment does not match first element value \n");
          ret |= 1<<2;
    }

    if(maxval != (vsip_scalar_i)(block_size-1)) 
    {
      vtprintf("Error in vsip_blockbind_i(); (3) \n"
               "Last value assignment does not match last element value \n");
      ret |= 1<<3;
    }
#endif
    /********************* VSIP_VGETLENGTH_D ************************
      Check the length of the block allocated */

    vsip_vgetattrib_i(blockview, &attrib);
    length = attrib.length;
    if(length != (vsip_length)block_size)
    {
      vtprintf("Error in vsip_blockbind(); (4) \n"
       "Size does not match allocated size \n");
      ret |= 1<<4;
    }

    /********************* VSIP_VDESTROY_D ***************************
      Destroy the view on the block */
    vsip_vdestroy_i(blockview);

    free(public_block);
  }
  report (ret);
  vsip_finalize ((void *)0);
  return (ret!=0);
}
