/* vsiptest_blockcreate_d.c Version 0.000 May 5, 1997
//
// This file contains the vsiptest_blockcreate_d test functionality.
//
// Functional Description:
//   vsip_blockcreate_f creates a private data array with at least N
//   vsip_scalar_f real scalar elements.
//
//
// Copyright March 1999, Georgia Tech Research Institute.
// This is an unpublished work. All rights reserved.
//
// Author: Randall Janka
//         randall.janka@gtri.gatech.edu
//
// Test description: This test suite will create real blocks of data with
//      various sizes and with various hints as input. Currently the hints
//      are not supported.
//
//      Check some of the parameters of this function
//        1) Check the first and last array elements
//        2) Check the size of the blocks
//        3) Check for all possible hints
//        4) Check for block sizes of negative numbers (error conditions)
//
// VSIP functions called: vsip_blockcreate_f, vsip_vbind_f, vsip_vramp_f,
//                        vsip_vminval_f, vsip_vmaxval_f, vsip_vgetlength_f,
//                        vsip_vdestroy_f, vsip_blockdestroy_f
//
// Revision History:
// May 5, 1997  Added copyright notice.
// May 6, 1997  Executed tests 1-4 without error and block sizes up to 5000 */

#include <stdio.h>
#include "vsiptest_block.h"

int main()
{
  size_t           block_size;
  vsip_memory_hint hint = 0;
  vsip_block_f     *block;
  vsip_vview_f     *blockview;
  vsip_scalar_f    minval, maxval;
  vsip_vattr_f     attrib;
  vsip_length      length;
  int              i, ret = 0;

  vsip_init ((void *)0);
  /* Check various legal block sizes from 2 to block_length_max */
  for (i=0; i<elements(VSIPTEST_BLOCKCREATE_F_BLOCK_SIZE); i++)
  {
    block_size = VSIPTEST_BLOCKCREATE_F_BLOCK_SIZE[i];
    /********************* VSIP_BLOCKCREATE_f ***********************
      Create the block */
    block = vsip_blockcreate_f(block_size, hint);

    /********************* VSIP_VBIND_f *****************************
      Create a view on the block */
    blockview = vsip_vbind_f(block, 0, 1, (vsip_length)block_size);

    /********************* VSIP_VRAMP_f *****************************
      Fill the vector view with a ramp from 0 to block_size-1 */
    vsip_vramp_f(0.0,1.0,blockview);

    /* Check some of the parameters of this function
    //    1) Check the first and last array elements
    //    2) Check the size of the block */

    /********************* VSIP_VMINVAL_f *****************************
     ********************* VSIP_VMAXVAL_f *****************************
      Check the minimum and maximum values of the view on the block */
    minval = vsip_vget_f (blockview, 0);
    maxval = vsip_vget_f (blockview, block_size-1);

    if(vtfcomp (minval, (vsip_scalar_f)0.0) )
    {
      vtprintf("Error in vsip_blockcreate_f(); (1) \n"
               "First value assignment does not match first element value \n");
      ret |= 1<<1;
      break;
    }

    if(vtfcomp (maxval, (vsip_scalar_f)(block_size-1))  )
    {
      vtprintf("Error in vsip_blockcreate_f(); (2) \n"
               "Last value assignment does not match last element value \n");
      ret |= 1<<2;
      break;
    }

    /********************* VSIP_VGETLENGTH_f ************************
      Check the length of the block allocated */

  /*-----------------3/26/99 10:36AM------------------
   * Use getattribs for checking length
   * --------------------------------------------------*/

    vsip_vgetattrib_f(blockview, &attrib);
    length = attrib.length;
    if(length != (vsip_length)block_size)
    {
      /* Report error */
      vtprintf("Error in vsip_blockcreate_f(); (3) \n"
               "Size does not match allocated size \n");
      ret |= 1<<3;
      break;
    }

    /********************* VSIP_VDESTROY_f ****************************
     ********************* VSIP_BLOCKDESTROY_f ************************
      Destroy the block after checking all the attributes */

  /*-----------------3/26/99 10:37AM------------------
   *  Do a blockdestroy, then bind a view to destroyed block and see
   *  if it fails.
   * --------------------------------------------------*/

    vsip_blockdestroy_f(vsip_vdestroy_f(blockview));

  }
  report (ret);
  vsip_finalize ((void *)0);
  return (ret != 0);
}
