/* vsiptest_blockcreate_i.c Version 0.000 April 7, 1999
//
// This file contains the vsiptest_blockcreate_i test functionality.
//
// Functional Description:
//   vsip_blockcreate_i creates a private data array with at least N
//   vsip_scalar_i real scalar elements.
//
//
// Copyright March 1999, Georgia Tech Research Institute.
// This is an unpublished work. All rights reserved.
//
// Author: Randall Janka
//         randall.janka@gtri.gatech.edu
//         Dan Campbell [adapted to _i]
//         dan.campbell@gtri.gatech.edu
//
// Test description: This test suite will create real blocks of data with
//      various sizes and with various hints as input. Currently the hints
//      are not supported.
//
//      Check some of the parameters of this function
//        1) Check the first and last array elements
//        2) Check the size of the blocks
//        3) Check for all possible hints
//        4) Check for block sizes of negative numbers (error conditions)
//
// VSIP functions called: vsip_blockcreate_i, vsip_vbind_i, vsip_vramp_i,
//                        vsip_vminval_i, vsip_vmaxval_i, vsip_vgetlength_i,
//                        vsip_vdestroy_i, vsip_blockdestroy_i
//
// Revision History:
// May   5, 1997  Added copyright notice.
// May   6, 1997  Executed tests 1-4 without error and block sizes up to 5000 
// April 7, 1999 Adapted to _i.  Some tests not available
*/

#include <stdio.h>
#include "vsiptest_block.h"

int main()
{
  size_t           block_size;
  vsip_memory_hint hint = 0;
  vsip_block_i     *block;
  vsip_vview_i     *blockview;
  vsip_vattr_i     attrib;
  vsip_length      length;
  int              i,ret = 0;

  vsip_init ((void *)0);
  /* Check various legal block sizes from 2 to block_length_max */
  for (i=0; i<elements(VSIPTEST_BLOCKCREATE_I_BLOCK_SIZE); i++)
  {
    block_size = VSIPTEST_BLOCKCREATE_I_BLOCK_SIZE[i];

    /********************* VSIP_BLOCKCREATE_i ***********************
      Create the block */
    block = vsip_blockcreate_i(block_size, hint);

    /********************* VSIP_VBIND_i *****************************
      Create a view on the block */
    blockview = vsip_vbind_i(block, 0, 1, (vsip_length)block_size);

#if 0
  /* can't do this in _i */

    /********************* VSIP_VRAMP_i *****************************
      Fill the vector view with a ramp from 0 to block_size-1 */
    vsip_vramp_i(0.0,1.0,blockview);

    /* Check some of the parameters of this function
    //    1) Check the first and last array elements
    //    2) Check the size of the block */

    /********************* VSIP_VMINVAL_i *****************************
     ********************* VSIP_VMAXVAL_i *****************************
      Check the minimum and maximum values of the view on the block */
    minval = vsip_vminval_i(blockview, &index1);
    maxval = vsip_vmaxval_i(blockview, &index2);

    if(minval != (vsip_scalar_i)0.0) 
    {
      vtprintf("Error in vsip_blockcreate_i(); (1) \n"
               "First value assignment does not match first element value \n");
      ret |= 1<<1;
      break;
    }

    if(maxval != (vsip_scalar_i)(block_size-1)) 
    {
      vtprintf("Error in vsip_blockcreate_i(); (2) \n"
               "Last value assignment does not match last element value \n");
      ret |= 1<<2;
      break;
    }
#endif

    /********************* VSIP_VGETLENGTH_i ************************
      Check the length of the block allocated */

  /*-----------------3/26/99 10:36AM------------------
   * Use getattribs for checking length
   * --------------------------------------------------*/

    vsip_vgetattrib_i(blockview, &attrib);
    length = attrib.length;
    if(length != (vsip_length)block_size)
    {
      /* Report error */
      vtprintf("Error in vsip_blockcreate_i(); (3) \n"
               "Size does not match allocated size \n");
      ret |= 1<<3;
      break;
    }

    /********************* VSIP_VDESTROY_i ****************************
     ********************* VSIP_BLOCKDESTROY_i ************************
      Destroy the block after checking all the attributes */

  /*-----------------3/26/99 10:37AM------------------
   *  Do a blockdestroy, then bind a view to destroyed block and see
   *  if it fails.
   * --------------------------------------------------*/

    vsip_blockdestroy_i(vsip_vdestroy_i(blockview));

  }
  report (ret);
  vsip_finalize ((void *)0);
  return (ret!=0);
}
