/* vsiptest_blockadmit_f.c Version 0.000 March 26, 1999
//
// This file contains the vsiptest_blockrebind_f functionality.
//
// Functional Description:
//   vsip_blockrebind_f "rebinds" an existing released real block object to a new
//   user data array.  this array must be at least as big as the original.  an attempt
//   to rebind a block in admitted state is an error, and should return null.  otherwise, 
//   a pointer to the old array should be returned.
//
// Copyright March 1997, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description:
//      create data arrays A and B
//      bind a block to array A
//      admit block
//      attempt to rebind to B, verify error
//      release block
//      attempt to rebind to NULL, verify error
//      attempt to rebind to B, verify we returned a pointer to A
//      create a complex block, and a real block derived from it
//      attempt to rebind derived block, verify error.
//      
// Revision History:
// March 26, 1999  Created 
*/


#include <stdio.h>
#include <stdlib.h>
#include "vsiptest_block.h"

int main()
{
  size_t                block_size = VSIPTEST_BLOCKREBIND_BLOCK_SIZE;
  vsip_memory_hint      hint = 0;
  vsip_block_f         *block;
  vsip_scalar_f        *result;
  vsip_block_f         *dblock;
  vsip_cvview_f        *cview;
  vsip_vview_f         *iview; 
  vsip_cblock_f        *cblock;
  
  int i;
  int ret = 0;
  
  vsip_scalar_f        *array1, *array2;

  vsip_init ((void *)0);

  array1 = (vsip_scalar_f *)malloc(block_size*sizeof(vsip_scalar_f));
  array2 = (vsip_scalar_f *)malloc(block_size*sizeof(vsip_scalar_f));
 
  for (i=0; i<block_size; i++)
  {
    array1[i] = (vsip_scalar_f)1.1;
    array2[i] = 2.2;
  }

  block = vsip_blockbind_f (array1, block_size, hint);
  
  if (vsip_blockadmit_f (block, 1))
  {
      vtprintf ("Error in vsip_blockrebind_f(); (1) \n"
                "Could not admit test block\n");
      ret |= 1<<1;
  }

  result = vsip_blockrebind_f (block, array2);
  if (result != NULL)
  {
      vtprintf ("Error in vsip_blockrebind_f(); (2) \n"
                "Rebind of admitted block did not return NULL\n");
      ret |= 1<<2;
  }

  if (!vsip_blockrelease_f (block, 0))
  {
      vtprintf ("Error in vsip_blockrebind_f(); (3) \n"
                "Could not release test block\n");
      ret |= 1<<3;
  }
  
  /*
  result = vsip_blockrebind_f (block, NULL);
  if (result != NULL)
  {
      vtprintf ("Error in vsip_blockrebind_f(); (4) \n"
               "Rebind to NULL did not return NULL\n");
      ret |= 1<<4;
  }
  */

  vsip_blockdestroy_f (block);
  block = vsip_blockbind_f (array1, block_size, hint);
  result = vsip_blockrebind_f (block, array2);
  if (result != array1)
  {
      vtprintf ("Error in vsip_blockrebind_f(); (5) \n"
                "Rebind did not return old data pointer\n");
      ret |= 1<<5;
  }

  vsip_blockdestroy_f (block);

  cblock = vsip_cblockcreate_f (10, 0);
  cview  = vsip_cvbind_f (cblock, 0, 1, 1);
  iview  = vsip_vimagview_f (cview);
  dblock = vsip_vgetblock_f (iview);
 
  result = vsip_blockrebind_f (dblock, array2);
  if (result)
  {
    vtprintf ("Error in vsip_blockrebind_f(); (6)\n"
              "Rebind of a derived block did not fail.\n");
    ret |= 1<<6;
  }

  vsip_cvdestroy_f (cview);
  vsip_cblockdestroy_f (cblock);
  report (ret);
  vsip_finalize ((void *)0);
  return (ret!=0);
}
