/* vsiptest_blockrebind_i.c Version 0.000 March 26, 1999
//
// This file contains the vsiptest_blockrebind_i functionality.
//
// Functional Description:
//   vsip_blockrebind_i "rebinds" an existing released real block object 
//   to a new    user data array.  this array must be at least as big as 
//   the original.  an attempt to rebind a block in admitted state is an
//   error, and should return null.  otherwise, a pointer to the old array 
//   should be returned.
//
// Copyright March 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description:
//      create data arrays A and B
//      bind a block to array A
//      admit block
//      attempt to rebind to B, verify error
//      release block
//      attempt to rebind to NULL, verify error
//      attempt to rebind to B, verify we returned a pointer to A
//      
// Revision History:
// March 26, 1999  Created 
// April  7, 1999  adapted to _i. 
*/


#include <stdio.h>
#include <stdlib.h>
#include "vsiptest_block.h"

int main()
{
  size_t                block_size = VSIPTEST_BLOCKREBIND_BLOCK_SIZE;
  vsip_memory_hint      hint = 0;
  vsip_block_i         *block;
  vsip_scalar_i        *result;

  int i;
  int ret = 0;
  
  vsip_scalar_i        *array1, *array2;

  vsip_init ((void *)0);
  array1 = (vsip_scalar_i *)malloc(block_size*sizeof(vsip_scalar_i));
  array2 = (vsip_scalar_i *)malloc(block_size*sizeof(vsip_scalar_i));
 
  for (i=0; i<block_size; i++)
  {
    array1[i] = (vsip_scalar_i)1.1;
    array2[i] = 2.2;
  }

  block = vsip_blockbind_i (array1, block_size, hint);
  
  if (vsip_blockadmit_i (block, 1))
  {
      vtprintf ("Error in vsip_blockrebind_i(); (1) \n"
                "Could not admit test block\n");
      ret |= 1<<1;
  }

  result = vsip_blockrebind_i (block, array2);
  if (result != NULL)
  {
      vtprintf ("Error in vsip_blockrebind_i(); (2) \n"
                "Rebind of admitted block did not return NULL\n");
      ret |= 1<<2;
  }

  if (!vsip_blockrelease_i (block, 0))
  {
      vtprintf ("Error in vsip_blockrebind_i(); (3) \n"
                "Could not release test block\n");
      ret |= 1<<3;
  }
  
  /*
  result = vsip_blockrebind_i (block, NULL);
  if (result != NULL)
  {
      vtprintf ("Error in vsip_blockrebind_i(); (4) \n"
               "Rebind to NULL did not return NULL\n");
      ret |= 1<<4;
  }
  */

  vsip_blockdestroy_i (block);
  block = vsip_blockbind_i (array1, block_size, hint);
  result = vsip_blockrebind_i (block, array2);
  if (result != array1)
  {
      vtprintf ("Error in vsip_blockrebind_i(); (5) \n"
                "Rebind did not return old data pointer\n");
      ret |= 1<<5;
  }

  vsip_blockdestroy_i (block);

  report (ret);
  vsip_finalize ((void *)0);
  return (ret!=0);
}
































