/* vsiptest_blockadmit_f.c Version 0.000 March 29, 1999
//
// This file contains the vsiptest_blockadmit_f test functionality.
//
// Functional Description:
//   vsip_blockadmit_f "admits" a VSIPL block object, vsip_dblock_p,
//   for VSIPL operations on the associated views.
//
// Copyright March 1997, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Randall Janka
//         randall.janka@gtri.gatech.edu
//
// Test description:
 *    Create valid VSIPL block and admit it.
 *     Admit it again.
 *     Destroy block and try to admit it.
 *     Update flag will not be tested in this routine, but in release routine.
//
//      Check some of the parameters of this function
//        1) Check the first and last array elements after binding
//        2) Check the size of the blocks
//        3) Check for all possible hints
//        4) Check for binding for negative input lengths (error conditions)
//
// VSIP functions called: vsip_blockadmit_f, vsip_vbind_f, vsip_vget_f,
//                        vsip_vgetlength_f, vsip_vdestroy_f
//
// Revision History:
// March 26, 1999  Created */


#include <stdio.h>
#include "vsiptest_block.h"

int main()
{
  size_t            block_size = VSIPTEST_BLOCKRELEASE_BLOCK_SIZE;
  vsip_memory_hint  hint = 0;
  vsip_block_f     *block;
  vsip_scalar_bl    update = 0;
  vsip_scalar_f    *result;
  vsip_cblock_f    *cblock;
  vsip_cvview_f    *cview;
  vsip_vview_f     *iview;
  vsip_block_f     *dblock;
  vsip_vview_f     *view;
  int               i;

  int ret = 0;
  static vsip_scalar_f *array1;

  vsip_init ((void *)0);
  array1 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*block_size);

  /********************* VSIP_BLOCKCREATE_f ***********************
   Create the block */
  block = vsip_blockbind_f(array1, block_size, hint);

  /********************* VSIP_BLOCKADMIT_f *****************************
   Try to admit the block */
  result = vsip_blockrelease_f(block, update);
  if (result != array1)
  {
    vtprintf ("Error in vsip_blockrelease_f(); (1)\n"
              "Release of block did not return valid address\n", block_size);
    ret |= 1<<1;
  }
  /*-----------------3/26/99 2:50PM-------------------
   * If release is successful, try to admit it again.
   * --------------------------------------------------*/
  if ( !(result = vsip_blockrelease_f(block, update)))
  {
    vtprintf ("Error in vsip_blockrelease_f(); (2)\n"
              "Failed release of valid released %i-el block. \n", block_size);
    ret |= 1<<2;
  }

  vsip_blockadmit_f (block, update);
  view = vsip_vbind_f (block, 0, 1, block_size);
  vsip_vramp_f(0.0,1,view);
  vsip_blockrelease_f (block, 1);
  for (i=0; i<block_size; i++)
  {
    if (vtfcomp (array1[i], i))
    {
      vtprintf ("Error in vsip_blockrelease_f(); (3)\n"
                "Released updated block element %i was not updated (%f).\n", 
                i, array1[i]);
      ret |= 1<<3;
    }
  }
  
  /*
   * Do a blockdestroy, then try to admit a view to destroyed block
   * and see if it fails.
  */
  vsip_vdestroy_f (view);
  vsip_blockdestroy_f (block);
  block = vsip_blockcreate_f (block_size, 0);
  result = vsip_blockrelease_f (block, update);
  if (result)
  {
      vtprintf ("Error in vsip_blockrelease_f(); (4)\n"
                "Released a VSIP block.\n");
      
    ret |= 1<<4;
  }

  cblock = vsip_cblockbind_f (array1, NULL, 10, 0);
  cview  = vsip_cvbind_f (cblock, 0, 1, 1);
  iview  = vsip_vimagview_f (cview);
  dblock = vsip_vgetblock_f (iview);

  result = vsip_blockrelease_f (dblock, 0);

  if (result)
  {
    vtprintf ("Error in vsip_blockrelease_f(); (5)\n"
              "release of derived block did not return NULL.\n");
    ret |= 1<<5;
  }
  vsip_cvdestroy_f (cview);
  vsip_cblockdestroy_f (cblock);
  report (ret);
  vsip_finalize ((void *)0);
  return (ret!=0);
}
































