/* vsiptest_blockrelease_i.c Version 0.000 March 29, 1999
//
// This file contains the vsiptest_blockrelease_i test functionality.
//
// Functional Description:
//   vsip_blockreleases "releases" a user block object, vsip_dblock_p,
//   for user operations on the associated data.
//
// Copyright March 1997, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Randall Janka
//         randall.janka@gtri.gatech.edu
//         Dan Campbell [adapted to _i]
//         dan.campbell@gtri.gatech.edu
//
// Test description:
 *    Create valid VSIPL block and admit it.
 *     Admit it again.
 *     Destroy block and try to admit it.
 *     Update flag will not be tested in this routine, but in release routine.
//
//      Check some of the parameters of this function
//        1) Check the first and last array elements after binding
//        2) Check the size of the blocks
//        3) Check for all possible hints
//        4) Check for binding for negative input lengths (error conditions)
//
// VSIP functions called: vsip_blockadmit_i, vsip_vbind_i, vsip_vget_i,
//                        vsip_vgetlength_i, vsip_vdestroy_i
//
// Revision History:
// March 26, 1999  Created 
// April  7, 1999  Adapted to _i.  Not all tests available
*/


#include <stdio.h>
#include "vsiptest_block.h"

int main()
{
  size_t            block_size = VSIPTEST_BLOCKRELEASE_BLOCK_SIZE;
  vsip_memory_hint  hint = 0;
  vsip_block_i     *block;
  vsip_scalar_bl    update = 0;
  vsip_scalar_i    *result;

  int ret = 0;
  static vsip_scalar_i *array1;

  vsip_init ((void *)0);

  array1 = (vsip_scalar_i *)malloc(sizeof(vsip_scalar_i)*block_size);

  /********************* VSIP_BLOCKCREATE_i ***********************
   Create the block */
  block = vsip_blockbind_i(array1, block_size, hint);

  /********************* VSIP_BLOCKADMIT_i *****************************
   Try to admit the block */
  result = vsip_blockrelease_i(block, update);
  if (result != array1)
  {
    vtprintf ("Error in vsip_blockrelease_i(); (1)\n"
              "Release of block did not return valid address\n", block_size);
    ret |= 1<<1;
  }
  /*-----------------3/26/99 2:50PM-------------------
   * If release is successful, try to admit it again.
   * --------------------------------------------------*/
  if ( !(result = vsip_blockrelease_i(block, update)))
  {
    vtprintf ("Error in vsip_blockrelease_i(); (2)\n"
              "Failed release of valid released %i-el block. \n", block_size);
    ret |= 1<<2;
  }

#if 0
/* this is not available in _i */

  vsip_blockadmit_i (block, update);
  view = vsip_vbind_i (block, 0, 1, block_size);
  vsip_vramp_i(0.0,1,view);
  vsip_blockrelease_i (block, 1);
  for (i=0; i<block_size; i++)
  {
    if (array1[i] != i)
    {
      vtprintf ("Error in vsip_blockrelease_i(); (3)\n"
                "Released updated block element %i was not updated (%f).\n", i, array1[i]);
      ret |= 1<<3;
    }
  }
  vsip_blockdestroy_i (block);

#endif
  block = vsip_blockcreate_i (block_size, 0);
  result = vsip_blockrelease_i (block, update);
  if (result)
  {
      vtprintf ("Error in vsip_blockrelease_i(); (4)\n"
                "Released a VSIP block.\n");
      
    ret |= 1<<3;
  }
  report (ret);
  vsip_finalize ((void *)0);
  return (ret!=0);
}

