/* vsiptest_cblockdestroy_f.c Version 0.000 April 21, 1999
// 
// This file contains the vsiptest_cblockdestroy_f test functionality.
//
// Functional Description:
//   vsip_cblockdestroy_f destroys the pointer (vsip_block_f* b) to the
//
// Copyright April 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Thomas Egolf
//         egolf@ece.gatech.edu
//         Dan Campbell [adapted to c*_f]
//         dan.campbell@gtri.gatech.edu
//
//
// Test description: This test suite will create real public blocks of data
//      with various sizes using the malloc function call and then bind the
//      public block to a private block using the function vsip_blockbind_f.
//      Also a set of tests will create private blocks using the
//      vsip_blockcreate_f and vsip_cblockcreate_f functions and test
//      whether the value returned by vsip_blockfind_f is null. Another set
//      of tests verifies that when there are multiple bindings to a private
//      object, the vsip_blockdestroy_f function will destroy the object
//      even if all bindings are not removed. In this case the user has the
//      option of setting the number of views to create and destroy using
//      the VSIP function vsip_vbind_f.c and vsip_vdestroy_f.c. If the
//      number of views created is greater than the number of views destroyed
//      then the vsip_blockdestroy_f function will warn the user if the
//      VSIP_DEVELOPMENT_MODE switch is turned on. It will notify the user how
//      many bindings are still attached to the block object.
//
//      Check some of the parameters of this function
//        1) Verify that when a public array is bound to a private area
//           and destroyed by the vsip_blockdestroy_f function, the returned
//           value is a pointer to the public data array. In this test we
//           check the pointer to the array that was returned from the
//           vsip_blockdestroy_f function and verify that the data is the
//           same as initially used to fill the array.
//        2) Verify that when a private block of data is destroyed using
//           vsip_blockdestroy_f, the returned value of the function is a
//           null pointer.
//        3) Verify that when there are multiple binding to a private object,
//           a call to vsip_blockdestroy_f does destroy the object even if
//           all the bindings are not removed.
//
// VSIP functions called: vsip_blockdestroy_f, vsip_blockcreate_f,
//                        vsip_cblockcreate_f, vsip_blockbind_f,
//                        vsip_vbind_f.c, vsip_vdestroy_f.c
//                        
//
// Revision History:
// May 6, 1997  Added copyright notice.             
// May 7, 1997  Executed tests 1-3 without error and block sizes up to 5000 */

#include<stdio.h>
#include<stdlib.h>  /* For malloc functionality */
#include "vsiptest_block.h"

int main()
{

  size_t            block_size;
  vsip_memory_hint  hint = 0;
  vsip_scalar_f    *public_block;
  vsip_cblock_f    *bind_block;
  int               j;
   
  vsip_init ((void *)0);

  /* Check various public block sizes from 1 to block_length_max */
  for (j=0; j<elements(VSIPTEST_CBLOCKDESTROY_F_BLOCK_SIZE); j++)
  {
    block_size = VSIPTEST_CBLOCKDESTROY_F_BLOCK_SIZE[j];
    public_block = (vsip_scalar_f*)malloc(block_size*2*sizeof(vsip_scalar_f));

    /********************* VSIP FUNCTION CALL ***********************
      Bind to the public block with at least block_size vsip_scalar_f
      real elements and look at the contents of the binded block. */

    /****************************************************************/

    bind_block = vsip_cblockbind_f (public_block, NULL, block_size, hint);
    vsip_cblockadmit_f (bind_block, 1);
    vsip_cblockdestroy_f (bind_block);
  }
  report (0);
  vsip_finalize ((void *)0);
  return (0);
}
