/* vsiptest_cblockrelease.c Version 0.000 April 23, 1999
//
// This file contains the vsiptest_cblockrelease test functionality.
//
// Functional Description:
//   vsip_blockadmit_f "admits" a VSIPL block object, vsip_dblock_p,
//   for VSIPL operations on the associated views.
//
// Copyright March 1997, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Randall Janka
//         randall.janka@gtri.gatech.edu
//         Dan Campbell [adapted to c*_f.]
//         dan.campbell@gtri.gatech.edu
//
// Test description:
//
// Revision History:
// March 26, 1999  Created 
// April 23, 1999  Adapted to c*_f
*/


#include <stdio.h>
#include "vsiptest_block.h"

int main()
{
  size_t            block_size = VSIPTEST_BLOCKRELEASE_BLOCK_SIZE;
  vsip_memory_hint  hint = 0;
  vsip_scalar_bl    update = 0;
  vsip_cvview_f    *ciview;
  vsip_cvview_f    *csview;
  vsip_vview_f     *iiview;
  vsip_vview_f     *riview;
  vsip_vview_f     *isview;
  vsip_vview_f     *rsview;
  vsip_cblock_f    *iblock;
  vsip_cblock_f    *sblock;
  int               i;

  int ret = 0;

  static vsip_scalar_f *array1, *array2, *array3;
  static vsip_scalar_f *prv_data1, *prv_data2, *prv_data3, *prv_data4;

  vsip_init ((void *)0);

  array1 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*block_size);
  array2 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*block_size);
  array3 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*block_size*2);

  /********************* VSIP_BLOCKCREATE_f ***********************
   Create the block */
  sblock = vsip_cblockbind_f(array1, array2, block_size, hint);
  iblock = vsip_cblockbind_f(array3, NULL,   block_size, hint);

  /********************* VSIP_BLOCKADMIT_f *****************************
   Try to admit the block */
  vsip_cblockrelease_f(sblock, update, &prv_data1, &prv_data2);
  vsip_cblockrelease_f(iblock, update, &prv_data3, &prv_data4);

  if ( (prv_data1 != array1) || (prv_data2 != array2) )
  {
    vtprintf ("Error in vsip_cblockrelease_f(); (1)\n"
              "Release of split block did not return valid addresses\n");
    ret |= 1<<1;
  }
  
  if (prv_data3 != array3 || prv_data4 != NULL)
  {
    vtprintf ("Error in vsip_cblockrelease_f(); (2)\n"
              "Release of interleaved block did not return "
              "valid addresses\n");
    ret |= 1<<2;
  }
  /*-----------------3/26/99 2:50PM-------------------
   * If release is successful, try to admit it again.
   * --------------------------------------------------*/

  vsip_cblockrelease_f(sblock, update, &prv_data1, &prv_data2);
  vsip_cblockrelease_f(iblock, update, &prv_data3, &prv_data4);

  if (!prv_data1)
  {
    vtprintf ("Error in vsip_cblockrelease_f(); (3)\n"
              "Failed re-release of valid released split block. \n");
    ret |= 1<<3;
  }

  if (!prv_data3 || prv_data4)
  {
    vtprintf ("Error in vsip_cblockrelease_f(); (4)\n"
              "Failed re-release of valid released interleaved block. \n");
    ret |= 1<<4;
  }
    
  vsip_cblockadmit_f (iblock, update);
  vsip_cblockadmit_f (sblock, update);

  ciview = vsip_cvbind_f (iblock, 0, 1, block_size);
  csview = vsip_cvbind_f (sblock, 0, 1, block_size);

  iiview = vsip_vimagview_f (ciview);
  riview = vsip_vrealview_f (ciview);

  isview = vsip_vimagview_f (csview);
  rsview = vsip_vrealview_f (csview);

  vsip_vramp_f(0.0, 2, riview);
  vsip_vramp_f(1.0, 2, iiview);
  vsip_vramp_f(3.3, 1, rsview);
  vsip_vramp_f(2.2, 1, isview);

  vsip_cblockrelease_f (iblock, 1, &prv_data1, &prv_data2);
  vsip_cblockrelease_f (sblock, 1, &prv_data1, &prv_data2);

  for (i=0; i<block_size; i++)
  {
    if (vtfcomp (array1[i], (vsip_scalar_f)i + (vsip_scalar_f)3.3))
    {
      vtprintf ("Error in vsip_cblockrelease_f(); (5)\n"
                "Released updated split block element %i of real array "
                "was not updated (%f).\n", i, array1[i]);
      ret |= 1<<5;
      break;
    }
  }

  for (i=0; i<block_size; i++)
  {
    if (vtfcomp (array2[i], (vsip_scalar_f)i + (vsip_scalar_f)2.2))
    {
      vtprintf ("Error in vsip_cblockrelease_f(); (6)\n"
                "Released updated split block element %i of imaginary array "
                "was not updated (%f).\n", i, array2[i]);
      ret |= 1<<6;
      break;
    }
  }

  for (i=0; i<block_size*2; i++)
  {
    if (vtfcomp (array3[i], i))
    {
      vtprintf ("Error in vsip_cblockrelease_f(); (7)\n"
                "Released updated interleaved block element %i of array"
                "was not updated (%f).\n", i, array3[i]);
      ret |= 1<<7;
      break;
    }
  }
  
  vsip_cvdestroy_f (ciview);
  vsip_cvdestroy_f (csview);
  vsip_cblockdestroy_f (iblock);
  vsip_cblockdestroy_f (sblock);

  iblock = vsip_cblockcreate_f  (block_size, 0);
  vsip_cblockrelease_f (iblock, update, &prv_data1, &prv_data2);

  if (prv_data1 || prv_data2)
  {
      vtprintf ("Error in vsip_cblockrelease_f(); (8)\n"
                "Released a VSIP block.\n");
      
    ret |= 1<<8;
  }

  vsip_cblockdestroy_f (iblock);
  report (ret);
  vsip_finalize ((void *)0);
  return (ret!=0);
}
