/* vsiptest_init.c Version 0.000 March 22, 1999
//
// This file contains the vsip_init and vsip_finalize 
// test functionality.
//
// Functional Description:
//   vsip_init and vsip_finalize must surround all vsip_ prefixed 
//   operations.  vsip_init and _finalize are NOT required to enforce
//   this rule in performance mode, but a vsip implementation is not
//   permitted to disallow vsip_ functionality wihin an init/finalize 
//   nest.  Implementations are required to support infinite nesting of 
//   init/finalize pairs, as well as infinite repetitions of the 
//   init/finalize.
//
//
// Copyright March 1999, Georgia Tech Research Institute.
// This is an unpublished work. All rights reserved.
//
// Author: Randall Janka
//         randall.janka@gtri.gatech.edu
//
// Test description: 
//   Testing can only verify that nesting and repititions of the init/finalize
//   pair are allowable.  Thus, this test will nest an arbitrary number of 
//   init/finalize pairs, and will execute some very basic vsip_ function 
//   at each level of nesting.  The test will then repeatedly init and finalize
//   with a basic vsip function inside each pair to test that repitions are 
//   allowed.
//
// Revision History:
// March 22, 1999  Created.
*/

#include <stdio.h>
#include "vsiptest_block.h"

int main()
{
  int              i, ret = 0;

  for (i=0; i<VSIPTEST_INIT_NESTING_LEVELS; i++)
  {
    vsip_init ((void *)0);
    vsip_cstorage ();
  }
  for (i=0; i<VSIPTEST_INIT_NESTING_LEVELS; i++)
  {
    vsip_cstorage ();
    vsip_finalize ((void *)0);
  }

  for (i=0; i<VSIPTEST_INIT_REPETITIONS; i++)
  {
    vsip_init ((void *)0);
    vsip_cstorage();
    vsip_finalize ((void *)0);
  }
  return (ret);
}
