%File:						vsip_firflt_gen28.m
%
%Author:						Randy Janka
%Date:						3/28/00
%
%Purpose:					Generate test data for vsip_firflt_f
%								Generate x[n] and compute y[n]=h[n]*x[n]
%									for different combinations of parameters
%
%Parameters:				N = Length = length(x) = 648
%								symm = FULL, EVEN, ODD
%								D = 1, 2, 3
%                       state = SAVE and NO_SAVE
%                            SAVE => Save filter state for following data.
%                         NO_SAVE => Don't save filter state; last segment.
%							    DATAPATH =  dir where data file is to go;
%								             	set in workspace
%
% Test #	 f#    symm     N    D   filter    M     state 
% ======  ==    ====    ===   =   ======   ==    =======
%
%   1a    f1    NON     324   1    bIII    26     SAVE
%   1b    f2    NON     324   1    bIII    26    NO_SAVE
%   2     f3    NON     648   1    bIV     23    NO_SAVE
%	 3a  	 f4    EVEN    324   2    bI      26     SAVE
%	 3b  	 f5    EVEN    324   2    bI      26    NO_SAVE
%	 4	 	 f6    EVEN    648   2    bI      26    NO_SAVE
%	 5a 	 f7    ODD     324   3    bII     23     SAVE
%	 5b 	 f8    ODD     324   3    bII     23    NO_SAVE
%	 6	 	 f9    ODD     648   3    bII     23    N0_SAVE

%Ordinal of decimation
k=1;
%kth decimated output is kept; other D-k samples are dropped.
%I.e., keep (nD+k+1)th samples where n=0,1,...,floor(N/D).
%           (nD+k+1)th samples in Matlab-land
%           (nD+k)th samples in C-land
%I.e., for D=3 and y=y[1],y[2],y[3],y[4],y[5],y[6],y[7],....,
%             then y_out=y[1],y[4],y[7],.... 
%I.e., for D=2 and y=y[1],y[2],y[3],y[4],y[5],y[6],y[7],....,
%             then y_out=y[1],y[3],y[5],.... 
%And for D=1 and y_out=y. 

%Define constants
M_even=12; % Filter order--must be even for script
M_odd=M_even-1;

N_whole=M_even*M_odd;
N_div_2=N_whole/2;
ntimes=0; %semi-infinite
%symm definitions
VSIP_NONSYM=0;
VSIP_SYM_EVEN_LEN_ODD=1;
VSIP_SYM_EVEN_LEN_EVEN=2;
%state definitions
VSIP_STATE_NO_SAVE=1;
VSIP_STATE_SAVE=2;

%Generate filter coefficients
%   M = Filter order
% M+1 = Number of filter coefficients
%
%bI
%==
%Type I filter coefficient generation
%Even coefficient symmetry
%LPF for D>1
%M is odd; M+1 coefficients (even # coeffs)
%VSIPL_SYM_EVEN_LEN_EVEN
%
%bII
%===
%Type II filter coefficient generation
%Even coefficient symmetry
%LPF for D>1
%M is even; M+1 coefficients (odd # coeffs)
%VSIPL_SYM_EVEN_LEN_ODD
%
%bIII
%====
%Type III filter coefficient generation
%Odd coefficient symmetry
%BPF; D=1
%M is odd; M+1 coefficients (even # coeffs)
%VSIPL_NONSYM
%
%bIV
%===
%Type IV filter coefficient generation
%Odd coefficient symmetry
%BPF; D=1
%M is even; M+1 coefficients (odd # coeffs)
%VSIPL_NONSYM

filter_coeffs_gen

%Generates signal to be filtered:
%  Three tones of unit amplitude plus AWGN.
%
sig_gen
xn_1of2=xn(1:N_div_2);
xn_2of2=xn(N_div_2+1:N_whole);

%Test #1
h=bI;
M=length(h)-1;
kcoef_vec_len=M+1;
N=N_div_2;
D=1;
symm=VSIP_NONSYM;
y1=filter(h,1,xn);
%Test #1a
x=xn_1of2;
y1a=y1([k:D:N_div_2]); % Decimation by D
%Write data file
y1a_out=[y1a,zeros(1,N-length(y1a))];
h_out=[h,zeros(1,N-(length(h)))];
out=[x; y1a_out; h_out];
state=VSIP_STATE_SAVE;
fid=fopen([DATAPATH,'\firflt_f1'],'w');
fprintf(fid,'%1d %3d %1d %1d %3d %2d\n',symm,N,D,state,ntimes,kcoef_vec_len);
fprintf(fid,'%12.7e\t%12.7e\t%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('firflt_f1 write successful.')
else
   disp('firflt_f1 write NOT successful.')
end
%Test #1b
x=xn_2of2;
y1b=y1([D*floor(N_div_2/D)+k:D:N_whole]); % Decimation by D
%Write data file
y1b_out=[y1b,zeros(1,N-length(y1b))];
out=[x; y1b_out; h_out];
state=VSIP_STATE_NO_SAVE;
fid=fopen([DATAPATH,'\firflt_f2'],'w');
fprintf(fid,'%1d %3d %1d %1d %3d %2d\n',symm,N,D,state,ntimes,kcoef_vec_len);
fprintf(fid,'%12.7e\t%12.7e\t%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('firflt_f2 write successful.')
else
   disp('firflt_f2 write NOT successful.')
end

%Test #2
h=bIV;
M=length(bIV)-1;
kcoef_vec_len=M+1;
N=N_whole;
D=1;
symm=VSIP_NONSYM;
x=xn;
y2=filter(h,1,x);
y2D=y2([k:D:N]); % Decimation by D
%Write data file
y2_out=[y2D,zeros(1,N-length(y2D))];
h_out=[h,zeros(1,N-(length(h)))];
out=[x; y2_out; h_out];
state=VSIP_STATE_NO_SAVE;
fid=fopen([DATAPATH,'\firflt_f3'],'w');
fprintf(fid,'%1d %3d %1d %1d %3d %2d\n',symm,N,D,state,ntimes,kcoef_vec_len);
fprintf(fid,'%12.7e\t%12.7e\t%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('firflt_f3 write successful.')
else
   disp('firflt_f3 write NOT successful.')
end

%Test #3
h=bI;
M=length(h)-1;
kcoef_vec_len=ceil((M+1)/2);
N=N_div_2;
D=2;
symm=VSIP_SYM_EVEN_LEN_EVEN;
y3=filter(h,1,xn);
%Test #3a
x=xn_1of2;
y3a=y3([k:D:N_div_2]); % Decimation by D
%Write data file
y3a_out=[y3a,zeros(1,N-length(y3a))];
h_out=[h,zeros(1,N-(length(h)))];
out=[x; y3a_out; h_out];
state=VSIP_STATE_SAVE;
fid=fopen([DATAPATH,'\firflt_f4'],'w');
fprintf(fid,'%1d %3d %1d %1d %3d %2d\n',symm,N,D,state,ntimes,kcoef_vec_len);
fprintf(fid,'%12.7e\t%12.7e\t%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('firflt_f4 write successful.')
else
   disp('firflt_f4 write NOT successful.')
end
%Test #3b
x=xn_2of2;
y3b=y3([D*floor(N_div_2/D)+k:D:N_whole]); % Decimation by D
%Write data file
y3b_out=[y3b,zeros(1,N-length(y3b))];
out=[x; y3b_out; h_out];
state=VSIP_STATE_NO_SAVE;
fid=fopen([DATAPATH,'\firflt_f5'],'w');
fprintf(fid,'%1d %3d %1d %1d %3d %2d\n',symm,N,D,state,ntimes,kcoef_vec_len);
fprintf(fid,'%12.7e\t%12.7e\t%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('firflt_f5 write successful.')
else
   disp('firflt_f5 write NOT successful.')
end

%Test #4
h=bI;
M=length(h)-1;
kcoef_vec_len=ceil((M+1)/2);
N=N_whole;
D=2;
symm=VSIP_SYM_EVEN_LEN_EVEN;
x=xn;
y4=filter(h,1,x);
y4D=y4([k:D:N]); % Decimation by D
%Write data file
y4_out=[y4D,zeros(1,N-length(y4D))];
h_out=[h,zeros(1,N-(length(h)))];
out=[x; y4_out; h_out];
state=VSIP_STATE_NO_SAVE;
fid=fopen([DATAPATH,'\firflt_f6'],'w');
fprintf(fid,'%1d %3d %1d %1d %3d %2d\n',symm,N,D,state,ntimes,kcoef_vec_len);
fprintf(fid,'%12.7e\t%12.7e\t%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('firflt_f6 write successful.')
else
   disp('firflt_f6 write NOT successful.')
end

%Test #5
h=bII;
M=length(h)-1;
kcoef_vec_len=ceil((M+1)/2);
N=N_div_2;
D=3;
symm=VSIP_SYM_EVEN_LEN_ODD;
y5=filter(h,1,xn);
%Test #5a
x=xn_1of2;
y5a=y5([k:D:N_div_2]); % Decimation by D
%Write data file
y5a_out=[y5a,zeros(1,N-length(y5a))];
h_out=[h,zeros(1,N-(length(h)))];
out=[xn_1of2; y5a_out; h_out];
state=VSIP_STATE_SAVE;
fid=fopen([DATAPATH,'\firflt_f7'],'w');
fprintf(fid,'%1d %3d %1d %1d %3d %2d\n',symm,N,D,state,ntimes,kcoef_vec_len);
fprintf(fid,'%12.7e\t%12.7e\t%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('firflt_f7 write successful.')
else
   disp('firflt_f7 write NOT successful.')
end
%Test #5b
x=xn_2of2;
y5b=y5([D*floor(N_div_2/D)+k:D:N_whole]); % Decimation by D
%Write data file
y5b_out=[y5b,zeros(1,N-length(y5b))];
out=[xn_2of2; y5b_out; h_out];
state=VSIP_STATE_NO_SAVE;
fid=fopen([DATAPATH,'\firflt_f8'],'w');
fprintf(fid,'%1d %3d %1d %1d %3d %2d\n',symm,N,D,state,ntimes,kcoef_vec_len);
fprintf(fid,'%12.7e\t%12.7e\t%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('firflt_f8 write successful.')
else
   disp('firflt_f8 write NOT successful.')
end

%Test #6
h=bII;
M=length(h)-1;
kcoef_vec_len=ceil((M+1)/2);
N=N_whole;
D=3;
symm=VSIP_SYM_EVEN_LEN_ODD;
x=xn;
y6=filter(h,1,x);
y6D=y6([k:D:N]); % Decimation by D
%Write data file
y6_out=[y6D,zeros(1,N-length(y6D))];
h_out=[h,zeros(1,N-(length(h)))];
out=[x; y6_out; h_out];
state=VSIP_STATE_NO_SAVE;
fid=fopen([DATAPATH,'\firflt_f9'],'w');
fprintf(fid,'%1d %3d %1d %1d %3d %2d\n',symm,N,D,state,ntimes,kcoef_vec_len);
fprintf(fid,'%12.7e\t%12.7e\t%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('firflt_f9 write successful.')
else
   disp('firflt_f9 write NOT successful.')
end
