%File:						vsip_vatan_gen.m
%
%Author:						Randy Janka
%Date:						11/16/99
%Reference:					Chapter 11 of Cody/Waite
%
%Purpose:					Generate test data for vsip_vatan_f
%								Generate x and y=atan(x)
%									where x is a vector of length N
%
%Parameters:				N = Length = length(x) = 1073
%								DATAPATH = dir where data file is to go;
%									set in workspace

%x=0
%
x0=0;
y0=atan(x0);

%x>0
%
%Small arg:					atan(x)=x, |x|<<1; |x|<0.01 for single precision
x1=10.^[-7:-3];
%Primary range:			|x|<xmax; xmax=2+sqrt(3)=3.73205080756888
xmax=2+sqrt(3);
x2=[0.01:0.01:xmax];
%Knee of atan(x):			20<x<100
x3=[3.8:0.1:20];
x4=[22:1:100];
%Asymptote of atan(x):	atan(x)->pi/2 as x->inf; x>100 for single precision
x5=10.^[3:7];
xpos=[x1,x2,x3,x4,x5];
ypos=atan(xpos);

%x<0
%
%Small arg
x1neg=-10.^[-3:-1:-7];
%Primary range
x2neg=[-xmax:0.01:-0.01];
%Knee of atan(x)			
x3neg=[-10:0.1:-3.8];
%Asymptote of atan(x)
x4neg=-10.^[7:-1:2];
xneg=[x4neg,x3neg,x2neg,x1neg];
yneg=atan(xneg);

%Form x
x=[xneg,x0,xpos];
x_out=shuffle(x);

%Form y
y=[yneg,y0,ypos];
y_out=atan(x_out);

%Write data file
N=length(x);
out=[x_out; y_out];
fid=fopen([DATAPATH,'\vatan_f'],'w');
fprintf(fid,'%4d\n',N);
fprintf(fid,'%12.7e\t%12.7e\n',out);
status=fclose(fid);

if ~status
   disp('vatan_f write successful.')
else
   disp('vatan_f write NOT successful.')
end
