/* vsiptest_vrandu_f.c Version 0.000 November 8, 1999
// 
// This file contains the vsiptest_vhisto_f test functionality.
// 
// Functional Description:
//   vsip_vrandu_f does a histogram on the input vector
//  
//
// Copyright November 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Revision History:
// Nov  8 1999 - Created
*/

#include"vsiptest_sigproc.h"
#include <stdio.h>

void vector_read (char *filename, 
                  vsip_vview_f **output, 
                  vsip_length *out_length, 
                  vsip_index *seed,
                  vsip_index *procs,
                  vsip_index *id)
{
  FILE *fp;
  int read = 0;
  float val1;
  int ol, s, p, i;

  fp = fopen (filename, "r");
  if (!fp)
  {
    vtprintf ("Error in vsip_vrandu_f (0)\n"
              "unable to open data file %s for reading\n", filename);
    exit(1);
  }

  read = fscanf (fp, "%i %i %i %i\n", &ol, &s, &p, &i);
  *out_length = ol;
  *seed = s;
  *procs = p;
  *id = i;


  if (read != 4)
  {
    vtprintf ("Error in vsip_vrandu_f (0)\n"
              "unable to read random parameters in %s\n", filename);
    fclose (fp);
    exit (1);
  }
  
  *output = vsip_vcreate_f (ol, 0);

  if (!output)
  {
    vtprintf ("Error in vsip_vhisto_f (0)\n"
              "unable to create output view of size %i for %s\n", ol, filename);
    fclose (fp);
    exit(1);
  }

  for (i=0; i<ol; i++)
  {
    read = fscanf (fp, "%f\n", &val1);
    if (read < 1)
    {
      vtprintf ("Error in vsip_vrandu_f (0)\n"
                "unable to read row %i in %s\n", i, filename);
      fclose (fp);
      exit (1);
    }
    vsip_vput_f (*output, i, val1);
  }
  fclose (fp);
}

int main(int argc, char *argv[])
{
  int ret = 0;
  int numfiles = VSIPTEST_VRANDU_F_NUMFILES;
  char filename[120];
  int trial;
  vsip_vview_f *tempv, *outputv;
  int i;
  vsip_scalar_f e_real, r_real;
  vsip_length out_length;
  vsip_randstate *rs;
  vsip_scalar_f val;
  vsip_index seed, procs, id;

  int foo = VSIP_PRNG;
  foo = foo;

  vsip_init ((void *)0);

  error_init (argv[1]);

  rs = vsip_randcreate (VSIPTEST_RANDCREATE_NPSEED, 
                        VSIPTEST_RANDCREATE_NPPROCS,
                        VSIPTEST_RANDCREATE_NPID,
                        VSIP_NPRNG);
  if (!rs)
  {
    vtprintf ("Error in vsip_randcreate (1)\n"
              "Could not create non-portable random state with\n"
              "seed=%i numprocs=%i id=%i\n", VSIPTEST_RANDCREATE_NPSEED,
              VSIPTEST_RANDCREATE_NPPROCS, VSIPTEST_RANDCREATE_NPID);
   ret |= 1<<1;
  }
  else
  {
    tempv = vsip_vcreate_f (VSIPTEST_VRANDU_F_NPSIZE, 0);
    if (!tempv)
    {
      vtprintf ("Error in vsip_vrandu_f (2)\n"
                "Could not create temporary real view of size %i for NPRNG test\n",
                VSIPTEST_VRANDU_F_NPSIZE);
      ret |= 1<<2;
    }
    else
    {
      vsip_vrandu_f (rs, tempv);
      for (i=0; i<VSIPTEST_VRANDU_F_NPSIZE; i++)
      {
        val = vsip_vget_f (tempv, i);
        if (val<0 || val>1)
        {
          vtprintf ("Error in vsip_vrandu_f (3)\n"
                    "Non-portable random generator produced illegal value (%f)\n"
                    "for seed=%i numprocs=%i id=%i, at element %i\n", val,
                    VSIPTEST_RANDCREATE_NPSEED,
                    VSIPTEST_RANDCREATE_NPPROCS,
                    VSIPTEST_RANDCREATE_NPID, i);
          ret |= 1<<3;
          break;
        }
      }
      vsip_valldestroy_f (tempv);
    }
    vsip_randdestroy (rs);
  }

  for (trial=0; trial<numfiles; trial++)
  {
    sprintf (filename, "data/%s%i", VSIPTEST_VRANDU_F_BASENAME, 
             trial+1);

    vector_read (filename, &outputv, &out_length, &seed, &procs, &id);

    tempv = vsip_vcreate_f (out_length, 0);

    if (!tempv)
    {
      vtprintf ("Error in vsip_vrandu_f (4)\n"
                "Could not create result vector for size %i\n", out_length);
      ret |= 1<<4;
      continue;
    } /* if tempv */

    rs = vsip_randcreate (seed, procs, id, VSIP_PRNG);
    if (!rs)
    {
      vtprintf ("Error in vsip_randcreate (5)\n"
                "Could not create portal random state for seed=%i numprocs=%i id=%i\n",
                seed, procs, id);
      ret |= 1<<5;
      continue;
    } /* if !rs */

    vsip_vrandu_f (rs, tempv);

    for (i=0; i<out_length; i++)
    {
      r_real = vsip_vget_f (tempv, i);

      e_real = vsip_vget_f (outputv, i);

      update_error_element (fabs(r_real-e_real));

      if (fabs (r_real - e_real) > VSIPTEST_VRANDU_F_MAX_EPSILON)
      {
        vtprintf ("Error in vsip_vrandu_f (6)\n"
                  "Element %i of result vector (%f) did not match "
                  "expected value (%f)\n"
                  "for seed=%i numprocs=%i id=%i trial=%i\n", i, r_real, 
                   e_real, seed, procs, id, trial+1);
        ret |= 1<<6;
        break;
      } /* if EPSILON */
      if ((r_real < 0 && e_real >= 0) || 
          (r_real == 0 && e_real != 0) ||
          (r_real > 0 && e_real <= 0) )
      {
        vtprintf ("Error in vsip_vrandu_f (7)\n"
                  "Element %i of result vector (%f) failed sign test "
                  "against expected value (%f)\n"
                  "for seed=%i numprocs=%i id=%i trial=%i \n", i, r_real,
                  e_real, seed, procs, id, trial+1);
        ret |= 1<<7;
        break;
      } /* if sign test */
    } /*for i*/
    update_error_total();
    error_report_test ("Test #%i for seed=%i numbprocs=%i id=%i", trial+1, seed, procs, id);
    vsip_valldestroy_f (tempv);
    vsip_valldestroy_f (outputv);
    vsip_randdestroy (rs);
  } /* for trial */
  error_finish();
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
} /* main */
