/* vsiptest_io.c Version 0.000 April 6, 1999
//
// This file contains the compartmentalized IO routines for vsiptest
//
// Functional Description:
//   vtprintf replaces printf as a virtualized output routine.  This will
//            allow vsiptest authors, as well as implementors/testors 
//            easy flexibility in changing where output from the tests goes.
//
// Copyright April 1999, Georgia Tech Research Institute.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell
//         dan.campbell@gtri.gatech.edu
//
// Revision History:
// April 6, 1999 Created
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <math.h>
#include "vsiptest_util.h"
#include "vsip.h"

int _malloc_fail = 0;

void vtprintf (char *fmt, ...)
{
  va_list argp;
  FILE *fp;

  fp = fopen (OUTPUT_FILENAME, "a");
  if (!fp)
  {
      fprintf (stderr, "could not open output file %s\n", OUTPUT_FILENAME);
      exit(1);
  }

  va_start(argp, fmt);
  vfprintf (fp, fmt, argp);
  va_end(argp);

  va_start(argp, fmt);
  vfprintf (stderr, fmt, argp);
  va_end(argp);
}

int vtfcomp (float v1, float v2)
{
  if(fabs (v1 - v2) <= fabs(COMPARE_EPSILON * v1))

  {
    return 0;
  }
  else
  {
    return 1;
  }
}

void report (int value)
{
}

void vtfail_next_malloc (int fail)
{
  _malloc_fail = fail;
}

int vthas_fail_mech (void)
{
  void *result;

  vtfail_next_malloc(1);
  result = (void *)malloc(1);
  free (result);
  if (result)
  {
    return 0;
  }
  else
  {
    return 1;
  }
}

void vtfill_array (vsip_scalar_f *block, int size, float first, float slope)
{
  int i;
  for (i=0; i<size; i++)
  {
    block[i] = i * slope + first;
  }
}

int vtcheck_f_block (vsip_block_f *block, vsip_length size)
{
  vsip_vview_f *blockview;
  int i;

  blockview = vsip_vbind_f(block, 0, 1, (vsip_length)size);
  if (!blockview)
  {
    return VT_CHECK_BLOCK_COULD_NOT_BIND;
  }
  vsip_vramp_f (0.0, 1.0, blockview);

  for (i=0; i<size; i++)
  {
    if (vtfcomp (vsip_vget_f(blockview, i), i))
    {
      vsip_vdestroy_f (blockview);
      return VT_CHECK_BLOCK_DATA_MISMATCH;
    }
  }
  vsip_vdestroy_f (blockview);
  return 0;
}

int vtcheck_cf_block (vsip_cblock_f *block, vsip_length size)
{
  vsip_cvview_f *blockview;
  vsip_cscalar_f cval;
  int i;

  blockview = vsip_cvbind_f(block, 0, 1, (vsip_length)size);
  if (!blockview)
  {
    return VT_CHECK_BLOCK_COULD_NOT_BIND;
  }
  for (i=0; i<size; i++)
  {
    cval.r = (vsip_scalar_f)i;
    cval.i = (vsip_scalar_f)3 * i;
    vsip_cvput_f (blockview, i, cval);
  }

  for (i=0; i<size; i++)
  {
    cval = vsip_cvget_f(blockview, i);

    if (vtfcomp (cval.r, i    ) || 
        vtfcomp (cval.i, 3 * i) )
    {
      return VT_CHECK_BLOCK_DATA_MISMATCH;
    }
  }
  return 0;
}

unsigned int vtrand (unsigned int max)
/* returns 0->max */
{
  static unsigned int last = 12345;
  last = (last * 1103515245 + 12345) % (1<<31);
 
  return last % max;
}

void vtshuffle (int *array, int size)
{
  int i;
  unsigned int swh, tmp;

  for (i=0; i<size; i++)
  {
    array[i] = i;
  }

  for (i=size-1; i>0; i--)
  {
    swh = vtrand(i);
    tmp = array[swh];
    array[swh] = i;
    array[i] = tmp;
  }
}
