/* vsiptest_cvalldestroy_f.c Version 0.000 August 4, 1999
// 
// This file contains the vsiptest_cvalldestroy_f test functionality.
// 
// Functional Description:
//   vsip_cvalldestroy_f destroys a view and its associated block
//
// Copyright August 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//   1) For a preset number of times:
//     a) Create a complex VSIPL block
//     b) Create a split user block
//     c) Create an interleaved user block
//     d) bind a view to each block
//     e) vcreate a complex view
//     f) valldestroy on the views
//     
//   NOTE:  in performance mode, there is very little that can be checked. 
//          implementors may wish to do a memory leak check around this test.
//
// Revision History:
// August 4, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int             i;
  int             ret = 0;
  vsip_cblock_f  *block;
  vsip_cvview_f  *vview;
  int             BLOCK_SIZE = VSIPTEST_CVALLDESTROY_F_BLOCK_SIZE;
  vsip_scalar_f *array1, *array2, *array3;

  vsip_init ((void *)0);

  for (i=0; i<VSIPTEST_CVALLDESTROY_F_ITERATIONS; i++)
  {
    block = vsip_cblockcreate_f (BLOCK_SIZE, 0);
    if (!block)
    {
      vtprintf ("Error in vsip_cvalldestroy_f() (1)\n"
                "Could not create VSIP block\n");
      ret |= 1<<1;
      report (ret); 
      vsip_finalize((void *)0);
      return (ret != 0);
    }

    vview = vsip_cvbind_f (block, 0, 1, BLOCK_SIZE);
    if (!vview)
    {
      vtprintf ("Error in vsip_cvalldestoroy_f() (2)\n"
                "Could not create view to VSIP block\n");
      ret |= 1<<2;
      report (ret); 
      vsip_finalize((void *)0);
      return (ret != 0);
    }
    vsip_cvalldestroy_f (vview);

    vview = vsip_cvcreate_f (BLOCK_SIZE, 0);
    if (!vview)
    {
      vtprintf ("Error in vsip_cvalldestroy_f() (3)\n"
                "Could not create VSIP view and block\n");
      ret |= 1<<3;
      report (ret); 
      vsip_finalize((void *)0);
      return (ret != 0);
    }
    vsip_cvalldestroy_f (vview);

    array1 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*BLOCK_SIZE);
    array2 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*BLOCK_SIZE);
    array3 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*2*BLOCK_SIZE);
    if (!array1 || !array2 || !array3)
    {
      vtprintf ("Error in vsip_cvalldestroy_f() (4)\n"
                "Could not allocate user arrays\n");
      ret |= 1<<4;
      report (ret); 
      vsip_finalize((void *)0);
      return (ret != 0);
    }

    block = vsip_cblockbind_f (array1, array2, BLOCK_SIZE, 0);
    if (!block)
    {
      vtprintf ("Error in vsip_cvalldestroy_f() (5)\n"
                "Could not create split user block\n");
      ret |= 1<<5;
      report (ret); 
      vsip_finalize((void *)0);
      return (ret != 0);
    }

    vview = vsip_cvbind_f (block, 0, 1, BLOCK_SIZE);
    if (!vview)
    {
      vtprintf ("Error in vsip_cvalldestoroy_f() (6)\n"
                "Could not create view to split user block\n");
      ret |= 1<<6;
      report (ret); 
      vsip_finalize((void *)0);
      return (ret != 0);
    }
    vsip_cvalldestroy_f (vview);

    block = vsip_cblockbind_f (array3, NULL, BLOCK_SIZE, 0);
    if (!block)
    {
      vtprintf ("Error in vsip_cvalldestroy_f() (7)\n"
                "Could not create interleaved user block\n");
      ret |= 1<<7;
      report (ret); 
      vsip_finalize((void *)0);
      return (ret != 0);
    }

    vview = vsip_cvbind_f (block, 0, 1, BLOCK_SIZE);
    if (!vview)
    {
      vtprintf ("Error in vsip_cvalldestoroy_f() (8)\n"
                "Could not create view to interleaved user block\n");
      ret |= 1<<8;
      report (ret); 
      vsip_finalize((void *)0);
      return (ret != 0);
    }
    vsip_cvalldestroy_f (vview);

    free (array1);
    free (array2);
    free (array3);
  }
  report (ret); 
  vsip_finalize((void *)0);
  return (ret != 0);
}
