/* vsiptest_cvgetblock_f.c Version 0.000 August 2, 1999
// 
// This file contains the vsiptest_cvgetblock_f test functionality.
// 
// Functional Description:
//   vsip_cvgetblock_f gets a pointer to the block from which a given view is
//   based 
//
// Copyright August 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//      1) Create a split user block
//      2) Create a interleaved user block
//      3) Create a VSIPL block
//      4) bind a view to each block
//      5) getblock on each view, and ensure that it matches the original block
//
// Revision History:
// August 3, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  vsip_cblock_f *uiblock, *usblock, *vblock;
  vsip_cvview_f *vview, *uiview, *usview;
  int            ret = 0;
  vsip_scalar_f  *array1, *array2, *array3;
  int            BLOCK_SIZE = VSIPTEST_CVGETBLOCK_F_BLOCK_LENGTH;

  vsip_init ((void *)0);
  array1 = (vsip_scalar_f *)malloc(BLOCK_SIZE*sizeof(vsip_scalar_f));
  array2 = (vsip_scalar_f *)malloc(BLOCK_SIZE*sizeof(vsip_scalar_f));
  array3 = (vsip_scalar_f *)malloc(BLOCK_SIZE*2*sizeof(vsip_scalar_f));

  if (!array1 || !array2 || !array3)
  {
    vtprintf ("Error in vsip_cvgetblock_f() (1)\n"
              "Could not allocate data arrays\n");
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  usblock = vsip_cblockbind_f (array1, array2, BLOCK_SIZE, 0);
  if (!usblock)
  {
    vtprintf ("Error in vsip_cvgetblock_f() (2)\n"
              "Could not bind block to split user array\n");
    ret |= 1<<2;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  vsip_cblockadmit_f (usblock, 1);

  uiblock = vsip_cblockbind_f (array3, NULL, BLOCK_SIZE, 0);
  if (!uiblock)
  {
    vtprintf ("Error in vsip_cvgetblock_f() (3)\n"
              "Could not bind block to interleaved user array\n");
    ret |= 1<<3;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  vsip_cblockadmit_f (uiblock, 1);

  vblock = vsip_cblockcreate_f (BLOCK_SIZE, 0);
  if (!vblock)
  {
    vtprintf ("Error in vsip_cvgetblock_f() (4)\n"
              "Could not create VSIPL block\n");
    ret |= 1<<4;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  usview = vsip_cvbind_f (usblock, 0, 1, BLOCK_SIZE);
  if (!usview)
  {
    vtprintf ("Error in vsip_cvgetblock_f() (5)\n"
              "Could not bind view to split user block\n");
    ret |= 1<<5;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  uiview = vsip_cvbind_f (uiblock, 0, 1, BLOCK_SIZE);
  if (!uiview)
  {
    vtprintf ("Error in vsip_cvgetblock_f() (6)\n"
              "Could not bind view to interleaved user block\n");
    ret |= 1<<6;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  vview = vsip_cvbind_f (vblock, 0, 1, BLOCK_SIZE);
  if (!vview)
  {
    vtprintf ("Error in vsip_cvgetblock_f() (7)\n"
              "Could not create view to all of VSIPL block\n");
    ret |= 1<<7;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (vsip_cvgetblock_f (usview) != usblock)
  {
    vtprintf ("Error in vsip_cvgetblock_f() (8)\n"
              "Value returned for split user view did not match split user "
              "block\n");
    ret |= 1<<8;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (vsip_cvgetblock_f (uiview) != uiblock)
  {
    vtprintf ("Error in vsip_cvgetblock_f() (9)\n"
              "Value returned for interleaved user view did not match "
              "interleaved user block\n");
    ret |= 1<<9;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  if (vsip_cvgetblock_f (vview) != vblock)
  {
    vtprintf ("Error in vsip_cvgetblock_f() (10)\n"
              "Could not getblock for real part of user interleaved view\n");
    ret |= 1<<10;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  vsip_cvdestroy_f (vview);
  vsip_cvdestroy_f (uiview);
  vsip_cvdestroy_f (usview);
  vsip_cblockdestroy_f (vblock);
  vsip_cblockdestroy_f (usblock);
  vsip_cblockdestroy_f (uiblock);
  free (array1); 
  free (array2); 
  free (array3); 
  
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
