/* vsiptest_cvput_f.c Version 0.000 June 9, 1999
// 
// This file contains the vsiptest_cvput_f test functionality.
// 
// Functional Description:
//   vsip_cvput_f 
//
// Copyright June 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//   1)  For each of several offset, stride, length
//     a) fill VSIP created block with 0s
//     b) fill user array with 0s
//     c) bind and admit array
//     d) bind a view
//     e) fill view with ramp values
//     f) destroy view, release block
//     g) compare all values in array 
/      h) compare all values in VSIP block
//
// Revision History:
// June 9, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i,j, k;
  vsip_cblock_f *ublock, *vblock, *iblock;
  vsip_cvview_f *vall_view, *vview, *uview, *iview;
  int            ret = 0;
  vsip_scalar_f  val1, val2;
  vsip_offset    offset;
  vsip_stride    stride;
  vsip_length    length;
  vsip_scalar_f *array1, *array2, *array3;
  int            mod, plc;
  vsip_cscalar_f cval;
  vsip_scalar_f *d1, *d2;

  int            *order;

  vsip_init ((void *)0);

  order = (int *)malloc(VSIPTEST_CVPUT_F_BLOCK_SIZE*sizeof(int));

  vtshuffle (order, VSIPTEST_CVPUT_F_BLOCK_SIZE);

  array1 = (vsip_scalar_f *)
            malloc(VSIPTEST_CVPUT_F_BLOCK_SIZE*sizeof(vsip_scalar_f));
  array2 = (vsip_scalar_f *)
            malloc(VSIPTEST_CVPUT_F_BLOCK_SIZE*sizeof(vsip_scalar_f));
  array3 = (vsip_scalar_f *)
            malloc(VSIPTEST_CVPUT_F_BLOCK_SIZE*2*sizeof(vsip_scalar_f));

  if (!array1 || !array2 || !array3)
  {
   
    vtprintf ("Error in vsip_cvput_f() (1)\n"
              "Could not allocate data arrays\n");
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  for (i=0; i<elements(VSIPTEST_CVPUT_F_OFFSET); i++)
  {
    offset = VSIPTEST_CVPUT_F_OFFSET[i];
    stride = VSIPTEST_CVPUT_F_STRIDE[i];
    length = VSIPTEST_CVPUT_F_LENGTH[i];

    /* Create a VSIP block, and fill it with 0's */
    vblock = vsip_cblockcreate_f (VSIPTEST_CVPUT_F_BLOCK_SIZE, 0);
    if (!vblock)
    {
      vtprintf ("Error in vsip_cvput_f() (2)\n"
                "Could not create VSIP block for o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<2;
      break;
    }
    vall_view = vsip_cvbind_f (vblock, 0, 1, VSIPTEST_CVPUT_F_BLOCK_SIZE);
    cval = vsip_cmplx_f (0, 0);
    for (j=0; j<VSIPTEST_CVPUT_F_BLOCK_SIZE; j++)
    {
      vsip_cvput_f (vall_view, j, cval);
    }
    /*vsip_cvfill_f (cval, vall_view);*/

    /* fill the user arrays with 0's */
    for (j=0; j<VSIPTEST_CVPUT_F_BLOCK_SIZE; j++)
    {
      array1[j]     = 0;
      array2[j]     = 0;
      array3[2*j]   = 0;
      array3[2*j+1] = 0;
    }

    /* bind and admit the blocks */
    ublock = vsip_cblockbind_f (array1, array2,
                                 VSIPTEST_CVPUT_F_BLOCK_SIZE, 0);
    if (!ublock)
    {
      vtprintf ("Error in vsip_cvput_f() (3)\n"
                "Could not bind split block for o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<3;
      break;
    }
    vsip_cblockadmit_f (ublock, 1);

    iblock = vsip_cblockbind_f (array3, NULL, VSIPTEST_CVPUT_F_BLOCK_SIZE, 0);
    if (!iblock)
    {
      vtprintf ("Error in vsip_cvput_f() (4)\n"
                "Could not bind interleaved block for o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<4;
      break;
    }
    vsip_cblockadmit_f (iblock, 1);

    uview = vsip_cvbind_f (ublock, offset, stride, length);
    if (!uview)
    {
      vtprintf ("Error in vsip_cvput_f() (5)\n"
                "Could not bind view to split user block for o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<5;
      break;
    }

    iview = vsip_cvbind_f (iblock, offset, stride, length);
    if (!iview)
    {
      vtprintf ("Error in vsipc_cvput_f() (6)\n"
                "Could not bind view to split user block for o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<6;
      break;
    }

    vview = vsip_cvbind_f (vblock, offset, stride, length);
    if (!vview)
    {
      vtprintf ("Error in vsip_cvput_f() (7)\n"
                "Could not bind view to VSIP block for o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<7;
      break;
    }

    /* fill the views with ramp data */
    for (j=0; j<length; j++)
    {
      cval = vsip_cmplx_f ((vsip_scalar_f)j*2, (vsip_scalar_f)(j*2+1));
      vsip_cvput_f (vview, j, cval); 
      vsip_cvput_f (uview, j, cval);
      vsip_cvput_f (iview, j, cval);
    }

    /* destroy the views and the user block */
    vsip_cvdestroy_f (uview);
    vsip_cvdestroy_f (vview);
    vsip_cvdestroy_f (iview);

    vsip_cblockrelease_f (iblock, 1, &d1, &d2);
    vsip_cblockdestroy_f (iblock);

    vsip_cblockrelease_f (ublock, 1, &d1, &d2);
    vsip_cblockdestroy_f (ublock);

    /* check the user array */

    for (j=0; j<VSIPTEST_CVPUT_F_BLOCK_SIZE; j++)
    {
      mod = (signed int)(j - offset ) % stride;
      plc = (signed int)(j - offset ) / stride;

      if (mod == 0 && plc < length)
      {
        val1 = (vsip_scalar_f)plc*2;
        val2 = (vsip_scalar_f)plc*2+1;
      }
      else
      {
        val1 = val2 = 0;
      }

      if (vtfcomp (val1, array1[j]) || vtfcomp (val2, array2[j]))
      {
        vtprintf ("Error in vsip_cvput_f (8)\n"
                  "Split user array value did not match as expected for "
                  "el:%i o:%i s:%i l:%i\n", j, offset, stride, length);
        ret |= 1<<8;
        break;
      }
      if (vtfcomp (val1, array3[j*2]) || vtfcomp (val2, array3[j*2+1]))
      {
        vtprintf ("Error in vsip_cvput_f (9)\n"
                  "Interleaved user array value did not match as expected for "
                  "el:%i o:%i s:%i l:%i\n", j, offset, stride, length);
        ret |= 1<<9;
        break;
      }

      if ( vtfcomp (val1, vsip_real_f(vsip_cvget_f(vall_view, j))) || 
           vtfcomp (val2, vsip_imag_f(vsip_cvget_f(vall_view, j)))   )
      {
        vtprintf ("Error in vsip_cvput_f (10)\n"
                  "VSIP block value did not match as expected for "
                  "el:%i o:%i s:%i l:%i\n", j, offset, stride, length);
        ret |= 1<<10;
        break;
      }
    }

    for (k=0; k<VSIPTEST_CVPUT_F_BLOCK_SIZE; k++)
    {
      j = order[k];
      mod = (signed int)(j - offset ) % stride;
      plc = (signed int)(j - offset ) / stride;

      if (mod == 0 && plc < length)
      {
        val1 = (vsip_scalar_f)plc*2;
        val2 = (vsip_scalar_f)plc*2+1;
      }
      else
      {
        val1 = val2 = 0;
      }

      if (vtfcomp (val1, array1[j]) || vtfcomp (val2, array2[j]) )
      {
        vtprintf ("Error in vsip_cvput_f (11)\n"
                  "Split user array value did not match as expected for random"
                  "el:%i o:%i s:%i l:%i\n", j, offset, stride, length);
        ret |= 1<<11;
        break;
      }
      if (vtfcomp (val1, array3[j*2]) || vtfcomp (val2, array3[j*2+1]) )
      {
        vtprintf ("Error in vsip_cvput_f (12)\n"
                  "Interleaved user array value did not match as expected for "
                  "random el:%i o:%i s:%i l:%i\n", j, offset, stride, length);
        ret |= 1<<12;
        break;
      }
      if (vtfcomp (val1, vsip_real_f (vsip_cvget_f(vall_view, j))) || 
          vtfcomp (val2, vsip_imag_f (vsip_cvget_f(vall_view, j)))  )
      {
        vtprintf ("Error in vsip_cvput_f (13)\n"
                  "VSIP block value did not match as expected for random "
                  "el:%i o:%i s:%i l:%i\n", j, offset, stride, length);
        ret |= 1<<13;
        break;
      }
    }

    vsip_cvdestroy_f (vall_view);
    vsip_cblockdestroy_f (vblock);
  }
  free (array1);
  free (array2);
  free (array3);
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
