/* vsiptest_vbind_d.c Version 0.000 May 12, 1997
// 
// This file contains the vsiptest_vbind_d test functionality.
// 
// Functional Description:
//   vsip_vbind_d binds a vector view to a block object
//  
//
// Copyright May 1997, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: This test suite will create real blocks of data with
//
// Revision History:
// May 21, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i,j;
  vsip_block_f  *block;
  vsip_block_f  *vsipl_block = NULL;
  vsip_vview_f  *view;
  int            have_malloc_failer = vthas_fail_mech();
  int            BLOCK_SIZE = VSIPTEST_VBIND_F_BLOCK_SIZE;
  int            ret = 0;
  vsip_scalar_f *array;
  vsip_vattr_f   attrib;
  vsip_offset    offset;
  vsip_stride    stride;
  vsip_length    length;

  vsip_init ((void *)0);

  if (have_malloc_failer)
  {
      block = vsip_blockcreate_f (10, 0);
      vtfail_next_malloc(1);
      view = vsip_vbind_f (block, 0, 1, 1);
      if (view != NULL)
      {
          vtprintf ("Error in vsip_vbind_f() (1)\n"
                    "Failed bind did not return NULL\n");
          ret |= 1<<1;
      }
  }

  array = (vsip_scalar_f *)malloc(BLOCK_SIZE*sizeof(vsip_scalar_f));
  if (array)
  {
    offset = 0; 
    stride = 1;
    length = BLOCK_SIZE;

    vtfill_array (array, BLOCK_SIZE, 0, 1);
    block = vsip_blockbind_f (array, BLOCK_SIZE, 0);
    vsip_blockadmit_f (block, 1);
    view = vsip_vbind_f (block, 0, 1, BLOCK_SIZE);
    if (!view)
    {
      vtprintf ("Error in vsip_vbind_f() (2)\n"
                "Failed to bind view at offset 0, stride 1, length %i\n",
                BLOCK_SIZE);
      ret |= 2;
    }
    else
    {
      for (i=0; i<BLOCK_SIZE; i++)
      {
        if (vtfcomp (vsip_vget_f(view, i), (vsip_scalar_f)i))
        {
          vtprintf ("Error in vsip_vbind_f (3)\n"
                    "Element %i does not match data array for offset 0"
                    " stride 1, length %i\n", i, BLOCK_SIZE);
          ret |= 3;
          break;
        }
      }
    }
    vsip_vgetattrib_f (view, &attrib);
    if (attrib.offset != offset ||
        attrib.stride != stride || 
        attrib.length != length)
    {
      vtprintf ("Error in vsip_vbind_f (4)\n"
                "View of offset %i stride %i legnth %i did not match attributes\n",
                offset, stride, length);
      ret |= 4;
    }

    vsip_vdestroy_f (view);
    vsip_blockdestroy_f (block);

    vtfill_array (array, BLOCK_SIZE, 0, 3);

    offset = BLOCK_SIZE/2 - 1;
    stride = 1;
    length = BLOCK_SIZE - BLOCK_SIZE/2 + 1;

    block = vsip_blockbind_f (array, BLOCK_SIZE, 0);
    vsip_blockadmit_f (block, 1);
    view = vsip_vbind_f (block, offset, stride, length);
    if (!view)
    {
      vtprintf ("Error in vsip_vbind_f() (5)\n"
                "Failed to bind view at offset %i, stride 1, length %i\n",
                offset, length);
      ret |= 5;
    }
    else
    {
      for (i=0; i<length; i++)
      {
        if (vtfcomp (vsip_vget_f(view, i), (i*stride+offset)*3))
        {
          vtprintf ("Error in vsip_vbind_f (6)\n"
                    "Element %i does not match data array for offset %i"
                    " stride 1, length %i\n", i, offset, stride);
          ret |= 6;
          break;
        }
      }
    }
    vsip_vgetattrib_f (view, &attrib);
    if (attrib.offset != offset ||
        attrib.stride != stride || 
        attrib.length != length)
    {
      vtprintf ("Error in vsip_vbind_f (7)\n"
                "View of offset %i stride %i legnth %i did not match attributes\n",
                offset, stride, length);
      ret |= 7;
    }
    vsip_vdestroy_f (view);
    vsip_blockdestroy_f (block);

    vtfill_array (array, BLOCK_SIZE, 0, 5);

    offset = BLOCK_SIZE - 1;
    stride = 1;
    length = BLOCK_SIZE - offset;

    block = vsip_blockbind_f (array, BLOCK_SIZE, 0);
    view = vsip_vbind_f (block, offset, stride, length);
    if (!view)
    {
      vtprintf ("Error in vsip_vbind_f() (8)\n"
                "Failed to bind view at offset %i, stride 1, length %i\n",
                offset, length);
      ret |= 8;
    }
    else
    {
      for (i=0; i<length; i++)
      {
        if (vtfcomp (vsip_vget_f(view, i), (i*stride+offset)*5))
        {
          vtprintf ("Error in vsip_vbind_f (9)\n"
                    "Element %i does not match data array for offset %i"
                    " stride 1, length %i\n", i, offset, stride);
          ret |= 9;
          break;
        }
      }
    }
    vsip_vgetattrib_f (view, &attrib);
    if (attrib.offset != offset ||
        attrib.stride != stride || 
        attrib.length != length)
    {
      vtprintf ("Error in vsip_vbind_f (9)\n"
                "View of offset %i stride %i legnth %i did not match attributes\n",
                offset, stride, length);
      ret |= 9;
    }
    vsip_vdestroy_f (view);
    vsip_blockdestroy_f (block);

        
    block = vsip_blockcreate_f (BLOCK_SIZE, 0);
    vsipl_block = block;
    view = vsip_vbind_f (block, 0, 1, BLOCK_SIZE);
    {
      vsip_vramp_f (0.0, 1.0, view);
    }
    vsip_vdestroy_f (view);

    offset = BLOCK_SIZE/2 - 1;
    stride = 1;
    length = BLOCK_SIZE - BLOCK_SIZE/2 + 1;

    view = vsip_vbind_f (block, offset, stride, length);
    if (!view)
    {
      vtprintf ("Error in vsip_vbind_f() (10)\n"
                "Failed to bind view at offset %i, stride 1, length %i for VSIP "
                "created block\n", offset, length);
      ret |= 10;
    }
    else
    {
      for (i=1; i<length; i++)
      {
        if (vtfcomp (vsip_vget_f(view, i), (i*stride+offset)))
        {
          vtprintf ("Error in vsip_vbind_f (11)\n"
                    "Element %i does not match data array for offset %i "
                    "stride 1, length %i (%f)in VSIP created block\n", 
                    i, offset, stride, vsip_vget_f (view, i));
          ret |= 11;
          break;
        }
      }
    }
    vsip_vgetattrib_f (view, &attrib);
    if (attrib.offset != offset ||
        attrib.stride != stride || 
        attrib.length != length)
    {
      vtprintf ("Error in vsip_vbind_f (12)\n"
                "View of offset %i stride %i legnth %i did not match attributes\n",
                offset, stride, length);
      ret |= 12;
    }
    vsip_vdestroy_f (view);

    offset = BLOCK_SIZE - 1;
    stride = 1;
    length = BLOCK_SIZE - offset;

    view = vsip_vbind_f (block, offset, stride, length);
    if (!view)
    {
      vtprintf ("Error in vsip_vbind_f() (13)\n"
                "Failed to bind view at offset %i, stride 1, length %i\n",
                offset, length);
      ret |= 13;
    }
    else
    {
      for (i=0; i<length; i++)
      {
        if (vtfcomp (vsip_vget_f(view, i), (i*stride+offset)))
        {
          vtprintf ("Error in vsip_vbind_f (14)\n"
                    "Element %i does not match data array for offset %i"
                    " stride 1, length %i in VSIP created block\n", 
                    i, offset, stride);
          ret |= 14;
          break;
        }
      }
    }
    vsip_vgetattrib_f (view, &attrib);
    if (attrib.offset != offset ||
        attrib.stride != stride || 
        attrib.length != length)
    {
      vtprintf ("Error in vsip_vbind_f (15)\n"
                "View of offset %i stride %i length %i did not match attributes\n",
                offset, stride, length);
      ret |= 15;
    }

    vsip_vdestroy_f (view);

    vtfill_array (array, BLOCK_SIZE, 0, 1);

    for (i=0; i<elements(VSIPTEST_VBIND_F_OFFSET); i++)
    {
      offset = VSIPTEST_VBIND_F_OFFSET[i];
      stride = VSIPTEST_VBIND_F_STRIDE[i];
      length = VSIPTEST_VBIND_F_LENGTH[i];

      block = vsipl_block;
      view = vsip_vbind_f (block, offset, stride, length);
      if (!view)
      {
        vtprintf ("Error in vsip_vbind_f (16)\n"
                  "View of offset %i stride %i length %i could not be bound"
                  " to VSIP created block\n", offset, stride, length);
        ret |= 16;
      }
       
      for (j=0; j<length; j++)
      {
        if (vtfcomp (vsip_vget_f (view, j), (offset + stride * j)))
        {
          vtprintf ("Error in vsip_vbind_f (17)\n"
                    "Element %i of view of offset %i stride %i length %i did "
                    "not match data in VSIP created block\n",
                    j, offset, stride, length);
          ret |= 17;
          break;
        }
      }

      vsip_vgetattrib_f (view, &attrib);
      if (attrib.offset != offset ||
          attrib.stride != stride || 
          attrib.length != length)
      {
        vtprintf ("Error in vsip_vbind_f (18)\n"
                  "View of offset %i stride %i length %i did not match attributes\n",
                  offset, stride, length);
        ret |= 18;
      }

      vsip_vdestroy_f (view);

      block = vsip_blockbind_f (array, BLOCK_SIZE, 0);
      view = vsip_vbind_f (block, offset, stride, length);
      if (!view)
      {
        vtprintf ("Error in vsip_vbind_f (19)\n"
                  "View of offset %i stride %i length %i could not be bound\n",
                  offset, stride, length);
        ret |= 19;
      }
       
      for (j=0; j<length; j++)
      {
        if (vtfcomp (vsip_vget_f (view, j), (offset + stride * j)))
        {
          vtprintf ("Error in vsip_vbind_f (20)\n"
                    "Element %i of view of offset %i stride %i length %i did "
                    "not match data\n", j, offset, stride, length);
          ret |= 20;
        }
      }

      vsip_vgetattrib_f (view, &attrib);
      if (attrib.offset != offset ||
          attrib.stride != stride || 
          attrib.length != length)
      {
        vtprintf ("Error in vsip_vbind_f (21)\n"
                  "View of offset %i stride %i length %i did not match attributes\n",
                  offset, stride, length);
        ret |= 21;
      }

      vsip_vdestroy_f (view);
      vsip_blockdestroy_f (block);
    }
  }
  vsip_blockdestroy_f (vsipl_block);
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
