/* vsiptest_vcloneview_f.c Version 0.000 May 25, 1999
// 
// This file contains the vsiptest_vcloneview_f test functionality.
// 
// Functional Description:
//   vsip_vcloneview_f clones a view
//
// Copyright May 199i, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//   1) Test that a memory allocation error causes cloneview to return NULL; 
//   2) Create a large block
//   3) for each of several view strides, offsets, and lengths:
//      a) create a view
//      b) fill the view with data
//      c) clone the view
//      d) verify that the new view is not the same as the old view
//      e) delete the original view
//      f) verify that the data in the cloned view is correct
//      g) delete the cloned view
//
// Revision History:
// May 25, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i,j;
  vsip_block_f  *block;
  vsip_vview_f  *view, *view2;
  int            have_malloc_failer = vthas_fail_mech();
  int            ret = 0;
  vsip_scalar_f  val;
  vsip_offset    offset;
  vsip_stride    stride;
  vsip_length    length;

  vsip_init ((void *)0);

  if (have_malloc_failer)
  {
    view = vsip_vcreate_f (1, 0);
    vtfail_next_malloc(1);
    view2 = vsip_vcloneview_f (view);
    if (view2 != NULL)
    {
        vtprintf ("Error in vsip_vcloneview_f() (1)\n"
                  "Failed clone did not return NULL\n");
        ret |= 1<<1;
    }
  }

  block = vsip_blockcreate_f (VSIPTEST_VCLONEVIEW_F_BLOCK_SIZE, 0);
  for (i=0; i<elements(VSIPTEST_VCLONEVIEW_F_LENGTH); i++)
  {
    offset = VSIPTEST_VCLONEVIEW_F_OFFSET[i];
    stride = VSIPTEST_VCLONEVIEW_F_STRIDE[i];
    length = VSIPTEST_VCLONEVIEW_F_LENGTH[i];

    view = vsip_vbind_f (block, offset, stride, length);

    if (view == NULL || block == NULL)
    {
      vtprintf ("Error in vsip_vcloneview_f() (2)\n"
                "Failed to create %i element block and view\n", length);
      ret |= 1<<2;
    }
    vsip_vramp_f ((vsip_scalar_f)i, (vsip_scalar_f)i, view);

    view2 = vsip_vcloneview_f (view);
    if (view == view2)
    {
      vtprintf ("Error in vsip_vcloneview_f() (3)\n"
                "Cloned view is the same as the original view for "
                "o:%i s:%i l:%i\n", offset, stride, length);
      ret |= 1<<3;
    }
    vsip_vdestroy_f (view);
    for (j=0; j<length; j++)
    {
      val = vsip_vget_f (view2, j);
      if ( vtfcomp (val, j * i + i) )
      {
        vtprintf ("Erorr in vsip_vcloneview_f() (4)\n"
                  "Cloned view data element %i did not match original view "
                  "for o:%i s:%i l:%i(%f)\n", j, offset, stride, length, val);
        ret |= 1<<4;
        break;
      }
    }
    vsip_vdestroy_f (view2);
  }
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
