/* vsiptest_vcopy_f_f.c Version 0.000 July 28, 1999
// 
// This file contains the vsiptest_vcopy_f_f test functionality.
// 
// Functional Description:
//     vsip_vcopy_f_f retruns a complex scalar formed by two real scalars.
//  
//
// Copyright July 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description:  This test will, for a specified set of block lengths
//                    create a base block, create a sub-block to be the copy
//                    target, create a source, block, fill the base block
//                    with 0, fill the source block with a ramp, and copy the
//                    source to the target.  It will then check the data in the
//                    base block to verify correct values
//                   
//
// Revision History:
// July 26, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i,j, n;
  int            ret = 0;
  vsip_scalar_f  val, got;
  vsip_vview_f  *supervector, *source, *target;

  vsip_init ((void *)0);

  for (i=0; i<elements(VSIPTEST_VCOPY_F_F_BLOCK_SIZE_BASE); i++)
  {
    n = VSIPTEST_VCOPY_F_F_BLOCK_SIZE_BASE[i];
    supervector = vsip_vcreate_f (n*2, 0);
    if (!supervector)
    {
      vtprintf ("Error in vsip_vcopy_f_f() (1)\n"
                "Could not create supervector for target size %i\n", n*2);
      ret |= 1<<1;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    vsip_vfill_f (0, supervector);
    target = vsip_vsubview_f (supervector, n/2, n);
    source = vsip_vcreate_f (n,0);
    if (!target)
    {
      vtprintf ("Error in vsip_vcopy_f_f() (2)\n"
                "Could not create target subvector of size %i\n", n);
      ret |= 1<<2;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    if (!source)
    {
      vtprintf ("Error in vsip_vcopy_f_f() (3)\n"
                "Could not create source vector of size %i\n", n);
      ret |= 1<<3;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    vsip_vramp_f (1.0, 1.0, source);
    vsip_vcopy_f_f (source, target);

    /* check the supertarget */
    for (j=0; j<n*2; j++)
    {
      if (j<n/2 || j>=n/2+n)
      {
        val = 0;
      }
      else
      {
        val = j - (n/2) + 1;
      }
      
      got = vsip_vget_f(supervector, j);
      if (vtfcomp (val, got))
      {
        vtprintf ("Error in vsip_vcopy_f_f() (4)\n"
                  "Value found in target (%f) did not match expected (%f)"
                  " at index %i\n", got, val, j);
        ret |= 1<<4;
        break;
      }
    }
  }
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
