/* vsiptest_vdestroy_i.c Version 0.000 May 21, 1999
//
// This file contains the vsiptest_vdestroy_i test functionality.
//
// Functional Description:
//   vsip_vdestroy_i destroys a view to a vector object
//
//
// Copyright May 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Randy Janka
//         randy.janka@gtri.gatech.edu
//
// Test description: This test suite will create user blocks of data
//      (including null blocks), then bind a block object and vbind a view object.
//      The vector view object will then be destroyed using the vsip_vdestroy_i
//      function.  The returned block pointer should be the same as the original
//      block pointer.
//
//      Check some of the parameters of this function
//        1)
//        2)
//        3)
//        4)
//        5)
//        6)
//
// VSIP functions called: vsip_blockbind_i, vsip_blockadmit_i, vsip_blockrelease_i,
//                        vsip_vbind_i, vsip_vdestroy_i
//
// Revision History:
//
// */

#include "vsiptest_view.h"

int main ()
{
  size_t             block_size=0, i, j;
  vsip_scalar_i     *public_block;
  vsip_memory_hint   hint = 0;
  vsip_block_i      *bind_block, *return_bind_block;
  vsip_vview_i      *bind_view;
  vsip_scalar_bl    update = 0;

  int ret = 0;

  public_block = NULL;

  /* API Checks */
  bind_block = vsip_blockbind_i (public_block, (vsip_length)(1), hint);
  if (bind_block == NULL)
  {
    vtprintf("Error in vsip_vdestroy_i(); (1)\n"
             "block bind to NULL pointer returns failure.\n");
    ret |= 1<<1;
  }
  bind_view = vsip_vbind_i(bind_block, 0, 1, (vsip_length)(1));
  if (bind_view == NULL)
  {
    vtprintf("Error in vsip_vdestroy_i(); (2)\n"
             "view bind to NULL block returns failure.\n");
    ret |= 1<<2;
  }

  /* Check various public block sizes from 2 to block_length_max */
  for (j=0; j<elements (VSIPTEST_VDESTROY_I_SIZE_ARRAY); j++)
  {
    block_size = VSIPTEST_VDESTROY_I_SIZE_ARRAY[j];
    public_block = (vsip_scalar_i *)malloc(block_size*sizeof(vsip_scalar_i));

    /* Fill the block with data values equal to its index value */
    for(i=0; i<block_size; i++)
    {
      *(public_block + i) = (vsip_scalar_i)i;
    }

    /********** VSIP_BLOCKBIND_I VSIP_BLOCKADMIT_I ************
      Bind to the public block & admit it for vector binding. */

    bind_block = vsip_blockbind_i(public_block, block_size, hint);
    if (bind_block == NULL)
    {
      vtprintf("Error in vsip_vdestroy_i(); (3)\n"
               "block bind to %d-el size block returns failure.\n", block_size);
      ret |= 1<<3;
      break;
    }
    if (vsip_blockadmit_i(bind_block, update))
    {
      vtprintf ("Error in vsip_vdestroy_i(); (4)\n"
               "Failed admit of %d-el block.\n",block_size);
      ret |= 1<<4;
      break;
    }

    /****************************************************************/

    /********************* VSIP_VBIND_I *****************************
      Create a view on the bound block */

    bind_view = vsip_vbind_i(bind_block, 0, 1, (vsip_length)block_size);
    if (bind_view == NULL)
    {
      vtprintf ("Error in vsip_vdestroy_i(); (5)\n"
               "Failed vector bind to %d-el block.\n",block_size);
      ret |= 1<<5;
      break;
    }

    /********************* VSIP_VDESTROY_I ***************************
      Destroy the view on the block and check the block pointer. */
    return_bind_block = vsip_vdestroy_i(bind_view);
    if (return_bind_block == NULL)
    {
      vtprintf("Error in vsip_vdestroy_i(); (6) \n"
               "vector destroy of valid vector returns failure.\n");
      ret |= 1<<6;
      break;
    }
    if (return_bind_block != bind_block)
    {
      vtprintf("Error in vsip_vdestroy_i(); (7) \n"
               "block pointer has changed.\n");
      ret |= 1<<7;
      break;
    }

    /********************* VSIP_BLOCKRELEASE_I *******************
      Release the block free the memory. */
    vsip_blockdestroy_i(bind_block);
    free(public_block);
  }
  report (ret); ;
  return (ret != 0);
}

