/* vsiptest_vgetattrib_f.c Version 0.000 August 3, 1999
// 
// This file contains the vsiptest_vgetattrib_f test functionality.
// 
// Functional Description:
//   vsip_vgetattrib_f gets the attributes for a float view
//
// Copyright June 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//   1) Create a user array filled with ramp values
//   2) Bind a block to this array, and admit it
//   3) Create a VSIPL block, fill it with ramp values
//   4) For each of several offset, stride, length
//      a) bind a view to each block
//      b) getattribs on each block
//      d) check the created view to make sure the values match up as 
//         expected for this SOL
//      e) destroy the views
//
// Revision History:
// August 3, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i;
  vsip_block_f  *block, *vblock;
  vsip_vview_f  *vview, *uview;
  int            ret = 0;
  vsip_offset    offset;
  vsip_stride    stride;
  vsip_length    length;
  vsip_scalar_f  *array;
  vsip_vattr_f   attrib, attrib2;

  vsip_init ((void *)0);
  array = (vsip_scalar_f *)
           malloc(VSIPTEST_VGETATTRIB_F_BLOCK_LENGTH*sizeof(vsip_scalar_f));

  if (!array)
  {
    vtprintf ("Error in vsip_vgetattrib_f() (1)\n"
              "Could not allocate data arrays\n");
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  for (i=0; i<VSIPTEST_VGETATTRIB_F_BLOCK_LENGTH; i++)
  {
    array[i]     = (vsip_scalar_f)    i;
  }

  block = vsip_blockbind_f (array, VSIPTEST_VGETATTRIB_F_BLOCK_LENGTH, 0);
  if (!block)
  {
    vtprintf ("Error in vsip_vgetattrib_f() (2)\n"
              "Could not bind block to user array\n");
    ret |= 1<<2;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  vsip_blockadmit_f (block, 1);

  vblock = vsip_blockcreate_f (VSIPTEST_VGETATTRIB_F_BLOCK_LENGTH, 0);
  if (!vblock)
  {
    vtprintf ("Error in vsip_vgetattrib_f() (3)\n"
              "Could not create VSIPL block\n");
    ret |= 1<<3;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  for (i=0; i<elements(VSIPTEST_VGETATTRIB_F_OFFSET); i++)
  {
    offset = VSIPTEST_VGETATTRIB_F_OFFSET[i];
    stride = VSIPTEST_VGETATTRIB_F_STRIDE[i];
    length = VSIPTEST_VGETATTRIB_F_LENGTH[i];

    /* create a view to each block */
    vview = vsip_vbind_f (vblock, offset, stride, length);
    uview = vsip_vbind_f (block, offset, stride, length);

    if (!vview)
    {
      vtprintf ("Error in vsip_vgetattrib_f (5)\n"
                "Could not create view to VSIPL block o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<5;
    }

    if (!uview)
    {
      vtprintf ("Error in vsip_vgetattrib_f (6)\n"
                "Could not create view to interleaved user block "
                "o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<6;
    }

    attrib.stride = 0;
    attrib.offset = 0;
    attrib.length = 0;

    vsip_vgetattrib_f (uview, &attrib2);
    if (attrib2.offset != offset || 
        attrib2.stride != stride || 
        attrib2.length != length)
    {
      vtprintf ("Error in vsip_vgetattrib_f (7)\n"
                "Actual attributes (%i, %i, %i) did not match set attributes "
                "(%i, %i, %i) for user block \n", attrib2.offset, attrib2.stride,
                attrib2.length, offset, stride, length);
     ret |= 1<<7;
    }

    vsip_vgetattrib_f (vview, &attrib2);
    if (attrib2.offset != offset || 
        attrib2.stride != stride || 
        attrib2.length != length)
    {
      vtprintf ("Error in vsip_vgetattrib_f (8)\n"
                "Actual attributes (%i, %i, %i) did not match set attributes "
                "(%i, %i, %i) for VSIPL block \n", attrib2.offset, attrib2.stride,
                attrib2.length, offset, stride, length);
     ret |= 1<<8;
    }
    /* check each datum */

    vsip_vdestroy_f (uview);
    vsip_vdestroy_f (vview);
  }

  vsip_blockdestroy_f (vblock);
  vsip_blockdestroy_f (block);
  free (array);
  
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
