/* vsiptest_cvput_f.c Version 0.000 June 9, 1999
// 
// This file contains the vsiptest_cvput_f test functionality.
// 
// Functional Description:
//   vsip_cvput_f 
//
// Copyright June 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//   1)  For each of several offset, stride, length
//     a) fill VSIP created block with 0s
//     b) fill 3 user arrays with 0s
//     c) bind and admit arrays to split and interleaved complex blocks
//     d) bind a view to each block
//     e) fill view with ramp values
//     f) destroy view, release blocks
//     g) compare all values in arrays 
/      h) compare all values in VSIP block
//
// Revision History:
// June 9, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i,j, k;
  vsip_cblock_f *vblock, *iblock, *sblock;
  vsip_cvview_f *vall_view, *vview, *iview, *sview;
  int            ret = 0;
  vsip_scalar_f  ival, rval ;
  vsip_cscalar_f cval;
  vsip_offset    offset;
  vsip_stride    stride;
  vsip_length    length;
  vsip_scalar_f *array1, *array2, *array3;
  int            mod, plc;
  vsip_scalar_f *dummy1, *dummy2;

  int            *order;

  vsip_init ((void *)0);
  
  order = (int *)malloc(VSIPTEST_CVPUT_F_BLOCK_SIZE*sizeof(int));

  vtshuffle (order, VSIPTEST_CVPUT_F_BLOCK_SIZE);

  array1 = (vsip_scalar_f *)
           malloc(VSIPTEST_CVPUT_F_BLOCK_SIZE*sizeof(vsip_scalar_f));
  array2 = (vsip_scalar_f *)
           malloc(VSIPTEST_CVPUT_F_BLOCK_SIZE*sizeof(vsip_scalar_f));
  array3 = (vsip_scalar_f *)
           malloc(VSIPTEST_CVPUT_F_BLOCK_SIZE*sizeof(vsip_scalar_f) * 2);

  if (!array1 || !array2 || !array3)
  {
   
    vtprintf ("Error in vsip_cvput_f() (1)\n"
              "Could not allocate data arrays\n");
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  for (i=0; i<elements(VSIPTEST_CVPUT_F_OFFSET); i++)
  {
    offset = VSIPTEST_CVPUT_F_OFFSET[i];
    stride = VSIPTEST_CVPUT_F_STRIDE[i];
    length = VSIPTEST_CVPUT_F_LENGTH[i];

    /* Create a VSIP block, and fill it with 0's */
    vblock = vsip_cblockcreate_f (VSIPTEST_CVPUT_F_BLOCK_SIZE, 0);
    if (!vblock)
    {
      vtprintf ("Error in vsip_cvput_f() (2)\n"
                "Could not create VSIP block for o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<2;
      break;
    }
    vall_view = vsip_cvbind_f (vblock, 0, 1, VSIPTEST_CVPUT_F_BLOCK_SIZE);

    /* fill the user array with 0's */
    for (j=0; j<VSIPTEST_CVPUT_F_BLOCK_SIZE; j++)
    {
      cval.i = 0;
      cval.r = 0;
      vsip_cvput_f (vall_view, j, cval);
      array1[j]     = 0;
      array2[j]     = 0;
      array3[j*2]   = 0;
      array3[j*2+1] = 0;
    }

    /* bind and admit the block */
    sblock = vsip_cblockbind_f (array1, array2, VSIPTEST_CVPUT_F_BLOCK_SIZE, 0);
    iblock = vsip_cblockbind_f (array3, array2, VSIPTEST_CVPUT_F_BLOCK_SIZE, 0);
    if (!iblock)
    {
      vtprintf ("Error in vsip_cvput_f() (3)\n"
                "Could not bind interleaved block for o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<3;
      break;
    }

    if (!sblock)
    {
      vtprintf ("Error in vsip_cvput_f() (4)\n"
                "Could not bind interleaved block for o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<4;
      break;
    }

    vsip_cblockadmit_f (iblock, 1);
    vsip_cblockadmit_f (sblock, 1);

    iview = vsip_cvbind_f (iblock, offset, stride, length);
    sview = vsip_cvbind_f (sblock, offset, stride, length);
    if (!iview)
    {
      vtprintf ("Error in vsip_cvput_f() (5)\n"
                "Could not bind view to user interleaved block for o:%i s:%i "
                "l:%i\n",
                offset, stride, length);
      ret |= 1<<5;
      break;
    }

    if (!sview)
    {
      vtprintf ("Error in vsip_cvput_f() (6)\n"
                "Could not bind view to user split block for o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<6;
      break;
    }

    vview = vsip_cvbind_f (vblock, offset, stride, length);
    if (!vview)
    {
      vtprintf ("Error in vsip_cvput_f() (7)\n"
                "Could not bind view to VSIP block for o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<7;
      break;
    }

    /* fill the views with ramp data */
    for (j=0; j<length; j++)
    {
      cval.r = j;
      cval.i = j * 3;

      vsip_cvput_f (iview, j, cval);
      vsip_cvput_f (sview, j, cval);
      vsip_cvput_f (vview, j, cval);
    }

    /* destroy the views and the user block */
    vsip_cvdestroy_f (iview);
    vsip_cvdestroy_f (sview);
    vsip_cvdestroy_f (vview);

    vsip_cblockrelease_f (iblock, 1, &dummy1, &dummy2);
    vsip_cblockrelease_f (sblock, 1, &dummy1, &dummy2);
    vsip_cblockdestroy_f (iblock);
    vsip_cblockdestroy_f (sblock);

    /* check the user array */

    for (j=0; j<VSIPTEST_CVPUT_F_BLOCK_SIZE; j++)
    {
      mod = (signed int)(j - offset ) % stride;
      plc = (signed int)(j - offset ) / stride;

      if (mod == 0 && plc < length)
      {
        rval = (vsip_scalar_f)plc;
        ival = rval * 3;
      }
      else
      {
        rval = 0;
        ival = 0;
      }

      if (vtfcomp (rval, array1[j]))
      {
        vtprintf ("Error in vsip_cvput_f (8)\n"
                  "User array for split block real value did not match "
                  "for el:%i o:%i s:%i l:%i\nexpected:%f got:%f\n", j,
                  offset, stride, length, rval, array1[j]);
        ret |= 1<<8;
        break;
      }

      if (vtfcomp (ival, array2[j]))
      {
        vtprintf ("Error in vsip_cvput_f (9)\n"
                  "User array for split block imaginary value did not match "
                  "for el:%i o:%i s:%i l:%i\nexpected:%f got:%f\n", j,
                  offset, stride, length, ival, array2[j]);
        ret |= 1<<9;
        break;
      }

      if (vtfcomp (rval, array3[2*j]))
      {
        vtprintf ("Error in vsip_cvput_f (10)\n"
                  "User array for interleaved block real value did not "
                  "match for el:%i o:%i s:%i l:%i\nexpected:%f got:%f\n", j,
                  offset, stride, length, rval, array3[j*2]);
        ret |= 1<<10;
        break;
      }

      if (vtfcomp (ival, array3[2*j+1]))
      {
        vtprintf ("Error in vsip_cvput_f (11)\n"
                  "User array for interleaved block imaginary value did not "
                  "match for el:%i o:%i s:%i l:%i\nexpected:%f got:%f\n", j,
                  offset, stride, length, ival, array3[j*2+1]);
        ret |= 1<<11;
        break;
      }

      cval = vsip_cvget_f (vall_view, j);

      if (vtfcomp (rval, cval.r))
      {
        vtprintf ("Error in vsip_cvput_f (12)\n"
                  "VSIP block real value did not match as expected for "
                  "el:%i o:%i s:%i l:%i\nexpected:%f got:%f\n", j, 
                  offset, stride, length, rval, cval.r);
        ret |= 1<<12;
        break;
      }

      if (vtfcomp (ival, cval.i))
      {
        vtprintf ("Error in vsip_cvput_f (13)\n"
                  "VSIP block imaginary value did not match as expected for "
                  "el:%i o:%i s:%i l:%i\nexpected:%f got:%f\n", j, 
                  offset, stride, length, ival, cval.i);
        ret |= 1<<13;
        break;
      }
    }

    for (k=0; k<VSIPTEST_CVPUT_F_BLOCK_SIZE; k++)
    {
      j = order[k];
      mod = (signed int)(j - offset ) % stride;
      plc = (signed int)(j - offset ) / stride;

      if (mod == 0 && plc < length)
      {
        rval = (vsip_scalar_f)plc;
        ival = rval * 3;
      }
      else
      {
        rval = 0;
        ival = 0;
      }

      if (vtfcomp (rval, array1[j]))
      {
        vtprintf ("Error in vsip_cvput_f (14)\n"
                  "User array for split block real value did not match "
                  "for random acces el:%i o:%i s:%i l:%i\nexpected:%f "
                  "got:%f\n", j, offset, stride, length, rval, array1[j]);
        ret |= 1<<14;
        break;
      }

      if (vtfcomp (ival, array2[j]))
      {
        vtprintf ("Error in vsip_cvput_f (15)\n"
                  "User array for split block imaginary value did not match "
                  "for random access el:%i o:%i s:%i l:%i\nexpected:%f "
                  "got:%f\n", j, offset, stride, length, ival, array2[j]);
        ret |= 1<<15;
        break;
      }

      if (vtfcomp (rval, array3[2*j]))
      {
        vtprintf ("Error in vsip_cvput_f (16)\n"
                  "User array for interleaved block real value did not "
                  "match for random access  el:%i o:%i s:%i l:%i\nexpected:%f "
                  "got:%f\n", j, offset, stride, length, rval, array3[j*2]);
        ret |= 1<<16;
        break;
      }

      if (vtfcomp (ival, array3[2*j+1]))
      {
        vtprintf ("Error in vsip_cvput_f (17)\n"
                  "User array for interleaved block imaginary value did not "
                  "match for el:%i o:%i s:%i l:%i\nexpected:%f got:%f\n", j,
                  offset, stride, length, ival, array3[j*2+1]);
        ret |= 1<<17;
        break;
      }

      cval = vsip_cvget_f (vall_view, j);

      if (vtfcomp (rval, cval.r))
      {
        vtprintf ("Error in vsip_cvput_f (18)\n"
                  "VSIP block real value did not match as expected for "
                  "random access el:%i o:%i s:%i l:%i\nexpected:%f got:%f\n",
                  j, offset, stride, length, rval, cval.r);
        ret |= 1<<18;
        break;
      }

      if (vtfcomp (ival, cval.i))
      {
        vtprintf ("Error in vsip_cvput_f (19)\n"
                  "VSIP block imaginary value did not match as expected for "
                  "random access el:%i o:%i s:%i l:%i\nexpected:%f got:%f\n", 
                  j, offset, stride, length, ival, cval.i);
        ret |= 1<<19;
        break;
      }
    }
    vsip_cvdestroy_f (vall_view);
    vsip_cblockdestroy_f (vblock);
  }
  free (array1);
  free (array2);
  free (array3);
  report (ret); 
    vsip_finalize ((void *)0);
  return (ret != 0);
}
