/* vsiptest_vputoffset_f.c Version 0.000 August 2, 1999
// 
// This file contains the vsiptest_vputoffset_f test functionality.
// 
// Functional Description:
//   vsip_vputoffset_f sets the attributes for a float view
//
// Copyright June 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//   1) Create a user array filled with ramp values
//   2) Bind a block to this array, and admit it
//   3) Create a VSIPL block, fill it with ramp values
//   4) For each of several offset, stride, length
//      a) bind a view to each block
//      b) putoffset on each block
//      c) check attribs with getattrib
//      d) check the created view to make sure the values match up as 
//         expected for this SOL
//      e) destroy the views
//
// Revision History:
// August 2, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i,j;
  vsip_block_f  *block, *vblock;
  vsip_vview_f  *vall_view, *vview, *uview;
  int            ret = 0;
  vsip_offset    offset;
  vsip_stride    stride;
  vsip_length    length;
  vsip_scalar_f  *array;
  vsip_scalar_f  val;
  vsip_vattr_f   attrib2;

  vsip_init ((void *)0);

  array = (vsip_scalar_f *)
           malloc(VSIPTEST_VPUTOFFSET_F_BLOCK_LENGTH*sizeof(vsip_scalar_f));

  if (!array)
  {
    vtprintf ("Error in vsip_vputoffset_f() (1)\n"
              "Could not allocate data arrays\n");
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  for (i=0; i<VSIPTEST_VPUTOFFSET_F_BLOCK_LENGTH; i++)
  {
    array[i]     = (vsip_scalar_f)    i;
  }

  block = vsip_blockbind_f (array, VSIPTEST_VPUTOFFSET_F_BLOCK_LENGTH, 0);
  if (!block)
  {
    vtprintf ("Error in vsip_vputoffset_f() (2)\n"
              "Could not bind block to user array\n");
    ret |= 1<<2;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  vsip_blockadmit_f (block, 1);

  vblock = vsip_blockcreate_f (VSIPTEST_VPUTOFFSET_F_BLOCK_LENGTH, 0);
  if (!vblock)
  {
    vtprintf ("Error in vsip_vputoffset_f() (3)\n"
              "Could not create VSIPL block\n");
    ret |= 1<<3;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  vall_view = vsip_vbind_f (vblock, 0, 1, VSIPTEST_VPUTOFFSET_F_BLOCK_LENGTH);
  if (!vall_view)
  {
    vtprintf ("Error in vsip_vputoffset_f() (4)\n"
              "Could not create view to all of VSIPL block\n");
    ret |= 1<<4;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  for (i=0; i<VSIPTEST_VPUTOFFSET_F_BLOCK_LENGTH; i++)
  {
    vsip_vput_f (vall_view, i, (vsip_scalar_f)i);
  }
  vsip_vdestroy_f (vall_view);

  for (i=0; i<elements(VSIPTEST_VPUTOFFSET_F_OFFSET); i++)
  {
    offset = VSIPTEST_VPUTOFFSET_F_OFFSET[i];
    stride = VSIPTEST_VPUTOFFSET_F_STRIDE[i];
    length = VSIPTEST_VPUTOFFSET_F_LENGTH[i];

    /* create a view to each block */
    vview = vsip_vbind_f (vblock, 0, stride, length);
    uview = vsip_vbind_f (block, 0, stride, length);

    if (!vview)
    {
      vtprintf ("Error in vsip_vputoffset_f (5)\n"
                "Could not create view to VSIPL block o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<5;
    }

    if (!uview)
    {
      vtprintf ("Error in vsip_vputoffset_f (6)\n"
                "Could not create view to interleaved user block "
                "o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<6;
    }

    vsip_vputoffset_f (vview, offset);
    vsip_vputoffset_f (uview, offset);

    vsip_vgetattrib_f (uview, &attrib2);
    if (attrib2.offset != offset || 
        attrib2.stride != stride || 
        attrib2.length != length)
    {
      vtprintf ("Error in vsip_vputoffsetf_f (7)\n"
                "Actual attributes (%i, %i, %i) did not match set attributes "
                "(%i, %i, %i) for user block \n", attrib2.offset, attrib2.stride,
                attrib2.length, offset, stride, length);
     ret |= 1<<7;
    }

    vsip_vgetattrib_f (vview, &attrib2);
    if (attrib2.offset != offset || 
        attrib2.stride != stride || 
        attrib2.length != length)
    {
      vtprintf ("Error in vsip_vputoffsetf_f (8)\n"
                "Actual attributes (%i, %i, %i) did not match set attributes "
                "(%i, %i, %i) for VSIPL block \n", attrib2.offset, attrib2.stride,
                attrib2.length, offset, stride, length);
     ret |= 1<<8;
    }
    /* check each datum */

    for (j=0; j<length; j++)
    {
      val = vsip_vget_f (vview, j);
      if (vtfcomp (val, (vsip_scalar_f)(offset + j * stride))) 
      {
        vtprintf ("Error in vsip_vputoffset_f (9)\n"
                  "Element %i of VSIPL view did not match expected " 
                  "for o:%i s:%i l:%i expected:%f got:%f\n", 
                  j,
                  offset, stride, length, (float)(offset + j * stride), val);
        ret |= 1<<9;
        break;
      }
      val = vsip_vget_f (uview, j);
      if (vtfcomp (val, (vsip_scalar_f)(offset + j * stride)))
      {
        vtprintf ("Error in vsip_vputoffset_f (10)\n"
                  "Element %i of VSIPL view did not match expected " 
                  "for o:%i s:%i l:%i expected:%f got:%f\n", 
                  j,
                  offset, stride, length, (float)(offset + j * stride), val);
        ret |= 1<<10;
        break;
      }
    }
    vsip_vdestroy_f (uview);
    vsip_vdestroy_f (vview);
  }

  vsip_blockdestroy_f (vblock);
  vsip_blockdestroy_f (block);
  free (array);
  
  report (ret); 
    vsip_finalize ((void *)0);
  return (ret != 0);
}
