/* vsiptest_blockadmit_f.c Version 0.000 March 26, 1999
//
// This file contains the vsiptest_blockadmit_f test functionality.
//
// Functional Description:
//   vsip_blockadmit_f "admits" a VSIPL block object, vsip_dblock_p,
//   for VSIPL operations on the associated views.
//
// Copyright March 1997, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Randall Janka
//         randall.janka@gtri.gatech.edu
//
// Test description:
 *    Create valid VSIPL block and admit it.
 *     Admit it again.
 *     Destroy block and try to admit it.
 *     Update flag will not be tested in this routine, but in release routine.
//
//      Check some of the parameters of this function
//        1) Check the first and last array elements after binding
//        2) Check the size of the blocks
//        3) Check for all possible hints
//        4) Check for binding for negative input lengths (error conditions)
//
// VSIP functions called: vsip_blockadmit_f, vsip_vbind_f, vsip_vget_f,
//                        vsip_vgetlength_f, vsip_vdestroy_f
//
// Revision History:
// March 26, 1999  Created 
// April 24, 2002  Removed erroneous check of readmitted VSIPL block 
*/


#include <stdio.h>
#include "vsiptest_block.h"

int main()
{
  size_t            block_size = VSIPTEST_BLOCKADMIT_BLOCK_SIZE;
  vsip_memory_hint  hint = 0;
  vsip_block_f      *block;
  vsip_scalar_bl    update = 0;
  int               ret = 0;
  int               i;
  vsip_vview_f      *view;

  static vsip_scalar_f  *array;
 
  vsip_init ((void *)0);
  array = (vsip_scalar_f *)malloc(block_size*sizeof(vsip_scalar_f));

  /********************* VSIP_BLOCKCREATE_f ***********************
                           Create the block */
  block = vsip_blockbind_f(array, block_size, hint);

  /* Test update flag */
  for (i=0; i<block_size; i++)
  {
    array[i] = (vsip_scalar_f)i;
  }

    /********************* VSIP_BLOCKADMIT_f *****************************
                      Try to admit the block */
  if (vsip_blockadmit_f(block, 1))
  {
    vtprintf ("Error in vsip_blockadmit_f(); (1)\n"
             "Failed simple instance of valid 5000-el block admit. \n");
    ret |= 1<<1;
  }

  /* test the update flag operation */
  view = vsip_vbind_f (block, 0, 1, block_size);
  for (i=0; i<block_size; i++)
  {
    if (vtfcomp (vsip_vget_f (view, i),(vsip_scalar_f)i))
    {
      vtprintf ("Error in vsip_blockadmit_f(); (2)\n"
        "Element %i of admitted and updated block did not match array.\n", i);
      ret |= 1<<2;
      break;
    }
  }

  /*-----------------3/26/99 2:50PM-------------------
   * If admit is successful, try to admit it again.
   * --------------------------------------------------*/
  if (vsip_blockadmit_f(block, update))
  {
    vtprintf ("Error in vsip_blockadmit_f(); (3)\n"
             "Failed readmit of valid admitted 5000-el block. \n");
    ret |= 1<<3;
  }

#if 0
  /*
   * Do a blockdestroy, then try to admit a view to destroyed block
   * and see if it fails.
   */

  vsip_blockdestroy_f(block);
  if (!vsip_blockadmit_f(block, update))
  {
    vtprintf ("Error in vsip_blockadmit_f(); (4)\n"
             "Admitted invalid (destroyed) 5000-el block. \n");
    ret |= 1<<4;
  }
#endif

  /*-----------------3/29/99 4:28PM-------------------
   * Create known invalid block and try to admit it.
   * --------------------------------------------------*/
  block = vsip_blockbind_f(NULL, 1, 0);
  if ( block != NULL)
  {
    if (!vsip_blockadmit_f(block, update))
    {
      vtprintf ("Error in vsip_blockadmit_f(); (5)\n"
               "Admitted block bound to NULL data. \n");
      ret |= 1<<5;
    }
  }
  vsip_blockdestroy_f (block);
    
#if 0
  block = vsip_blockcreate_f (1, 0);
  if (!vsip_blockadmit_f(block, 0))
  {
    vtprintf ("Error in vsip_blockadmit_f(); (6)\n"
              "Admitted VSIP block. \n");
    ret |= 1<<6;
  }
  vsip_blockdestroy_f (block);
#endif

  report(ret);
  vsip_finalize ((void *)0);
  return (ret != 0);
}
