/* vsiptest_blockadmit_i.c Version 0.000 April 7, 1999
//
// This file contains the vsiptest_blockadmit_i test functionality.
//
// Functional Description:
//   vsip_blockadmit_i "admits" a VSIPL block object, vsip_dblock_p,
//   for VSIPL operations on the associated views.
//
// Copyright March 1997, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Randall Janka
//         randall.janka@gtri.gatech.edu
//         Dan Campbell [adapted to _i]
//         dan.campbell@gtri.gatech.edu
//
// Test description:
 *    Create valid VSIPL block and admit it.
 *     Admit it again.
 *     Destroy block and try to admit it.
 *     Update flag will not be tested in this routine, but in release routine.
//
//      Check some of the parameters of this function
//        1) Check the first and last array elements after binding
//        2) Check the size of the blocks
//        3) Check for all possible hints
//        4) Check for binding for negative input lengths (error conditions)
//
// VSIP functions called: vsip_blockadmit_i, vsip_vbind_i, vsip_vget_i,
//                        vsip_vgetlength_i, vsip_vdestroy_i
//
// Revision History:
// March 26, 1999  Created 
// April  7, 1999  Adapted to _i 
*/

#include <stdio.h>
#include "vsiptest_block.h"

int main()
{
  size_t            block_size = VSIPTEST_BLOCKADMIT_BLOCK_SIZE;
  vsip_memory_hint  hint = 0;
  vsip_block_i      *block;
  vsip_scalar_bl    update = 0;
  int               ret = 0;
  int               i;

  static vsip_scalar_i  *array;
 
  vsip_init ((void *)0);
  
  array = (vsip_scalar_i *)malloc(block_size*sizeof(vsip_scalar_i));

  /********************* VSIP_BLOCKCREATE_i ***********************
                           Create the block */
  block = vsip_blockbind_i(array, block_size, hint);

  /* Test update flag */
  for (i=0; i<block_size; i++)
  {
    array[i] = (vsip_scalar_i)i;
  }

    /********************* VSIP_BLOCKADMIT_i *****************************
                      Try to admit the block */
  if (vsip_blockadmit_i(block, update))
  {
    vtprintf ("Error in vsip_blockadmit_i(); (1)\n"
             "Failed simple instance of valid 5000-el block admit. \n");
    ret |= 1<<1;
  }

#if 0
  /* I don't see a way to test this in _i */
  /* test the update flag operation */
  for (i=0; i<block_size; i++)
  {
    view = vsip_vbind_i (block, i, 1, 1);
    if (vsip_vmaxval_i (view, &index1) != (vsip_scalar_i)i)
    {
      vtprintf ("Error in vsip_blockadmit_i(); (2)\n"
           "Element %i of admitted and updated block did not match array.\n");
      ret |= 1<<2;
      break;
    }
  }
#endif


  /*-----------------3/26/99 2:50PM-------------------
   * If admit is successful, try to admit it again.
   * --------------------------------------------------*/
  if (vsip_blockadmit_i(block, update))
  {
    vtprintf ("Error in vsip_blockadmit_i(); (3)\n"
             "Failed readmit of valid admitted 5000-el block. \n");
    ret |= 1<<3;
  }

#if 0
  /*
   * Do a blockdestroy, then try to admit a view to destroyed block
   * and see if it fails.
   */

  vsip_blockdestroy_i(block);
  if (!vsip_blockadmit_i(block, update))
  {
    vtprintf ("Error in vsip_blockadmit_i(); (4)\n"
             "Admitted invalid (destroyed) 5000-el block. \n");
    ret |= 1<<4;
  }
#endif

  /*-----------------3/29/99 4:28PM-------------------
   * Create known invalid block and try to admit it.
   * --------------------------------------------------*/
  block = vsip_blockbind_i(NULL, 1, 0);
  if ( block != NULL)
  {
    if (!vsip_blockadmit_i(block, update))
    {
      vtprintf ("Error in vsip_blockadmit_i(); (5)\n"
               "Admitted block bound to NULL data. \n");
      ret |= 1<<5;
    }
  }
  vsip_blockdestroy_i (block);
    
#if 0
  block = vsip_blockcreate_i (1, 0);
  if (!vsip_blockadmit_i(block, 0))
  {
    vtprintf ("Error in vsip_blockadmit_i(); (6)\n"
              "Admitted VSIP block. \n");
    ret |= 1<<6;
  }
  vsip_blockdestroy_i (block);
#endif

  report (ret);
  vsip_finalize ((void *)0);
  return (ret != 0);
}
