/* vsiptest_cblockbind_f.c Version 0.000 April 9, 1999
// 
// This file contains the vsiptest_cblockbind_f test functionality.
//
// Functional Description:
//
// Copyright April 9 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Thomas Egolf
//         egolf@ece.gatech.edu
//         Dan Campbell [adapted to c*_f]
//         dan.campbell@gtri.gatech.edu
//
// Test description: This test suite will create real blocks of data with
//
// VSIP functions called: vsip_cblockbind_f, vsip_vbind_f, vsip_vget_f,
//                        vsip_vgetlength_f, vsip_vdestroy_f
//
// Revision History:
// May    6, 1997  Added copyright notice.             
// May    6, 1997  Executed tests 1-4 without error and block sizes up to 5000
// April  9, 1999  Adapted to c*_f
*/

#include<stdio.h>
#include<stdlib.h>  /* For malloc functionality */
#include "vsiptest_block.h"

int main ()
{
  size_t             block_size, i;
  vsip_scalar_f     *array1, *array2;
  vsip_memory_hint   hint = 0;
  vsip_cblock_f      *bind_block;
  vsip_cvview_f      *blockview;
  vsip_vview_f       *iview, *rview;
  vsip_scalar_f      minval, maxval;
  vsip_scalar_vi     index1, index2;
  vsip_length        length;
  vsip_cvattr_f      cattrib;
  vsip_cmplx_mem     cm;

  int ret = 0;

  vsip_init ((void *)0);

  block_size = VSIPTEST_BLOCKBIND_BLOCK_SIZE;

  cm = VSIP_CMPLX_INTERLEAVED;
  cm = VSIP_CMPLX_SPLIT;
  cm = VSIP_CMPLX_NONE;
  cm = cm;

  array1 = (float *)malloc(block_size*sizeof(float));
  array2 = (float *)malloc(block_size*sizeof(float));

  if (!array1 || !array2)
  {
      vtprintf ("Error in vsip_cblockbind_f(); (system) \n"
                "Could not allocate memory buffers of size: %i for tests\n",
                block_size);
      ret |= -1;
  }


  /* API Checks */
  bind_block = vsip_cblockbind_f (NULL, NULL, (vsip_length)(1), hint);
  if (bind_block == NULL)
  {
    vtprintf("Error in vsip_cblockbind_f(); (1) \n"
             "block bind to both NULL pointers returns failure\n");
    ret |= 1<<1;
  }
  if (bind_block) 
  {
    vsip_cblockdestroy_f (bind_block);
  }
  
  bind_block = vsip_cblockbind_f (array1, NULL, (vsip_length)(1), hint);
  if (bind_block == NULL)
  {
    vtprintf("Error in vsip_cblockbind_f(); (2) \n"
             "block bind to interleaved data returns failure\n");
    ret |= 1<<2;
  }
  if (bind_block) 
  {
    vsip_cblockdestroy_f (bind_block);
  }

  bind_block = vsip_cblockbind_f (array1, array2, (vsip_length)(1), hint);
  if (bind_block == NULL)
  {
    vtprintf("Error in vsip_cblockbind_f(); (3) \n"
             "block bind to split data returns failure\n");
    ret |= 1<<3;
  }
  if (bind_block) 
  {
    vsip_cblockdestroy_f (bind_block);
  }


    /* Fill the block with data values equal to its index value */
    for(i=0; i<block_size; i++) 
    {
      array1[i] = (vsip_scalar_f)i;
      array2[i] = (vsip_scalar_f)i*2;
    }

    /********************* VSIP_CBLOCKBIND_F **************************
      Bind to the public block with at least block_size real
      elements and look at the contents of the binded block. */

    bind_block = vsip_cblockbind_f(array1, array2, block_size, hint);
    vsip_cblockadmit_f (bind_block, 1);
    /****************************************************************/

    /********************* VSIP_VBIND_D *****************************
      Create a view on the bound block */

    blockview = vsip_cvbind_f(bind_block, 0, 1, (vsip_length)block_size);
    iview = vsip_vimagview_f (blockview);
    rview = vsip_vrealview_f (blockview);

    /* Check some of the parameters of this function
    //    1) Check the first and last array elements
    //    2) Check the size of the block */

    /********************* VSIP_VGET_D *******************************
      Check the minimum and maximum values of the view on the block */

    index1 = 0; index2 = block_size-1;
    minval = vsip_vget_f(iview, index1);
    maxval = vsip_vget_f(iview, index2);

    if(vtfcomp (minval, (vsip_scalar_f)0.0) )
    {
          vtprintf("Error in vsip_blockbind_f(); (4) \n"
            "First value assignment does not match first element value \n");
          ret |= 1<<4;
    }

    if(vtfcomp (maxval, (vsip_scalar_f)(block_size-1)*2) )
    {
      vtprintf("Error in vsip_blockbind_f(); (5) \n"
               "Last value assignment does not match last element value \n");
      ret |= 1<<5;
    }

    index1 = 0; index2 = block_size-1;
    minval = vsip_vget_f(rview, index1);
    maxval = vsip_vget_f(rview, index2);

    if(vtfcomp (minval,(vsip_scalar_f)0.0) )
    {
          vtprintf("Error in vsip_blockbind_f(); (6) \n"
            "First value assignment does not match first element value \n");
          ret |= 1<<6;
    }

    if(vtfcomp (maxval, (vsip_scalar_f)(block_size-1)) )
    {
      vtprintf("Error in vsip_blockbind_f(); (7) \n"
               "Last value assignment does not match last element value \n");
      ret |= 1<<7;
    }

    vsip_cvdestroy_f (blockview);
    vsip_vdestroy_f (iview);
    vsip_vdestroy_f (rview);
    vsip_cblockdestroy_f (bind_block);

  /******************************************
  *                                         *
  *                Interleaved              *             
  *                                         *
  *******************************************/
    block_size /= 2;

    bind_block = vsip_cblockbind_f(array1, NULL, block_size, hint);
    vsip_cblockadmit_f (bind_block, 1);

    /********************* VSIP_VBIND_D *****************************
      Create views on the bound block */

    blockview = vsip_cvbind_f(bind_block, 0, 1, (vsip_length)block_size);
    iview = vsip_vimagview_f (blockview);
    rview = vsip_vrealview_f (blockview);

    /* Check some of the parameters of this function
    //    1) Check the first and last array elements
    //    2) Check the size of the block */

    /********************* VSIP_VGET_D *******************************
      Check the minimum and maximum values of the view on the block */

    index1 = 0; 
    index2 = block_size-1;
    minval = vsip_vget_f(iview, index1);
    maxval = vsip_vget_f(iview, index2);

    if(vtfcomp (minval, (vsip_scalar_f)1.0) )
    {
          vtprintf("Error in vsip_blockbind_f(); (8) \n"
            "First value assignment does not match first element value \n");
          ret |= 1<<8;
    }

    if(vtfcomp (maxval, (vsip_scalar_f)(block_size*2-1)) )
    {
      vtprintf("Error in vsip_blockbind_f(); (9) \n"
               "Last value assignment does not match last element value \n");
      ret |= 1<<9;
    }

    index1 = 0; index2 = block_size-1;
    minval = vsip_vget_f(rview, index1);
    maxval = vsip_vget_f(rview, index2);

    if(vtfcomp (minval, (vsip_scalar_f)0.0) )
    {
          vtprintf("Error in vsip_blockbind_f(); (10) \n"
            "First value assignment does not match first element value \n");
          ret |= 1<<10;
    }

    if(vtfcomp (maxval ,(vsip_scalar_f)(block_size-1)*2) )
    {
      vtprintf("Error in vsip_blockbind_f(); (11) \n"
               "Last value assignment does not match last element value \n");
      ret |= 1<<11;
    }


    /********************* VSIP_VGETLENGTH_D ************************
      Check the length of the block allocated */

    vsip_cvgetattrib_f(blockview, &cattrib);
    length = cattrib.length;
    if(length != (vsip_length)block_size)
    {
      vtprintf("Error in vsip_blockbind(); (12) \n"
       "Size does not match allocated size \n");
      ret |= 1<<4;
    }

    /********************* VSIP_VDESTROY_D ***************************
      Destroy the view on the block */
    vsip_cvdestroy_f(blockview);
    vsip_vdestroy_f (iview);
    vsip_vdestroy_f (rview);
    vsip_cblockdestroy_f (bind_block);
    

    free(array1);
    free(array2);

  report (ret);
  vsip_finalize ((void *)0);
  return (ret!=0);
}

