%File:						vsip_firflt_gen.m
%
%Author:						Randy Janka
%Date:						2/25/00
%
%Purpose:					Generate test data for vsip_firflt_f
%								Generate x[n] and compute y[n]=h[n]*x[n]
%									for different combinations of parameters
%
%Parameters:				N = Length = length(x) = 648
%								symm = FULL, EVEN, ODD
%								D = 1, 2, 3
%                       state = SAVE and NO_SAVE
%                            SAVE => s=1,p=1; Internal state variables
%                         NO_SAVE => s=0,p=0; used only in data generation.
%								DATAPATH = dir where data file is to go;
%									set in workspace
%
% Test #	 f#    symm     N    D   filter    M     state  => s p
% ======  ==    ====    ===   =   ======   ==    =======    = =
%
%   1a    f1    NON     324   1    bIII    26     SAVE      1 1
%   1b    f2    NON     324   1    bIII    26    NO_SAVE    0 0
%   2     f3    NON     648   1    bIV     23    NO_SAVE    0 0
%	 3a  	 f4    EVEN    324   2    bI             SAVE      1 1
%	 3b  	 f5    EVEN    324   2    bI            NO_SAVE    0 0
%	 4	 	 f6    EVEN    648   2    bI            NO_SAVE    0 0
%	 5a 	 f7    ODD     324   3    bII            SAVE      1 1
%	 5b 	 f8    ODD     324   3    bII           NO_SAVE    0 0
%	 6	 	 f9    ODD     648   3    bII           N0_SAVE    0 0

%Define constants
N_whole=648;
N_div_2=N_whole/2;
ntimes=0; %semi-infinite
%symm definitions
VSIP_NONSYM=0;
VSIP_SYM_EVEN_LEN_ODD=1;
VSIP_SYM_EVEN_LEN_EVEN=2;
%state definitions
VSIP_STATE_NO_SAVE=1;
VSIP_STATE_SAVE=2;

%Generate filter coefficients
%   M = Filter order
% M+1 = Number of filter coefficients
%
%bI
%==
%Type I filter coefficient generation
%Even coefficient symmetry
%M=26 (even), M+1 odd
%
%bII
%===
%Type II filter coefficient generation
%Even coefficient symmetry
%M=23 (odd), M+1 even
%
%bIII
%====
%Type III filter coefficient generation
%Odd coefficient symmetry
%M=26 (even), M+1 odd
%
%bIV
%===
%Type IV filter coefficient generation
%Odd coefficient symmetry
%M=23 (odd), M+1 even

filter_coeffs_gen

%Generates signal to be filtered:
%  Three tones of unit amplitude plus AWGN.
%
sig_gen
xn_1of2=xn(1:N_div_2);
xn_2of2=xn(N_div_2+1:N_whole);

%Test #1
M=length(bIII)-1;
N=N_div_2;
D=1;
symm=VSIP_NONSYM;
%Test #1a
state=VSIP_STATE_SAVE;
%Write data file
[y1a,zf]=filter(bIII,1,xn_1of2);
h=[bIII,zeros(1,N-(length(bIII)))];
out=[xn_1of2; y1a; h];
fid=fopen([DATAPATH,'\firflt_f1'],'w');
fprintf(fid,'%1d %3d %1d %1d %3d %2d\n',symm,N,D,state,ntimes,M);
fprintf(fid,'%12.7e\t%12.7e\t%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('firflt_f1 write successful.')
else
   disp('firflt_f1 write NOT successful.')
end
%Test #1b
y1b=filter(bIII,1,xn_2of2,zf);
out=[xn_2of2; y1b; h];
fid=fopen([DATAPATH,'\firflt_f2'],'w');
fprintf(fid,'%1d %3d %1d %1d %3d %2d\n',symm,N,D,state,ntimes,M);
fprintf(fid,'%12.7e\t%12.7e\t%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('firflt_f2 write successful.')
else
   disp('firflt_f2 write NOT successful.')
end

