%File:						vsip_histo_gen.m
%
%Author:						Randy Janka
%Date:						2/15/00
%
%Purpose:					Generate test data for vsip_ccfftop_f
%								Generate x[n] and compute X[k]=F{x[n]}
%									for different combinations of parameters
%
%Parameters:				N = Length = length(x) = 4, 768, 1024, 19
%								scale = 1, 3, 1/N
%								ntimes = 0, 1
%								DATAPATH = dir where data file is to go;
%									set in workspace
%
% Test #		Input signal	  N	  scale	ntimes
% ======		=============	====	  =====  ======
%
%    1		Tone at n0=1	   4		 1			0
%    2		Tone at n0=2	 768		 3			1
%	  3		Tone at n0=3	1024		1/N		0
%	  4		Tone at n0=4	  19		 1			1
%	  5		Tone at n0=5	1024		 1			1
%	  6		Pulse at n0=6	1024		 1			1
%	  7		Constant A=7	1024		 1			1
%	  8		Noise only		1024		 1			1

%Constants
pi2=2*pi;

%Test #1
%Define parameters
N     =4;
scale =1;
ntimes=0;
%Generate tone at n0=1
k=1;
x=exp(j*pi2*[0:(N-1)]*k/N);
n=noise(N)/N;
xn=x+n;
%Form y
y=scale*fft(xn);
%Write data file
out=[real(xn); imag(xn); real(y); imag(y)];
fid=fopen([DATAPATH,'\ccfftop_f1'],'w');
fprintf(fid,'%4d %4d %4d\n',N,scale,ntimes);
fprintf(fid,'%12.7e,%12.7e\t%12.7e,%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('ccfftop_f1 write successful.')
else
   disp('ccfftop_f1 write NOT successful.')
end
