%File:						vsip_ccfftop_gen.m
%
%Author:						Randy Janka
%Date:						12/15/99
%
%Purpose:					Generate test data for vsip_ccfftop_f
%								Generate x[n] and compute X[k]=F{x[n]}
%									for different combinations of parameters
%
%Parameters:				N = Length = length(x) = 4, 768, 1024, 19
%								scale = 1, 3, 1/N
%								ntimes = 0, 1
%								DATAPATH = dir where data file is to go;
%									set in workspace
%
% Test #		Input signal	  N	  scale	ntimes
% ======		=============	====	  =====  ======
%
%    1		Tone at n0=1	  23		 1			0
%    2		Tone at n0=2	 768		 3			1
%	  3		Tone at n0=3	1024		1/N		0
%	  4		Tone at n0=4	  19		 1			1
%	  5		Tone at n0=5	1024		 1			1
%	  6		Pulse at n0=6	1024		 1			1
%	  7		Constant A=7	1024		 1			1
%	  8		Noise only		1024		 1			1

%Constants
pi2=2*pi;

%Test #1
%Define parameters
N     =23;
scale =1;
ntimes=0;
%Generate tone at n0=1
k=1;
x=exp(j*pi2*[0:(N-1)]*k/N);
n=noise(N)/N;
xn=x+n;
%Form y
y=scale*fft(xn);
%Write data file
out=[real(xn); imag(xn); real(y); imag(y)];
fid=fopen([DATAPATH,'\ccfftop_f1'],'w');
fprintf(fid,'%4d %4d %4d\n',N,scale,ntimes);
fprintf(fid,'%12.7e,%12.7e\t%12.7e,%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('ccfftop_f1 write successful.')
else
   disp('ccfftop_f1 write NOT successful.')
end

%Test #2
%Define parameters
N     =768;
scale =3;
ntimes=1;
%Generate tone at n0=2
n0=2;
x=exp(j*pi2*[0:(N-1)]*n0/N);
n=noise(N)/N;
xn=x+n;
%Form y
y=scale*fft(xn);
%Write data file
out=[real(xn); imag(xn); real(y); imag(y)];
fid=fopen([DATAPATH,'\ccfftop_f2'],'w');
fprintf(fid,'%4d %4d %4d\n',N,scale,ntimes);
fprintf(fid,'%12.7e,%12.7e\t%12.7e,%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('ccfftop_f2 write successful.')
else
   disp('ccfftop_f2 write NOT successful.')
end

%Test #3
%Define parameters
N     =1024;
scale =1/N;
ntimes=0;
%Generate tone at n0=3
n0=3;
x=exp(j*pi2*[0:(N-1)]*n0/N);
n=noise(N)/N;
xn=x+n;
%Form y
y=scale*fft(xn);
%Write data file
out=[real(xn); imag(xn); real(y); imag(y)];
fid=fopen([DATAPATH,'\ccfftop_f3'],'w');
fprintf(fid,'%4d %4d %4d\n',N,scale,ntimes);
fprintf(fid,'%12.7e,%12.7e\t%12.7e,%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('ccfftop_f3 write successful.')
else
   disp('ccfftop_f3 write NOT successful.')
end

%Test #4
%Define parameters
N     =19;
scale =1;
ntimes=1;
%Generate tone at n0=4
n0=4;
x=exp(j*pi2*[0:(N-1)]*n0/N);
n=noise(N)/N;
xn=x+n;
%Form y
y=scale*fft(xn);
%Write data file
out=[real(xn); imag(xn); real(y); imag(y)];
fid=fopen([DATAPATH,'\ccfftop_f4'],'w');
fprintf(fid,'%4d %4d %4d\n',N,scale,ntimes);
fprintf(fid,'%12.7e,%12.7e\t%12.7e,%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('ccfftop_f4 write successful.')
else
   disp('ccfftop_f4 write NOT successful.')
end

%Test #5
%Define parameters
N     =1024;
scale =1;
ntimes=1;
%Generate tone at n0=5
n0=5;
x=exp(j*pi2*[0:(N-1)]*n0/N);
n=noise(N)/N;
xn=x+n;
%Form y
y=scale*fft(xn);
%Write data file
out=[real(xn); imag(xn); real(y); imag(y)];
fid=fopen([DATAPATH,'\ccfftop_f5'],'w');
fprintf(fid,'%4d %4d %4d\n',N,scale,ntimes);
fprintf(fid,'%12.7e,%12.7e\t%12.7e,%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('ccfftop_f5 write successful.')
else
   disp('ccfftop_f5 write NOT successful.')
end

%Test #6
%Define parameters
N     =1024;
scale =1;
ntimes=1;
%Generate pulse at n0=6
n0=6;
x=zeros(1,N);
x(n0)=1;
n=noise(N)/N;
xn=x+n;
%Form y
y=scale*fft(xn);
%Write data file
out=[real(xn); imag(xn); real(y); imag(y)];
fid=fopen([DATAPATH,'\ccfftop_f6'],'w');
fprintf(fid,'%4d %4d %4d\n',N,scale,ntimes);
fprintf(fid,'%12.7e,%12.7e\t%12.7e,%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('ccfftop_f6 write successful.')
else
   disp('ccfftop_f6 write NOT successful.')
end

%Test #7
%Define parameters
N     =1024;
scale =1;
ntimes=1;
%Generate flat signal, A=6
A=7;
x=A*ones(1,N);
n=noise(N)/N;
xn=x+n;
%Form y
y=scale*fft(xn);
%Write data file
out=[real(xn); imag(xn); real(y); imag(y)];
fid=fopen([DATAPATH,'\ccfftop_f7'],'w');
fprintf(fid,'%4d %4d %4d\n',N,scale,ntimes);
fprintf(fid,'%12.7e,%12.7e\t%12.7e,%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('ccfftop_f7 write successful.')
else
   disp('ccfftop_f7 write NOT successful.')
end

%Test #8
%Define parameters
N     =1024;
scale =1;
ntimes=1;
%Generate noise signal
n=noise(N)/N;
xn=n;
%Form y
y=scale*fft(xn);
%Write data file
out=[real(xn); imag(xn); real(y); imag(y)];
fid=fopen([DATAPATH,'\ccfftop_f8'],'w');
fprintf(fid,'%4d %4d %4d\n',N,scale,ntimes);
fprintf(fid,'%12.7e,%12.7e\t%12.7e,%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('ccfftop_f8 write successful.')
else
   disp('ccfftop_f8 write NOT successful.')
end
