%File:						vsip_histo_gen.m
%
%Author:						Randy Janka
%Date:						2/17/00
%
%Purpose:					Generate test data for vsip_histo_f
%								Generate x[n] and compute X[k]=F{x[n]}
%									for different combinations of parameters
%
%Parameters:				src		Source vector to be "histo'd"
%								in_len	Length of src
%								out_len	Length of histogram
%												Bin count includes two for outliers;
%												i.e., hist(1) is bin to left of "min_bin"
%												and hist(out_len) is bin to right of "max_bin"
%								min_bin	Min value to be in histogram (min edge)
%								max_bin	Max value to be in histogram (max edge)
%								opt		Histogram state flag
%												opt=1==VSIP_HISTO_RESET
%												opt=2==VSIP_HISTO_ACCUM
%
%								DATAPATH = dir where data file is to go;
%									set in workspace
%
% Test #		     Input signal	       in_len   out_len   min_bin   max_bin    opt
% ======		=====================    ======   =======   =======   =======   =====
%
%    1		  Ramp [-1:0.1:10.9]      120      10+2        0        10        1 
%    2		  Ramp [-2:0.1:11.9]      140      10+2        0        10        2 
%    3	 	 Variable ramp (below)    195      15+2       -5        10        1 
%           (Inputs are shuffled.)

%TEST #1
%=======

%Generate input vector
src_gen=[-1:0.1:10.9];
src=shuffle(src_gen);
in_len=length(src);
out_len=10+2;
min_bin=0;
max_bin=10;
hist1=zeros(out_len,1);

%Form histogram
opt=1; % Reset histogram.
hist1=vsip_histo(src,min_bin,max_bin,opt,hist1);
y_out=[hist1; zeros((in_len-out_len),1)]';

%Write data file
out=[src; y_out];
fid=fopen([DATAPATH,'\vhisto_f1'],'w');
fprintf(fid,'%4d %4d %4d %4d %4d\n',in_len,out_len,min_bin,max_bin,opt);
fprintf(fid,'%12.7e\t%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('vhisto_f1 write successful.')
else
   disp('vhisto_f1 write NOT successful.')
end

%TEST #2
%=======

%Generate input vector
src_gen=[-2:0.1:11.9];
src=shuffle(src_gen);
in_len=length(src);
out_len=10+2;
min_bin=0;
max_bin=10;

%Form histogram
opt=2; % Use last histogram; accumulate.
hist2=vsip_histo(src,min_bin,max_bin,opt,hist1);
y_out=[hist2; zeros((in_len-out_len),1)]';

%Write data file
out=[src; y_out];
fid=fopen([DATAPATH,'\vhisto_f2'],'w');
fprintf(fid,'%4d %4d %4d %4d %4d\n',in_len,out_len,min_bin,max_bin,opt);
fprintf(fid,'%12.7e\t%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('vhisto_f2 write successful.')
else
   disp('vhisto_f2 write NOT successful.')
end

%TEST #3
%=======

%Generate input vector
src_gen=[-4.5*ones(50,1);
   -3.5*ones(40,1);
   -2.5*ones(30,1);
   -1.5*ones(20,1);
   -0.5*ones(10,1);
   1.5;2.5*ones(2,1);3.5*ones(3,1);4.5*ones(4,1);5.5*ones(5,1);
   6.5*ones(6,1);7.5*ones(7,1);8.5*ones(8,1);9.5*ones(9,1)]';
src=shuffle(src_gen);
in_len=length(src);
out_len=15+2;
min_bin=-5;
max_bin=10;
hist3=ones(out_len,1); % Should reset inside vsip_histo.

%Form histogram
opt=1; % Reset histogram.
hist3=vsip_histo(src,min_bin,max_bin,opt,hist3);
y_out=[hist3; zeros((in_len-out_len),1)]';

%Write data file
out=[src; y_out];
fid=fopen([DATAPATH,'\vhisto_f3'],'w');
fprintf(fid,'%4d %4d %4d %4d %4d\n',in_len,out_len,min_bin,max_bin,opt);
fprintf(fid,'%12.7e\t%12.7e\n',out);
status=fclose(fid);
if ~status
   disp('vhisto_f3 write successful.')
else
   disp('vhisto_f3 write NOT successful.')
end
