%File:						vsip_vatan2_gen.m
%
%Author:						Randy Janka
%Date:						11/24/99
%Reference:					Chapter 11 of Cody/Waite
%
%Purpose:					Generate test data for vsip_vatan2_f
%								Generate x,y and z=atan2(v,u)
%									where x,y are vectors of length N
%									and x=v/u.
%									We set v=x*sqrt(3) and u=(+/-)sqrt(3)
%
%Parameters:				N = Length = length(x) = 1072
%								DATAPATH = dir where data file is to go;
%									set in workspace

%Unit circle axis values
%
v0=[1,0,-1,0];
u0=[0,1,0,-1];

%x>0
%
%Small arg:					atan2(x)=x, |x|<<1; |x|<0.01 for single precision
x1=10.^[-7:-3];
%Primary range:			|x|<xmax; xmax=2+sqrt(3)=3.73205080756888
xmax=2+sqrt(3);
x2=[0.01:0.01:xmax];
%Knee of atan2(x):			20<x<100
x3=[3.8:0.1:20];
x4=[22:1:100];
%Asymptote of atan2(x):	atan2(x)->pi/2 as x->inf; x>100 for single precision
x5=10.^[3:7];
xpos=[x1,x2,x3,x4,x5];

%x<0
%
%Small arg
x1neg=-10.^[-3:-1:-7];
%Primary range
x2neg=[-xmax:0.01:-0.01];
%Knee of atan2(x)			
x3neg=[-10:0.1:-3.8];
%Asymptote of atan2(x)
x4neg=-10.^[7:-1:2];
xneg=[x4neg,x3neg,x2neg,x1neg];

%Form x, x_out, and v_out & u_out
x=[xneg,xpos];
x_out=shuffle(x);
v=sqrt(3)*x_out;
N=length(x_out);
u=sqrt(3)*ones(1,length(v));
u(1:2:N)=-u(1:2:N);
v_out=[v0, v];
u_out=[u0, u];

%Form y
y_out=atan2(v_out,u_out);

%Write data file
out=[v_out; u_out; y_out];
fid=fopen([DATAPATH,'\vatan2_f'],'w');
fprintf(fid,'%4d\n',N);
fprintf(fid,'%12.7e\t%12.7e\t%12.7e\n',out);
status=fclose(fid);

if ~status
   disp('vatan2_f write successful.')
else
   disp('vatan2_f write NOT successful.')
end
