%File:						vsip_vlog_gen.m
%
%Author:						Randy Janka
%Date:						11/26/99
%Reference:					Chapter 5 of Cody/Waite
%
%Purpose:					Generate test data for vsip_vlog_f
%								Generate x and y=log(x)
%									where x is a vector of length N
%
%Parameters:				N = Length = length(x) = 1254
%								DATAPATH = dir where data file is to go;
%									set in workspace

%Close to zero:			x->0+
x0=10.^[-7:-2];

%Inbetween:					0+<x<(1/sqrt(2))
delta=0.01;
x1=[(max(x0)+delta):delta:((1/sqrt(2))-delta)];

%Primary range:			xmin=1/sqrt(2)<x<xmax=45/16
delta=0.00025;
xmin=1/sqrt(2);
xmax=15/16;
x2=[xmin:delta:xmax];

%Large values of x:		xmin=16<x<xmax=240; start at x=ceil(45/16)
xmin=16;
xmax=240;
x3=[[ceil(45/16):xmin-1],[xmin:xmax]];

%Larger values of x:		x>>xmax
x4=[[250:50:950],10.^[3:7]];

%Form x
x=[x0,x1,x2,x3,x4];
x_out=shuffle(x);

%Form y
y_out=log(x_out);

%Write data file
N=length(x);
out=[x_out; y_out];
fid=fopen([DATAPATH,'\vlog_f'],'w');
fprintf(fid,'%4d\n',N);
fprintf(fid,'%12.7e\t%12.7e\n',out);
status=fclose(fid);

if ~status
   disp('vlog_f write successful.')
else
   disp('vlog_f write NOT successful.')
end
