/* vsiptest_ccfftop_f.c Version 0.000 October 24, 1999
// 
// This file contains the vsiptest_ccfftop_f test functionality.
// 
// Functional Description:
//   vsip_ccfftop_f does a complex FFT
//  
// Copyright October 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Revision History:
// Oct 25 1999 - Created
*/

#include"vsiptest_sigproc.h"
#include <stdio.h>

void vector_read (char *filename, vsip_cvview_f **input, 
                  vsip_cvview_f **output, vsip_length *size, 
                  vsip_scalar_f *scale, vsip_length *ntimes)
{
  FILE *fp;
  int read = 0;
  int i;
  float val1, val2, val3, val4;
  int sze, nt;
  vsip_alg_hint ah;
  vsip_symmetry vs;
  
  ah = VSIP_ALG_TIME;
  ah = VSIP_ALG_SPACE;
  ah = VSIP_ALG_NOISE;
  ah = ah;

  vs = VSIP_NONSYM;
  vs = VSIP_SYM_EVEN_LEN_ODD;
  vs = VSIP_SYM_EVEN_LEN_EVEN;
  vs = vs;


  fp = fopen (filename, "r");
  if (!fp)
  {
    vtprintf ("Error in vsip_ccfftop_f (0)\n"
              "unable to open data file %s for reading\n", filename);
    exit(1);
  }

  read = fscanf (fp, "%i %f %i\n", &sze, scale, &nt);
  *size = sze;
  *ntimes = nt;
  if (read != 3)
  {
    vtprintf ("Error in vsip_ccfftop_f (0)\n"
              "unable to read FFT parameters in %s\n", filename);
    fclose (fp);
    exit (1);
  }
  
  *input = vsip_cvcreate_f (*size, 0);
  *output = vsip_cvcreate_f (*size, 0);

  if (!input || !output)
  {
    vtprintf ("Error in vsip_ccfftop_f (0)\n"
              "unable to create view of size %i for %s\n", *size, filename);
    fclose (fp);
    exit(1);
  }

  for (i=0; i<*size; i++)
  {
    read = fscanf (fp, "%f,%f %f,%f\n", &val1, &val2, &val3, &val4);
    if (read < 4)
    {
      vtprintf ("Error in vsip_ccfftop_f (0)\n"
                "unable to read row %i in %s\n", i, filename);
      fclose (fp);
      exit (1);
    }
    vsip_cvput_f (*input, i, vsip_cmplx_f (val1, val2));
    vsip_cvput_f (*output, i, vsip_cmplx_f (val3, val4));
  }
  
  fclose (fp);
}


int main(int argc, char *argv[])
{
  int ret = 0;
  int numfiles = VSIPTEST_CCFFTOP_F_NUMFILES;
  char filename[120];
  int trial;
  vsip_cvview_f *tempv, *inputv, *outputv;
  vsip_scalar_f scale;
  vsip_length size, ntimes;
  vsip_fft_f *fft1, *fft2;
  int i;
  vsip_scalar_f e_real, e_imag, r_real, r_imag;

  vsip_fft_dir dir;
  vsip_alg_hint alg;

  vsip_init ((void *)0);

  dir = VSIP_FFT_FWD;
  dir = VSIP_FFT_INV;
  dir = dir;

  alg = VSIP_ALG_TIME;
  alg = VSIP_ALG_SPACE;
  alg = VSIP_ALG_NOISE;
  alg = alg;

  error_init (argv[1]);

  for (trial=0; trial<numfiles; trial++)
  {
    sprintf (filename, "data/%s%i", VSIPTEST_CCFFTOP_F_BASENAME, 
             trial+1);

    /* this creates two vectors */
    vector_read (filename, &inputv, &outputv, &size, &scale, &ntimes);

    tempv = vsip_cvcreate_f (size, 0);
    fft1 = vsip_ccfftop_create_f (size, scale, VSIP_FFT_FWD, ntimes, 0);
    if (!fft1)
    {
      vtprintf ("Error in vsip_ccfftop_create_f (1)\n"
                "Could not create FFT object of size=%i scale=%f "
                "ntimes=%i trial=%i forward\n", size, scale, ntimes, trial+1);
      ret |= 1<<1;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    fft2 = vsip_ccfftop_create_f (size, 1/(scale*size), VSIP_FFT_INV, ntimes, 0);
    if (!fft2)
    {
      vtprintf ("Error in vsip_ccfftop_create_f (2)\n"
                "Could not create FFT object of size=%i scale=%f "
                "ntimes=%i inverse\n", size, scale, ntimes);
      ret |= 1<<2;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    vsip_ccfftop_f (fft1, inputv, tempv);

    for (i=0; i<size; i++)
    {
      r_real = vsip_real_f (vsip_cvget_f (tempv, i));
      r_imag = vsip_imag_f (vsip_cvget_f (tempv, i));

      e_real = vsip_real_f (vsip_cvget_f (outputv, i));
      e_imag = vsip_imag_f (vsip_cvget_f (outputv, i));

      update_error_element (sqrt((r_real-e_real)*(r_real-e_real)+(r_imag-e_imag)*(r_imag-e_imag)));

      if ( (fabs (r_real - e_real) > VSIPTEST_CCFFTOP_F_MAX_EPSILON) ||
           (fabs (r_imag - e_imag) > VSIPTEST_CCFFTOP_F_MAX_EPSILON)    )
      {
        vtprintf ("Error in vsip_ccfftop_f (3)\n"
                  "Element %i of result vector (%f, %f) did not match "
                  "expected value (%f, %f)\n"
                  "for size=%i scale=%f ntimes=%i trial=%i forward\n", i, r_real,
                  r_imag, e_real, e_imag, size, scale, ntimes, trial+1);
        ret |= 1<<3;
        exit(1);
        break;
      }
      if ((r_real < 0 && e_real >= 0) || 
          (r_real == 0 && e_real != 0) ||
          (r_real > 0 && e_real <= 0) ||
          (r_imag < 0 && e_imag >= 0) || 
          (r_imag == 0 && e_imag != 0) ||
          (r_imag > 0 && e_imag <= 0))
      {
        vtprintf ("Error in vsip_ccftop_f (4)\n"
                  "Element %i of result vector (%f, %f) failed sign test "
                  "against expected value (%f, %f)\n"
                  "for size=%i scale=%f ntimes=%i trial=%i forward\n", i, r_real,
                  r_imag, e_real, e_imag, size, scale, ntimes, trial+1);
        ret |= 1<<4;
        break;
      }
    }
    update_error_total();
    error_report_test ("Forward test #%i for size=%i scale=%f ntimes=%i", trial, size, scale, ntimes);

    vsip_ccfftop_f (fft2, outputv, tempv);

    for (i=0; i<size; i++)
    {
      r_real = vsip_real_f (vsip_cvget_f (tempv, i));
      r_imag = vsip_imag_f (vsip_cvget_f (tempv, i));

      e_real = vsip_real_f (vsip_cvget_f (inputv, i));
      e_imag = vsip_imag_f (vsip_cvget_f (inputv, i));

      update_error_element (sqrt((r_real-e_real)*(r_real-e_real)+(r_imag-e_imag)*(r_imag-e_imag)));

      if (fabs (r_real - e_real) > VSIPTEST_CCFFTOP_F_MAX_EPSILON ||
          fabs (r_imag - e_imag) > VSIPTEST_CCFFTOP_F_MAX_EPSILON)
      {
        vtprintf ("Error in vsip_ccfftop_f (5)\n"
                  "Element %i of result vector (%f, %f) did not match "
                  "expected value (%f, %f)\n"
                  "for size=%i scale=%f ntimes=%i trial=%i inverse\n", i, r_real,
                  r_imag, e_real, e_imag, size, scale, ntimes, trial+1);
        vtprintf ("Trial #%i\n", trial+1);
        ret |= 1<<5;
        break;
      }
      if ((r_real < 0 && e_real >= 0) || 
          (r_real == 0 && e_real != 0) ||
          (r_real > 0 && e_real <= 0) ||
          (r_imag < 0 && e_imag >= 0) || 
          (r_imag == 0 && e_imag != 0) ||
          (r_imag > 0 && e_imag <= 0))
      {
        vtprintf ("Error in vsip_ccfftop_f (6)\n"
                  "Element %i of result vector (%f, %f) failed sign test "
                  "against expected value (%f, %f)\n"
                  "for size=%i scale=%f ntimes=%i trial=%i forward\n", i, r_real,
                  r_imag, e_real, e_imag, size, scale, ntimes, trial+1);
        ret |= 1<<6;
        break;
      }
    }
    update_error_total();
    error_report_test ("Inverse test #%i for size=%i scale=%f ntimes=%i", trial+1, size, scale, ntimes);
    vsip_cvalldestroy_f (inputv);
    vsip_cvalldestroy_f (outputv);
    vsip_cvalldestroy_f (tempv);

    vsip_fft_destroy_f (fft1);
    vsip_fft_destroy_f (fft2);
  }
  error_finish();
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
