/* vsiptest_CMPLX_f.c Version 0.000 July 28, 1999
// 
// This file contains the vsiptest_CMPLX_f test functionality.
// 
// Functional Description:
//     vsip_CMPLX_f retruns a complex scalar formed by two real scalars.
//  
//
// Copyright July 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: This test will step through a specified range of real
//                   and imaginary numbers, creating complex numbers, and 
//                   checking to make sure that the reverse operation yields
//                   the input values
//                   
//
// Revision History:
// July 26, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i,j;
  int            ret = 0;
  vsip_scalar_f  rscalar,iscalar;
  vsip_cscalar_f cscalar;

  vsip_init ((void *)0);

  for (i=VSIPTEST_CMPLX_F_REAL_MIN; i<VSIPTEST_CMPLX_F_REAL_MAX; i++)
  {
    for (j=VSIPTEST_CMPLX_F_IMAG_MIN; j<VSIPTEST_CMPLX_F_IMAG_MAX; j++)
    {
       vsip_CMPLX_f (i, j, &cscalar);

      rscalar = vsip_real_f (cscalar);
      iscalar = vsip_imag_f (cscalar);

      if (vtfcomp (rscalar, (vsip_scalar_f)i))
      {
        vtprintf ("Error in vsip_CMPLX_f() (1)\n"
                  "Real part of (%i,%i) did not match expected (%f)\n",
                  i, j, rscalar);
        ret |= 1<<1;
      }

      if (vtfcomp (iscalar, (vsip_scalar_f)j))
      {
        vtprintf ("Error in vsip_CMPLX_f() (2)\n"
                  "Imaginary part of (%i,%i) did not match expected (%f)\n",
                  i, j, iscalar);
        ret |= 1<<2;
      }
    }
  }
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
