/* vsiptest_vbind_d.c Version 0.000 May 12, 1997
// 
// This file contains the vsiptest_vbind_d test functionality.
// 
// Functional Description:
//   vsip_vbind_d binds a vector view to a block object
//  
//
// Copyright May 1997, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: This test suite will create real blocks of data with
//
// Revision History:
// May 21, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i,j;
  vsip_cblock_f *iblock, *sblock;
  vsip_cvview_f *iview, *sview;
  vsip_scalar_f  rval, ival;
  vsip_cscalar_f cval;
  int            have_malloc_failer = vthas_fail_mech();
  int            BLOCK_SIZE = VSIPTEST_CVBIND_F_BLOCK_SIZE;
  int            ret = 0;
  vsip_scalar_f *array1, *array2, *array3;
  vsip_cvattr_f   attrib;
  vsip_offset    offset;
  vsip_stride    stride;
  vsip_length    length;

  vsip_init ((void *)0);
  if (have_malloc_failer)
  {
      iblock = vsip_cblockcreate_f (1,0);
      vtfail_next_malloc(1);
      iview = vsip_cvbind_f (iblock, 0, 1, 1);
      if (iview != NULL)
      {
          vtprintf ("Error in vsip_cvbind_f() (1)\n"
                    "Failed bind did not return NULL\n");
          ret |= 1<<1;
      }
  }

  array1 = (vsip_scalar_f *)malloc(BLOCK_SIZE*sizeof(vsip_scalar_f));
  array2 = (vsip_scalar_f *)malloc(BLOCK_SIZE*sizeof(vsip_scalar_f));
  array3 = (vsip_scalar_f *)malloc(BLOCK_SIZE*sizeof(vsip_scalar_f) * 2);
  if (array1 && array2 && array3)
  {
    offset = 0; 
    stride = 1;
    length = BLOCK_SIZE;

    vtfill_array (array1, BLOCK_SIZE,   0, 1);
    vtfill_array (array2, BLOCK_SIZE,   1, 3);
    vtfill_array (array3, BLOCK_SIZE*2, 5, 5);

    sblock = vsip_cblockbind_f (array1, array2, BLOCK_SIZE, 0);
    iblock = vsip_cblockbind_f (array3, NULL, BLOCK_SIZE, 0);
    vsip_cblockadmit_f (sblock, 1);
    vsip_cblockadmit_f (iblock, 1);


    sview = vsip_cvbind_f (sblock, 0, 1, BLOCK_SIZE);
    iview = vsip_cvbind_f (iblock, 0, 1, BLOCK_SIZE);

    if (!sview)
    {
      vtprintf ("Error in vsip_cvbind_f() (2)\n"
                "Failed to bind view to split block "
                "at offset 0, stride 1, length %i\n",
                BLOCK_SIZE);
      ret |= 2;
    }
    else
    {
      for (i=0; i<BLOCK_SIZE; i++)
      {
        cval = vsip_cvget_f (sview, i);
        rval = vsip_real_f (cval);
        ival = vsip_imag_f (cval);
        if (vtfcomp (rval, (vsip_scalar_f)i) ||
            vtfcomp (ival, (vsip_scalar_f)(i*3+1) ) )
        {
          vtprintf ("Error in vsip_vbind_f (3)\n"
                    "Element %i in split block "
                    "does not match data array for offset 0"
                    " stride 1, length %i\n", i, BLOCK_SIZE);
          ret |= 3;
          break;
        }
      }
    }
    if (!iview)
    {
      vtprintf ("Error in vsip_cvbind_f() (4)\n"
                "Failed to bind view to interleaved block "
                "at offset 0, stride 1, length %i\n",
                BLOCK_SIZE);
      ret |= 4;
    }
    else
    {
      for (i=0; i<BLOCK_SIZE; i++)
      {
        rval = vsip_real_f (vsip_cvget_f (iview, i));
        ival = vsip_imag_f (vsip_cvget_f (iview, i));
        if (vtfcomp (rval, (vsip_scalar_f)i * 10 + 5) ||
            vtfcomp (ival, (vsip_scalar_f)i * 10 + 10) )
        {
          vtprintf ("Error in vsip_vbind_f (5)\n"
                    "Element %i in interleaved block "
                    "does not match data array for offset 0"
                    " stride 1, length %i\n", i, BLOCK_SIZE);
          ret |= 5;
          break;
        }
      }
    }
    vsip_cvgetattrib_f (sview, &attrib);
    if (attrib.offset != offset ||
        attrib.stride != stride || 
        attrib.length != length)
    {
      vtprintf ("Error in vsip_cvbind_f (6)\n"
                "View of split block of offset %i stride %i legnth %i "
                "did not match attributes\n",
                offset, stride, length);
      ret |= 6;
    }

    vsip_cvgetattrib_f (iview, &attrib);
    if (attrib.offset != offset ||
        attrib.stride != stride || 
        attrib.length != length)
    {
      vtprintf ("Error in vsip_cvbind_f (7)\n"
                "View of interleaved block of offset %i stride %i legnth %i "
                "did not match attributes\n",
                offset, stride, length);
      ret |= 7;
    }
    vsip_cvdestroy_f (iview);
    vsip_cvdestroy_f (sview);
    vsip_cblockdestroy_f (sblock);
    vsip_cblockdestroy_f (iblock);

    vtfill_array (array1, BLOCK_SIZE,   0, 3);
    vtfill_array (array2, BLOCK_SIZE,   1, 4);
    vtfill_array (array3, BLOCK_SIZE*2, 3, 5);

    offset = BLOCK_SIZE/2 - 1;
    stride = 1;
    length = BLOCK_SIZE - BLOCK_SIZE/2 + 1;

    sblock = vsip_cblockbind_f (array1, array2, BLOCK_SIZE, 0);
    iblock = vsip_cblockbind_f (array3, NULL, BLOCK_SIZE, 0);
    vsip_cblockadmit_f (sblock, 1);
    vsip_cblockadmit_f (iblock, 1);
    iview = vsip_cvbind_f (iblock, offset, stride, length);
    sview = vsip_cvbind_f (sblock, offset, stride, length);

    if (!iview)
    {
      vtprintf ("Error in vsip_cvbind_f() (8)\n"
                "Failed to bind view to interleaved block "
                "at offset %i, stride 1, length %i\n",
                offset, length);
      ret |= 8;
    }
    else
    {
      for (i=0; i<length; i++)
      {
        rval = vsip_real_f (vsip_cvget_f (iview, i));
        ival = vsip_imag_f (vsip_cvget_f (iview, i));
        if (vtfcomp (rval,(i*stride+offset)* 10 + 3)   ||
            vtfcomp (ival,(i*stride+offset)* 10 + 8) )
        {
          vtprintf ("Error in vsip_cvbind_f (9)\n"
                    "Element %i of interleaved block "
                    "does not match data array for offset %i"
                    " stride 1, length %i\n", i, offset, stride);
          ret |= 9;
          break;
        }
      }
    }
    vsip_cvgetattrib_f (iview, &attrib);
    if (attrib.offset != offset ||
        attrib.stride != stride || 
        attrib.length != length)
    {
      vtprintf ("Error in vsip_cvbind_f (10)\n"
                "View of interleaved block of "
                "offset %i stride %i legnth %i did not match attributes\n",
                offset, stride, length);
      ret |= 10;
    }
    vsip_cvdestroy_f (iview);

    if (!sview)
    {
      vtprintf ("Error in vsip_cvbind_f() (11)\n"
                "Failed to bind view to split block "
                "at offset %i, stride 1, length %i\n",
                offset, length);
      ret |= 11;
    }
    else
    {
      for (i=0; i<length; i++)
      {
        rval = vsip_real_f (vsip_cvget_f (sview, i));
        ival = vsip_imag_f (vsip_cvget_f (sview, i));
        if (vtfcomp (rval, (i*stride+offset)*3) ||
            vtfcomp (ival, (i*stride+offset)*4 + 1) )
        {
          vtprintf ("Error in vsip_cvbind_f (12)\n"
                    "Element %i of split block "
                    "does not match data array for offset %i"
                    " stride 1, length %i\n", i, offset, stride, rval, ival);
          ret |= 12;
          break;
        }
      }
    }
    vsip_cvgetattrib_f (sview, &attrib);
    if (attrib.offset != offset ||
        attrib.stride != stride || 
        attrib.length != length)
    {
      vtprintf ("Error in vsip_cvbind_f (13)\n"
                "View of split block of "
                "offset %i stride %i legnth %i did not match attributes\n",
                offset, stride, length);
      ret |= 13;
    }
    vsip_cvdestroy_f (sview);

    for (i=0; i<elements(VSIPTEST_CVBIND_F_OFFSET); i++)

    {
      offset = VSIPTEST_CVBIND_F_OFFSET[i];
      stride = VSIPTEST_CVBIND_F_STRIDE[i];
      length = VSIPTEST_CVBIND_F_LENGTH[i];

      iview = vsip_cvbind_f (iblock, offset, stride, length);
      if (!iview)
      {
        vtprintf ("Error in vsip_cvbind_f (14)\n"
                  "View of interleaved block "
                  "of offset %i stride %i length %i could not be bound"
                  " to VSIP created block\n", offset, stride, length);
        ret |= 14;
      }
       
      for (j=0; j<length; j++)
      {
        rval = vsip_real_f (vsip_cvget_f (iview, j));
        ival = vsip_imag_f (vsip_cvget_f (iview, j));
        if (vtfcomp (rval, (offset + stride * j) * 10 + 3) ||
            vtfcomp (ival, (offset + stride * j) * 10 + 8) )
        {
          vtprintf ("Error in vsip_cvbind_f (15)\n"
                    "Element %i of interleaved block of view of "
                    "offset %i stride %i length %i did "
                    "not match data in VSIP created block\n",
                    j, offset, stride, length);
          ret |= 15;
          break;
        }
      }

      vsip_cvgetattrib_f (iview, &attrib);
      if (attrib.offset != offset ||
          attrib.stride != stride || 
          attrib.length != length)
      {
        vtprintf ("Error in vsip_cvbind_f (16)\n"
                  "View of inerleaved block offset %i "
                  "stride %i length %i did not match attributes\n",
                  offset, stride, length);
        ret |= 16;
      }
      vsip_cvdestroy_f (iview);

      sview = vsip_cvbind_f (sblock, offset, stride, length);
      if (!sview)
      {
        vtprintf ("Error in vsip_vbind_f (17)\n"
                  "View of split block of offset %i stride %i "
                  "length %i could not be bound\n",
                  offset, stride, length);
        ret |= 17;
      }
       
      for (j=0; j<length; j++)
      {
        rval = vsip_real_f (vsip_cvget_f (sview, j));
        ival = vsip_imag_f (vsip_cvget_f (sview, j));

        if (vtfcomp (rval, (offset + stride * j)*3) || 
            vtfcomp (ival, (offset + stride * j)*4 + 1) )
        {
          vtprintf ("Error in vsip_vbind_f (18)\n"
                    "Element %i of split block view of offset %i stride %i "
                    "length %i did "
                    "not match data\n", j, offset, stride, length);
          ret |= 18;
          break;
        }
      }
      vsip_cvgetattrib_f (sview, &attrib);
      if (attrib.offset != offset ||
          attrib.stride != stride || 
          attrib.length != length)
      {
        vtprintf ("Error in vsip_vbind_f (19)\n"
                  "View of split block offset %i stride %i length %i "
                  "did not match attributes\n",
                  offset, stride, length);
        ret |= 19;
      }
      vsip_cvdestroy_f (sview);
    }
    vsip_cblockdestroy_f (iblock);
    vsip_cblockdestroy_f (sblock);
  }
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
