/* vsiptest_vcopy_f_f.c Version 0.000 July 28, 1999
// 
// This file contains the vsiptest_vcopy_f_f test functionality.
// 
// Functional Description:
//     vsip_vcopy_f_f retruns a complex scalar formed by two real scalars.
//  
//
// Copyright July 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description:  This test will, for a specified set of block lengths
//                    create a base block, create a sub-block to be the copy
//                    target, create a source, block, fill the base block
//                    with 0, fill the source block with a ramp, and copy the
//                    source to the target.  It will then check the data in the
//                    base block to verify correct values.
//
//                    This test is repeated for each combination of split
//                    and interleaved blocks for source and destination.
//                   
//
// Revision History:
// July 28, 1999   Created
*/

#include"vsiptest_view.h"

void rezero (vsip_cblock_f *cblock, int n);
void check_target (vsip_cvview_f *foo, int n, int err, int *ret);

int main()
{
  int            i,j, n;
  int            ret = 0;
  vsip_cscalar_f  cval;
  vsip_cblock_f  *iblock, *sblock;
  vsip_cblock_f  *iblock2, *sblock2;
  vsip_cvview_f  *ssupervector, *ssource, *starget;
  vsip_cvview_f  *isupervector, *isource, *itarget;
  vsip_scalar_f *array1, *array2, *array3, *array4, *array5, *array6;

  vsip_init ((void *)0);
  for (i=0; i<elements(VSIPTEST_CVCOPY_F_F_BLOCK_SIZE_BASE); i++)
  {
    n = VSIPTEST_VCOPY_F_F_BLOCK_SIZE_BASE[i];
    array1 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*n*2);
    array2 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*n*2);
    array3 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*n*4);
    array4 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*n*2);
    array5 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*n*2);
    array6 = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*n*4);
    if (!array1 || !array2 || !array3 || !array4 || !array5 || !array6)
    {
      vtprintf ("Error in vsip_cvcopy_f_f() (1)\n"
                "Could not allocate data arrays for block size %i\n", n*2);
      ret |= 1<<1;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    sblock = vsip_cblockbind_f (array1, array2, n*2, 0);
    if (!sblock)
    {
      vtprintf ("Error in vsip_cvcopy_f_f() (2)\n"
                "Could not bind split block for block size %i\n", n*2);
      ret |= 1<<2;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    iblock = vsip_cblockbind_f (array3, NULL, n*2, 0);
    if (!iblock)
    {
      vtprintf ("Error in vsip_cvcopy_f_f() (3)\n"
                "Could not bind interleaved block for block size %i\n", n*2);
      ret |= 1<<3;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    vsip_cblockadmit_f (sblock, 1);
    vsip_cblockadmit_f (iblock, 1);

    ssupervector = vsip_cvbind_f (sblock, 0, 1, n*2);
    if (!ssupervector)
    {
      vtprintf ("Error in vsip_cvcopy_f_f() (4)\n"
                "Could not create supervector on split block for target "
                "size %i\n", n*2);
      ret |= 1<<4;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    isupervector = vsip_cvbind_f (iblock, 0, 1, n*2);
    if (!isupervector)
    {
      vtprintf ("Error in vsip_cvcopy_f_f() (5)\n"
                "Could not create supervector on interleaved block for target "
                "size %i\n", n*2);
      ret |= 1<<5;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    for (j=0; j<n*2;j++)
    {
      vsip_cvput_f (isupervector, j, cval);
      vsip_cvput_f (ssupervector, j, cval);
    }
     
    starget = vsip_cvsubview_f (ssupervector, n/2, n);
    itarget = vsip_cvsubview_f (isupervector, n/2, n);
    /* verify target views */
    if (!itarget)
    {
      vtprintf ("Error in vsip_cvcopy_f_f() (6)\n"
                "Could not create interleaved subvector of size %i\n", n);
      ret |= 1<<6;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    if (!starget)
    {
      vtprintf ("Error in vsip_cvcopy_f_f() (7)\n"
                "Could not create split subvector of size %i\n", n);
      ret |= 1<<7;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    sblock2 = vsip_cblockbind_f (array4, array5, n, 0);
    if (!sblock2)
    {
      vtprintf ("Error in vsip_cvcopy_f_f() (8)\n"
                "Could not bind split target block of size %i\n", n);
      ret |= 1<<8;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    ssource = vsip_cvbind_f (sblock2, 0, 1, n);
    if (!ssource)
    {
      vtprintf ("Error in vsip_cvcopy_f_f() (9)\n"
                "Could not bind split source vector of size %i\n", n);
      ret |= 1<<9;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    iblock2 = vsip_cblockbind_f (array6, NULL, n, 0);
    if (!iblock2)
    {
      vtprintf ("Error in vsip_cvcopy_f_f() (10)\n"
                "Could not bind interleaved target block of size %i\n", n);
      ret |= 1<<10;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    isource = vsip_cvbind_f (iblock2, 0, 1, n);
    if (!isource)
    {
      vtprintf ("Error in vsip_cvcopy_f_f() (11)\n"
                "Could not bind split source vector of size %i\n", n);
      ret |= 1<<11;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    for (j=0; j<n; j++)
    {
      array4[j]     = 3*j;
      array5[j]     = 3*j+1;
      array6[2*j]   = 3*j;
      array6[2*j+1] = 3*j+1;
    }
    vsip_cblockadmit_f (sblock2, 1);
    vsip_cblockadmit_f (iblock2, 1);

    /* split to split */
    rezero (sblock, n);
    vsip_cvcopy_f_f (ssource, starget);
    check_target(ssupervector, n, 12, &ret);


    /* split to interleaved */
    rezero (iblock, n);
    vsip_cvcopy_f_f (ssource, itarget);
    check_target (isupervector, n, 13, &ret);

    /* int to split */
    rezero (sblock, n);
    vsip_cvcopy_f_f (isource, starget);
    check_target (ssupervector, n, 14, &ret);

    /* int to int */
    rezero (iblock, n);
    vsip_cvcopy_f_f (isource, itarget);
    check_target (isupervector, n, 15, &ret);
  }
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}

void rezero (vsip_cblock_f *cblock, int n)
{
  vsip_scalar_f *array1, *array2;
  int j;

  vsip_cblockrelease_f (cblock, 1, &array1, &array2);
  
  for (j=0; j<n*2; j++)
  {
    array1[j] = 0;
    if (array2)
    {
      array2[j] = 0;
    }
    else
    {
      array1[j+n*2] = 0;
    }
  }
  vsip_cblockadmit_f (cblock, 1);
}


void check_target (vsip_cvview_f *target, int n, int err, int *ret)
{
  int j;

  vsip_scalar_f ival, rval;
  vsip_cscalar_f got;
  vsip_scalar_f rgot, igot;

  for (j=0; j<n*2; j++)
  {
    if (j<n/2 || j>=n/2+n)
    {
      ival = 0;
      rval = 0;
    }
    else
    {
      rval = (j - (n/2))*3;
      ival = (j - (n/2))*3+1;
    }
      
    got = vsip_cvget_f(target, j);
    rgot = vsip_real_f (got);
    igot = vsip_imag_f (got);

    if (vtfcomp (ival, igot) || vtfcomp (rval, rgot))
    {
        vtprintf ("Error in vsip_cvcopy_f_f() (%i)\n"
                  "Value found in target (%f,%f) did not match expected (%f,%f)"
                  " at index %i\n", err, rgot, igot, rval, ival, j);
        *ret |= 1<<err;
        break;
    }
  }
}
