/* vsiptest_vcreate_f.c Version 0.000 May 24, 1997
// 
// This file contains the vsiptest_vbind_d test functionality.
// 
// Functional Description:
//   vsip_vreate_f creates a block, and a view bound to that block
//  
//
// Copyright May 1997, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: This test suite will create real blocks of data with
//
// Revision History:
// May 24, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i,j;
  vsip_cblock_f *cblock;
  vsip_cvview_f *cview;
  int            have_malloc_failer = vthas_fail_mech();
  int            BLOCK_SIZE;
  int            ret = 0;
  vsip_cvattr_f  attrib;
  vsip_cscalar_f cval;
  int result;

  vsip_init ((void *)0);
  if (have_malloc_failer)
  {
    vtfail_next_malloc(1);
    cview = vsip_cvcreate_f (1, 0);
    if (cview != NULL)
    {
        vtprintf ("Error in vsip_cvcreate_f() (1)\n"
                  "Failed create did not return NULL\n");
        ret |= 1<<1;
    }
  }

  for (i=0; i<elements(VSIPTEST_CVCREATE_F_BLOCK_SIZE); i++)
  {
    BLOCK_SIZE = VSIPTEST_CVCREATE_F_BLOCK_SIZE[i];
    cview = vsip_cvcreate_f (BLOCK_SIZE, 0);
    cblock = vsip_cvgetblock_f (cview);

    if (cview == NULL || cblock == NULL)
    {
      vtprintf ("Error in vsip_cvcreate_f() (2)\n"
                "Failed to create %i element block and view\n", BLOCK_SIZE);
      ret |= 1<<2;
    }

    result = vtcheck_cf_block(cblock, BLOCK_SIZE);
    if (result)
    {
      vtprintf ("Error in vsip_cvcreate_f() (3)\n"
                "Block created returned error code 0x%x from validy check\n", 
                result);
      ret |= 1<<3;
    }

    for (j=0; j<BLOCK_SIZE; j++)
    {
      cval.r = 1.1*j;
      cval.i = 1.8*j;
      vsip_cvput_f (cview, j, cval);
    }

    vsip_cvgetattrib_f (cview, &attrib);
    if (attrib.length != BLOCK_SIZE ||
        attrib.offset != 0          ||
        attrib.stride != 1            )
    {
      vtprintf ("Error in vsip_cvcreate_f() (4)\n"
                "Created view did not have correct attributes: %i %i %i\n",
                attrib.offset, attrib.stride, attrib.length);
      ret |= 1<<4;
    }

    for (j=0; j<BLOCK_SIZE; j++)
    {
      cval = vsip_cvget_f (cview, j);
      if (vtfcomp (cval.r, 1.1*j) || 
          vtfcomp (cval.i, 1.8*j) )
      {
        vtprintf ("Error in vsip_cvcreate_f() (5)\n"
                  "Element %i in view of %i element block did not match"
                  " inserted data\n", j, BLOCK_SIZE);
        ret |= 1<<5;
        break;
      }
    }
    vsip_cvdestroy_f (cview);
    vsip_cblockdestroy_f (cblock);
  }
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
