/* vsiptest_cvdestroy_f.c Version 0.000 August 4, 1999
//
// This file contains the vsiptest_cvdestroy_f test functionality.
//
// Functional Description:
//   vsip_cvdestroy_f destroys a view to a complex vector object
//
//
// Copyright August 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Randy Janka
//         randy.janka@gtri.gatech.edu
//         Dan Campbell [adapted from _vdestroy_f]
//         dan.campbell@gtri.gatech.edu
//
// Test description: This test suite will create user blocks of data
//      (including null blocks), then bind a block object and vbind a view object.
//      The vector view object will then be destroyed using the vsip_cvdestroy_f
//      function.  The returned block pointer should be the same as the original
//      block pointer.
//
//      Check some of the parameters of this function
//        1)
//        2)
//        3)
//        4)
//        5)
//        6)
//
// VSIP functions called: vsip_blockbind_f, vsip_blockadmit_f, vsip_blockrelease_f,
//                        vsip_vbind_f, vsip_vdestroy_f
//
// Revision History:
//
// */

#include "vsiptest_view.h"

int main ()
{
  size_t             block_size, j;
  vsip_scalar_f     *public_block1;
  vsip_scalar_f     *public_block2;
  vsip_scalar_f     *public_block3;
  vsip_memory_hint   hint = 0;
  vsip_cblock_f      *block, *return_bind_block;
  vsip_cvview_f      *view;
  vsip_scalar_bl    update = 0;

  int ret = 0;

  vsip_init ((void *)0);
  public_block1 = public_block2 = public_block3 = NULL;

  /* Check various public block sizes from 2 to block_length_max */
  for (j=0; j<elements (VSIPTEST_CVDESTROY_F_SIZE_ARRAY); j++)
  {
    block_size = VSIPTEST_CVDESTROY_F_SIZE_ARRAY[j];
    public_block1 = (vsip_scalar_f *)malloc(block_size*sizeof(vsip_scalar_f));
    public_block2 = (vsip_scalar_f *)malloc(block_size*sizeof(vsip_scalar_f));
    public_block3 = (vsip_scalar_f *)malloc(block_size*2*sizeof(vsip_scalar_f));

    block = vsip_cblockbind_f(public_block1, public_block2, block_size, hint);
    if (block == NULL)
    {
      vtprintf("Error in vsip_cviewdestroy_f(); (3)\n"
               "block bind to %d-el size split block returns failure.\n", 
               block_size);
      ret |= 1<<3;
      break;
    }
    if (vsip_cblockadmit_f(block, update))
    {
      vtprintf ("Error in vsip_cvdestroy_f(); (4)\n"
               "Failed admit of %d-el split user block.\n",block_size);
      ret |= 1<<4;
      break;
    }

    /********************* VSIP_CVBIND_F *****************************
      Create a view on the bound block */

    view = vsip_cvbind_f(block, 0, 1, (vsip_length)block_size);
    if (view == NULL)
    {
      vtprintf ("Error in vsip_cvdestroy_f(); (5)\n"
                "Failed vector bind to %d-el split user block.\n",block_size);
      ret |= 1<<5;
      break;
    }

    /********************* VSIP_CVDESTROY_F ***************************
      Destroy the view on the block and check the block pointer. */
    return_bind_block = vsip_cvdestroy_f(view);
    if (return_bind_block == NULL)
    {
      vtprintf("Error in vsip_cvdestroy_f(); (6) \n"
               "vector destroy of valid user split vector returns failure.\n");
      ret |= 1<<6;
      break;
    }
    if (return_bind_block != block)
    {
      vtprintf("Error in vsip_cvdestroy_f(); (7) \n"
               "block pointer has changed.\n");
      ret |= 1<<7;
      break;
    }
    vsip_cblockdestroy_f (block);

    block = vsip_cblockbind_f(public_block3, NULL, block_size, hint);
    if (block == NULL)
    {
      vtprintf("Error in vsip_cviewdestroy_f(); (8)\n"
               "block bind to %d-el size interleaved returns failure.\n", 
               block_size);
      ret |= 1<<8;
      break;
    }
    if (vsip_cblockadmit_f(block, update))
    {
      vtprintf ("Error in vsip_cvdestroy_f(); (9)\n"
               "Failed admit of %d-el interleaved user block.\n",block_size);
      ret |= 1<<9;
      break;
    }

    /********************* VSIP_CVBIND_F *****************************
      Create a view on the bound block */

    view = vsip_cvbind_f(block, 0, 1, (vsip_length)block_size);
    if (view == NULL)
    {
      vtprintf ("Error in vsip_cvdestroy_f(); (9)\n"
                "Failed vector bind to %d-el interleaved user block.\n",
                block_size);
      ret |= 1<<9;
      break;
    }

    /********************* VSIP_CVDESTROY_F ***************************
      Destroy the view on the block and check the block pointer. */
    return_bind_block = vsip_cvdestroy_f(view);
    if (return_bind_block == NULL)
    {
      vtprintf("Error in vsip_cvdestroy_f(); (10) \n"
               "vector destroy of valid user split vector returns failure.\n");
      ret |= 1<<10;
      break;
    }
    if (return_bind_block != block)
    {
      vtprintf("Error in vsip_cvdestroy_f(); (11) \n"
               "block pointer has changed.\n");
      ret |= 1<<11;
      break;
    }
    vsip_cblockdestroy_f (block);

    block = vsip_cblockcreate_f (block_size, hint);
    if (block == NULL)
    {
      vtprintf("Error in vsip_cviewdestroy_f(); (12)\n"
               "block bind to %d-el size VSIP block returns failure.\n", 
               block_size);
      ret |= 1<<12;
      break;
    }

    /********************* VSIP_CVBIND_F *****************************
      Create a view on the bound block */

    view = vsip_cvbind_f(block, 0, 1, (vsip_length)block_size);
    if (view == NULL)
    {
      vtprintf ("Error in vsip_cvdestroy_f(); (14)\n"
                "Failed vector bind to %d-el VSIP block.\n",block_size);
      ret |= 1<<14;
      break;
    }

    /********************* VSIP_CVDESTROY_F ***************************
      Destroy the view on the block and check the block pointer. */
    return_bind_block = vsip_cvdestroy_f(view);
    if (return_bind_block == NULL)
    {
      vtprintf("Error in vsip_cvdestroy_f(); (15) \n"
               "vector destroy of valid VSIP vector returns failure.\n");
      ret |= 1<<15;
      break;
    }
    if (return_bind_block != block)
    {
      vtprintf("Error in vsip_cvdestroy_f(); (16) \n"
               "block pointer has changed.\n");
      ret |= 1<<16;
      break;
    }
    vsip_cblockdestroy_f (block);

    free(public_block1);
    free(public_block2);
    free(public_block3);
  }
  report (ret); ;
  vsip_finalize ((void *)0);
  return (ret != 0);
}

