/* vsiptest_vput_f.c Version 0.000 June 9, 1999
// 
// This file contains the vsiptest_vput_f test functionality.
// 
// Functional Description:
//   vsip_vput_f 
//
// Copyright June 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//   1) create 3 user arrays with ramp values
//   2) bind and admit arrays to split and interleaved blocks
//   2) create VSIPL block with ramp values
//   3)  for each of several offset, stride, length
//     a) bind a view to each user, vsip blocks
//     b) compare all values in each view sequentially
/      c) compare all values in each view randomly
//
// Revision History:
// June 9, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i,j, k;
  vsip_cblock_f *iblock, *sblock, *vblock;
  vsip_cvview_f *vall_view, *vview, *iview, *sview;
  int            ret = 0;
  vsip_cscalar_f cval;
  vsip_offset    offset;
  vsip_stride    stride;
  vsip_length    length;
  vsip_scalar_f *array1, *array2, *array3;

  int            *order;

  vsip_init ((void *)0);

  order = (int *)malloc(VSIPTEST_CVGET_F_BLOCK_SIZE*sizeof(int));

  vtshuffle (order, VSIPTEST_CVGET_F_BLOCK_SIZE);

  array1 = (vsip_scalar_f *)
           malloc(VSIPTEST_CVGET_F_BLOCK_SIZE*sizeof(vsip_scalar_f));
  array2 = (vsip_scalar_f *)
           malloc(VSIPTEST_CVGET_F_BLOCK_SIZE*sizeof(vsip_scalar_f));
  array3 = (vsip_scalar_f *)
           malloc(VSIPTEST_CVGET_F_BLOCK_SIZE*sizeof(vsip_scalar_f) * 2);

  if (!array1 || !array2 || !array3)
  {
    vtprintf ("Error in vsip_cvget_f() (1)\n"
              "Could not allocate data arrays\n");
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  for (i=0; i<VSIPTEST_CVGET_F_BLOCK_SIZE; i++)
  {
    array1[i]     = (vsip_scalar_f)    i;
    array2[i]     = (vsip_scalar_f)3 * i;
    array3[i*2]   = (vsip_scalar_f)5 * i;
    array3[i*2+1] = (vsip_scalar_f)7 * i;
  }
  sblock = vsip_cblockbind_f (array1, array2, VSIPTEST_CVGET_F_BLOCK_SIZE, 0);
  iblock = vsip_cblockbind_f (array3, NULL,   VSIPTEST_CVGET_F_BLOCK_SIZE, 0);
  if (!iblock)
  {
    vtprintf ("Error in vsip_cvget_f() (2)\n"
              "Could not bind interleaved block to user array\n");
    ret |= 1<<2;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  if (!sblock)
  {
    vtprintf ("Error in vsip_cvget_f() (3)\n"
              "Could not bind split block to user array\n");
    ret |= 1<<3;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  vsip_cblockadmit_f (iblock, 1);
  vsip_cblockadmit_f (sblock, 1);

  vblock = vsip_cblockcreate_f (VSIPTEST_CVGET_F_BLOCK_SIZE, 0);
  if (!vblock)
  {
    vtprintf ("Error in vsip_cvget_f() (4)\n"
              "Could not create VSIPL block\n");
    ret |= 1<<4;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  vall_view = vsip_cvbind_f (vblock, 0, 1, VSIPTEST_CVGET_F_BLOCK_SIZE);
  if (!vall_view)
  {
    vtprintf ("Error in vsip_cvget_f() (5)\n"
              "Could not create view to all of VSIPL block\n");
    ret |= 1<<5;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  for (i=0; i<VSIPTEST_CVGET_F_BLOCK_SIZE; i++)
  {
    cval.r = (vsip_scalar_f) i;
    cval.i = (vsip_scalar_f) 3 * i;
    vsip_cvput_f (vall_view, i, cval);
  }
  vsip_cvdestroy_f (vall_view);

  for (i=0; i<elements(VSIPTEST_CVGET_F_OFFSET); i++)
  {
    offset = VSIPTEST_CVGET_F_OFFSET[i];
    stride = VSIPTEST_CVGET_F_STRIDE[i];
    length = VSIPTEST_CVGET_F_LENGTH[i];

    vtshuffle (order, length);

    /* create a view to each block */
    vview = vsip_cvbind_f (vblock, offset, stride, length);
    iview = vsip_cvbind_f (iblock, offset, stride, length);
    sview = vsip_cvbind_f (sblock, offset, stride, length);

    if (!vview)
    {
      vtprintf ("Error in vsip_cvget_f (6)\n"
                "Could not create view to VSIPL block o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<6;
    }

    if (!iview)
    {
      vtprintf ("Error in vsip_cvget_f (7)\n"
                "Could not create view to interleaved user block "
                "o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<7;
    }

    if (!sview)
    {
      vtprintf ("Error in vsip_cvget_f (8)\n"
                "Could not create view to split user block o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<8;
    }

    /* check each datum */

    for (j=0; j<length; j++)
    {
      cval = vsip_cvget_f (vview, j);
      if (vtfcomp (cval.r, (vsip_scalar_f)(offset + j * stride)) ||
          vtfcomp (cval.i, (vsip_scalar_f)(offset + j * stride) * 3 ) )
      {
        vtprintf ("Error in vsip_cvget_f (9)\n"
                  "Element %i of VSIPL view did not match expected\n"
                  "for sequential access o:%i s:%i l:%i expected:%f got:%f\n", 
                  j,
                  offset, stride, length, (float)(offset + j * stride), cval.r);
        ret |= 1<<9;
        break;
      }
    }

    for (j=0; j<length; j++)
    {
      cval = vsip_cvget_f (iview, j);
      if (vtfcomp (cval.r, (vsip_scalar_f)(offset + j * stride) * 5) ||
          vtfcomp (cval.i, (vsip_scalar_f)(offset + j * stride) * 7) )
      {
        vtprintf ("Error in vsip_cvget_f (10)\n"
                  "Element %i of user interleaved view did not match expected\n"
                  "for sequential access o:%i s:%i l:%i expected:%f got:%f\n",
                  j,
                  offset, stride, length, (float)(offset + j * stride), cval.r);
        ret |= 1<<10;
        break;
      }
    }

    for (j=0; j<length; j++)
    {
      cval = vsip_cvget_f (sview, j);
      if (vtfcomp (cval.r, (vsip_scalar_f)(offset + j * stride) * 1) ||
          vtfcomp (cval.i, (vsip_scalar_f)(offset + j * stride) * 3) )
      {
        vtprintf ("Error in vsip_cvget_f (11)\n"
                  "Element %i of user split view did not match expected\n"
                  "for sequential access o:%i s:%i l:%i expected:%f got:%f\n",
                  j,
                  offset, stride, length, (float)(offset + j * stride), cval.r);
        ret |= 1<<11;
        break;
      }
    }

    for (k=0; k<length; k++)
    {
      j = order[k];
      cval = vsip_cvget_f (vview, j);
      if (vtfcomp (cval.r, (vsip_scalar_f)(offset + j * stride)) ||
          vtfcomp (cval.i, (vsip_scalar_f)(offset + j * stride) * 3) )
      {
        vtprintf ("Error in vsip_cvget_f (12)\n"
                  "Element %i of VSIPL view did not match expected\n"
                  "for random access o:%i s:%i l:%i expected:%f got:%f\n", 
                  offset, stride, length, (float)(offset + j * stride), cval.r);
        ret |= 1<<9;
        break;
      }
    }

    for (k=0; k<length; k++)
    {
      j = order[k];
      cval = vsip_cvget_f (sview, j);
      if (vtfcomp (cval.r, (vsip_scalar_f)(offset + j * stride)) ||
          vtfcomp (cval.i, (vsip_scalar_f)(offset + j * stride) * 3) )
      {
        vtprintf ("Error in vsip_cvget_f (13)\n"
                  "Element %i of user split view did not match expected\n"
                  "for random access o:%i s:%i l:%i expected:%f got:%f\n",
                  offset, stride, length, (float)(offset + j * stride), cval.r);
        ret |= 1<<13;
        break;
      }
    }
    for (k=0; k<length; k++)
    {
      j = order[k];
      cval = vsip_cvget_f (iview, j);
      if (vtfcomp (cval.r, (vsip_scalar_f)(offset + j * stride) * 5) ||
          vtfcomp (cval.i, (vsip_scalar_f)(offset + j * stride) * 7) )
      {
        vtprintf ("Error in vsip_cvget_f (14)\n"
                  "Element %i of user interleaved view did not match "
                  "expected\n"
                  "for random access o:%i s:%i l:%i expected:%f got:%f\n",
                  offset, stride, length, (float)(offset + j * stride)*5, 
                  cval.r);
        ret |= 1<<15;
        break;
      }
    }
    vsip_cvdestroy_f (iview);
    vsip_cvdestroy_f (sview);
    vsip_cvdestroy_f (vview);
  }

  vsip_cblockdestroy_f (vblock);
  vsip_cblockdestroy_f (iblock);
  vsip_cblockdestroy_f (sblock);
  
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
