/* vsiptest_cvputattrib_f.c Version 0.000 August 3, 1999
// 
// This file contains the vsiptest_cvgetattrib_f test functionality.
// 
// Functional Description:
//   vsip_cvgetattrib_f sets the attributes for a float view
//
// Copyright June 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//   1) Create 3 arrays 
//   2) Bind a split and an interleaved block to these arrays, and admit them
//   4) For each of several offset, stride, length
//      a) bind a view to each block
//      b) getattribs on each block
//      d) check the created view to make sure the values match up as 
//         expected for this SOL
//      e) destroy the views
//
// Revision History:
// August 3, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i;
  vsip_cblock_f  *iblock, *sblock, *vblock;
  vsip_cvview_f  *vview, *iview, *sview;
  int            ret = 0;
  vsip_offset    offset;
  vsip_stride    stride;
  vsip_length    length;
  vsip_scalar_f  *array1, *array2, *array3;
  vsip_cvattr_f  attrib2;


  vsip_init ((void *)0);

  array1 = (vsip_scalar_f *)
           malloc(VSIPTEST_CVPUTATTRIB_F_BLOCK_LENGTH*sizeof(vsip_scalar_f));
  array2 = (vsip_scalar_f *)
           malloc(VSIPTEST_CVPUTATTRIB_F_BLOCK_LENGTH*sizeof(vsip_scalar_f));
  array3 = (vsip_scalar_f *)
           malloc(VSIPTEST_CVPUTATTRIB_F_BLOCK_LENGTH*2*sizeof(vsip_scalar_f));

  if (!array1 || !array2 || !array3)
  {
    vtprintf ("Error in vsip_cvgetattrib_f() (1)\n"
              "Could not allocate data arrays\n");
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  sblock = vsip_cblockbind_f (array1, array2,  
                              (vsip_length)VSIPTEST_CVPUTATTRIB_F_BLOCK_LENGTH, 
                              0);
  iblock = vsip_cblockbind_f (array3, NULL,  
                              (vsip_length)VSIPTEST_CVPUTATTRIB_F_BLOCK_LENGTH, 
                              0);

  if (!iblock)
  {
    vtprintf ("Error in vsip_cvgetattrib_f() (2)\n"
              "Could not bind interleaved block to user arrays\n");
    ret |= 1<<2;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  vsip_cblockadmit_f (iblock, 1);

  if (!sblock)
  {
    vtprintf ("Error in vsip_cvgetattrib_f() (3)\n"
              "Could not bind split block to user arrays\n");
    ret |= 1<<3;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  vsip_cblockadmit_f (sblock, 1);

  vblock = vsip_cblockcreate_f (VSIPTEST_CVPUTATTRIB_F_BLOCK_LENGTH, 0);
  if (!vblock)
  {
    vtprintf ("Error in vsip_cvgetattrib_f() (4)\n"
              "Could not create VSIPL block\n");
    ret |= 1<<4;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  for (i=0; i<elements(VSIPTEST_CVPUTATTRIB_F_OFFSET); i++)
  {
    offset = VSIPTEST_CVPUTATTRIB_F_OFFSET[i];
    stride = VSIPTEST_CVPUTATTRIB_F_STRIDE[i];
    length = VSIPTEST_CVPUTATTRIB_F_LENGTH[i];

    /* create a view to each block */
    vview = vsip_cvbind_f (vblock, offset, stride, length);
    sview = vsip_cvbind_f (sblock, offset, stride, length);
    iview = vsip_cvbind_f (iblock, offset, stride, length);

    if (!vview)
    {
      vtprintf ("Error in vsip_cvgetattrib_f (6)\n"
                "Could not create view to VSIPL block o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<6;
    }

    if (!iview)
    {
      vtprintf ("Error in vsip_cvgetattrib_f (7)\n"
                "Could not create view to interleaved user block "
                "o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<7;
    }

    if (!sview)
    {
      vtprintf ("Error in vsip_cvgetattrib_f (8)\n"
                "Could not create view to split user block "
                "o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<8;
    }

    vsip_cvgetattrib_f (sview, &attrib2);
    if (attrib2.offset != offset || 
        attrib2.stride != stride || 
        attrib2.length != length)
    {
      vtprintf ("Error in vsip_cvgetattrib_f (9)\n"
                "Actual attributes (%i, %i, %i) did not match set attributes "
                "(%i, %i, %i) for user block \n", attrib2.offset, attrib2.stride,
                attrib2.length, offset, stride, length);
     ret |= 1<<9;
    }

    vsip_cvgetattrib_f (iview, &attrib2);
    if (attrib2.offset != offset || 
        attrib2.stride != stride || 
        attrib2.length != length)
    {
      vtprintf ("Error in vsip_cvgetattrib_f (10)\n"
                "Actual attributes (%i, %i, %i) did not match set attributes "
                "(%i, %i, %i) for interleaved user block \n", 
                attrib2.offset, attrib2.stride, attrib2.length, 
                offset, stride, length);
     ret |= 1<<10;
    }

    vsip_cvgetattrib_f (vview, &attrib2);

    vsip_cvgetattrib_f (vview, &attrib2);
    if (attrib2.offset != offset || 
        attrib2.stride != stride || 
        attrib2.length != length)
    {
      vtprintf ("Error in vsip_cvgetattrib_f (12)\n"
                "Actual attributes (%i, %i, %i) did not match set attributes "
                "(%i, %i, %i) for VSIPL block \n", attrib2.offset, attrib2.stride,
                attrib2.length, offset, stride, length);
     ret |= 1<<12;
    }

    vsip_cvdestroy_f (sview);
    vsip_cvdestroy_f (iview);
    vsip_cvdestroy_f (vview);
  }

  vsip_cblockdestroy_f (vblock);
  vsip_cblockdestroy_f (sblock);
  vsip_cblockdestroy_f (iblock);
  free (array1);
  free (array2);
  free (array3);
  
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
