/* vsiptest_cvputlength_f.c Version 0.000 August 2, 1999
// 
// This file contains the vsiptest_cvputlength_f test functionality.
// 
// Functional Description:
//   vsip_cvputlength_f sets the length for a complex float view
//
// Copyright August 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//   1) Create 3 array filled with ramp values
//   2) Bind a split and an interleaved block to these arrays, and admit them
//   3) Create a VSIPL block, fill it with ramp values
//   4) For each of several offset, stride, length
//      a) bind a view to each block
//      b) putlength on each block
//      c) check attribs with getattrib
//      d) check the created view to make sure the values match up as 
//         expected for this SOL
//      e) destroy the views
//
// Revision History:
// August 29, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i,j;
  vsip_cblock_f  *iblock, *sblock, *vblock;
  vsip_cvview_f  *vall_view, *vview, *iview, *sview;
  int            ret = 0;
  vsip_offset    offset;
  vsip_stride    stride;
  vsip_length    length;
  vsip_scalar_f  *array1, *array2, *array3;
  vsip_cvattr_f  attrib2;
  vsip_cscalar_f cval;
  vsip_scalar_f  ival, rval, ides, rdes;

  vsip_init ((void *)0);

  array1 = (vsip_scalar_f *)
           malloc(VSIPTEST_CVPUTLENGTH_F_BLOCK_LENGTH*sizeof(vsip_scalar_f));
  array2 = (vsip_scalar_f *)
           malloc(VSIPTEST_CVPUTLENGTH_F_BLOCK_LENGTH*sizeof(vsip_scalar_f));
  array3 = (vsip_scalar_f *)
           malloc(VSIPTEST_CVPUTLENGTH_F_BLOCK_LENGTH*2*sizeof(vsip_scalar_f));

  if (!array1 || !array2 || !array3)
  {
    vtprintf ("Error in vsip_cvputlength_f() (1)\n"
              "Could not allocate data arrays\n");
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  for (i=0; i<VSIPTEST_CVPUTLENGTH_F_BLOCK_LENGTH; i++)
  {
    array1[i]     = (vsip_scalar_f) 2*i;
    array2[i]     = (vsip_scalar_f) 2*i+1;
    array3[i*2]   = (vsip_scalar_f) 2*i;
    array3[i*2+1] = (vsip_scalar_f) 2*i+1;
  }

  sblock = vsip_cblockbind_f (array1, array2,  
                              (vsip_length)VSIPTEST_CVPUTLENGTH_F_BLOCK_LENGTH, 
                              0);
  iblock = vsip_cblockbind_f (array3, NULL,  
                              (vsip_length)VSIPTEST_CVPUTLENGTH_F_BLOCK_LENGTH, 
                              0);

  if (!iblock)
  {
    vtprintf ("Error in vsip_cvputlength_f() (2)\n"
              "Could not bind interleaved block to user arrays\n");
    ret |= 1<<2;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  vsip_cblockadmit_f (iblock, 1);

  if (!sblock)
  {
    vtprintf ("Error in vsip_cvputlength_f() (3)\n"
              "Could not bind split block to user arrays\n");
    ret |= 1<<3;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  vsip_cblockadmit_f (sblock, 1);

  vblock = vsip_cblockcreate_f (VSIPTEST_CVPUTLENGTH_F_BLOCK_LENGTH, 0);
  if (!vblock)
  {
    vtprintf ("Error in vsip_cvputlength_f() (4)\n"
              "Could not create VSIPL block\n");
    ret |= 1<<4;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  vall_view = vsip_cvbind_f (vblock, 0, 1, VSIPTEST_CVPUTLENGTH_F_BLOCK_LENGTH);
  if (!vall_view)
  {
    vtprintf ("Error in vsip_cvputlength_f() (5)\n"
              "Could not create view to all of VSIPL block\n");
    ret |= 1<<5;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  for (i=0; i<VSIPTEST_CVPUTLENGTH_F_BLOCK_LENGTH; i++)
  {
    cval = vsip_cmplx_f ((vsip_scalar_f)2*i, (vsip_scalar_f)2*i+1);
    vsip_cvput_f (vall_view, i, cval);
  }
  vsip_cvdestroy_f (vall_view);

  for (i=0; i<elements(VSIPTEST_CVPUTLENGTH_F_OFFSET); i++)
  {
    offset = VSIPTEST_CVPUTLENGTH_F_OFFSET[i];
    stride = VSIPTEST_CVPUTLENGTH_F_STRIDE[i];
    length = VSIPTEST_CVPUTLENGTH_F_LENGTH[i];

    /* create a view to each block */
    vview = vsip_cvbind_f (vblock, offset, stride, 1);
    sview = vsip_cvbind_f (sblock, offset, stride, 1);
    iview = vsip_cvbind_f (iblock, offset, stride, 1);

    if (!vview)
    {
      vtprintf ("Error in vsip_cvputlength_f (6)\n"
                "Could not create view to VSIPL block o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<6;
    }

    if (!iview)
    {
      vtprintf ("Error in vsip_cvputlength_f, (7)\n"
                "Could not create view to interleaved user block "
                "o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<7;
    }

    if (!sview)
    {
      vtprintf ("Error in vsip_cvputlength_f, (8)\n"
                "Could not create view to split user block "
                "o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<8;
    }

    vsip_cvputlength_f (vview, length);
    vsip_cvputlength_f (iview, length);
    vsip_cvputlength_f (sview, length);

    vsip_cvgetattrib_f (sview, &attrib2);
    if (attrib2.offset != offset || 
        attrib2.stride != stride || 
        attrib2.length != length)
    {
      vtprintf ("Error in vsip_cvputlength_f (9)\n"
                "Actual attributes (%i, %i, %i) did not match set attributes "
                "(%i, %i, %i) for user block \n", attrib2.offset, attrib2.stride,
                attrib2.length, offset, stride, length);
     ret |= 1<<9;
    }

    vsip_cvgetattrib_f (iview, &attrib2);
    if (attrib2.offset != offset || 
        attrib2.stride != stride || 
        attrib2.length != length)
    {
      vtprintf ("Error in vsip_cvputlength_f (10)\n"
                "Actual attributes (%i, %i, %i) did not match set attributes "
                "(%i, %i, %i) for interleaved user block \n", 
                attrib2.offset, attrib2.stride, attrib2.length, 
                offset, stride, length);
     ret |= 1<<10;
    }

    vsip_cvgetattrib_f (sview, &attrib2);
    if (attrib2.offset != offset || 
        attrib2.stride != stride || 
        attrib2.length != length)
    {
      vtprintf ("Error in vsip_cvputlength_f (11)\n"
                "Actual attributes (%i, %i, %i) did not match set attributes "
                "(%i, %i, %i) for split user block \n", attrib2.offset, 
                attrib2.stride, attrib2.length, offset, stride, length);
     ret |= 1<<11;
    }

    vsip_cvgetattrib_f (vview, &attrib2);
    if (attrib2.offset != offset || 
        attrib2.stride != stride || 
        attrib2.length != length)
    {
      vtprintf ("Error in vsip_cvputlength_f (12)\n"
                "Actual attributes (%i, %i, %i) did not match set attributes "
                "(%i, %i, %i) for VSIPL block \n", attrib2.offset, attrib2.stride,
                attrib2.length, offset, stride, length);
     ret |= 1<<12;
    }
    /* check each datum */

    for (j=0; j<length; j++)
    {
      cval = vsip_cvget_f (vview, j);
      rval = vsip_real_f (cval);
      ival = vsip_imag_f (cval);
      rdes = (offset + j * stride) * 2;
      ides = (offset + j * stride) * 2 + 1;
      if (vtfcomp (rval, rdes) || vtfcomp (ival, ides)) 
      {
        vtprintf ("Error in vsip_cvputlength_f (13)\n"
                  "Element %i of VSIPL view did not match expected " 
                  "for o:%i s:%i l:%i expected:%f,%f got:%f,%f\n", 
                  j,
                  offset, stride, length, rdes, ides, rval, ival);
        ret |= 1<<13;
        break;
      }

      cval = vsip_cvget_f (iview, j);
      rval = vsip_real_f (cval);
      ival = vsip_imag_f (cval);
      rdes = (offset + j * stride) * 2;
      ides = (offset + j * stride) * 2 + 1;
      if (vtfcomp (rval, rdes) || vtfcomp (ival, ides)) 
      {
        vtprintf ("Error in vsip_cvputlength_f (14)\n"
                  "Element %i of interleaved user view did not match expected " 
                  "for o:%i s:%i l:%i expected:%f,%f got:%f,%f\n", 
                  j,
                  offset, stride, length, rdes, ides, rval, ival);
        ret |= 1<<14;
        break;
      }

      cval = vsip_cvget_f (sview, j);
      rval = vsip_real_f (cval);
      ival = vsip_imag_f (cval);
      rdes = (offset + j * stride) * 2;
      ides = (offset + j * stride) * 2 + 1;
      if (vtfcomp (rval, rdes) || vtfcomp (ival, ides)) 
      {
        vtprintf ("Error in vsip_cvputlength_f (15)\n"
                  "Element %i of split user view did not match expected " 
                  "for o:%i s:%i l:%i expected:%f,%f got:%f,%f\n", 
                  j,
                  offset, stride, length, rdes, ides, rval, ival);
        ret |= 1<<15;
        break;
      }
    }
    vsip_cvdestroy_f (sview);
    vsip_cvdestroy_f (iview);
    vsip_cvdestroy_f (vview);
  }

  vsip_cblockdestroy_f (vblock);
  vsip_cblockdestroy_f (sblock);
  vsip_cblockdestroy_f (iblock);
  free (array1);
  free (array2);
  free (array3);
  
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
