/* vsiptest_real_f.c Version 0.000 May 24, 1997
// 
// This file contains the vsiptest_real_f test functionality.
// 
// Functional Description:
//     vsip_real_f retruns the real part of a complex scalar.
//  
//
// Copyright July 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description:  This test will fill a complex view with ramp data
//                   get each element, extract the real part, and compare
//                   it to the expected value
//
// Revision History:
// July 26, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i;
  vsip_cvview_f *cview;
  int            BLOCK_SIZE = VSIPTEST_REAL_F_BLOCK_SIZE;
  int            ret = 0;
  vsip_scalar_f  ival=0, rval=0;
  vsip_cscalar_f cval;

  vsip_init ((void *)0);

  cview = vsip_cvcreate_f (VSIPTEST_REAL_F_BLOCK_SIZE, 0);
  if (!cview)
  {
    vtprintf ("Error in vsip_real_f() (1)\n"
              "Failed to create %i element complex block and view\n", 
              BLOCK_SIZE);
    ret |= 1<<1;
  }

  for (i=0; i<VSIPTEST_REAL_F_BLOCK_SIZE; i++)
  {
    rval = i*3;
    ival = i*3+1.5;

    cval = vsip_cmplx_f (rval, rval);
    vsip_cvput_f (cview, i, cval);
  }

  for (i=0; i<VSIPTEST_REAL_F_BLOCK_SIZE; i++)
  {
    cval = vsip_cvget_f (cview, i);
    rval = vsip_real_f (cval);

    if (vtfcomp (rval, i*3))
    {
      vtprintf ("Error in vsip_real_f() (2)\n"
                "Element %i real value of %f did not match expected %f\n",
                i, ival, i*3);
      ret |= 1<<2;
      break;
    }
  }
  vsip_cvalldestroy_f (cview);
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
