/* vsiptest_valldestroy_f.c Version 0.000 August 4, 1999
// 
// This file contains the vsiptest_valldestroy_f test functionality.
// 
// Functional Description:
//   vsip_valldestroy_f destroys a view and its associated block
//
// Copyright August 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//   1) For a preset number of times:
//     a) Create a VSIPL block
//     b) bind a view to the block
//     c) valldestroy on the view
//     
//   NOTE:  in performance mode, there is very little that can be checked. 
//          implementors may wish to do a memory leak check around this test.
//
// Revision History:
// August 4, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i;
  int            ret = 0;
  vsip_block_f  *block;
  vsip_vview_f  *vview;
  int            BLOCK_SIZE = VSIPTEST_VALLDESTROY_F_BLOCK_SIZE;
  vsip_scalar_f *array;

  vsip_init ((void *)0);

  for (i=0; i<VSIPTEST_VALLDESTROY_F_ITERATIONS; i++)
  {
    block = vsip_blockcreate_f (BLOCK_SIZE, 0);
    if (!block)
    {
      vtprintf ("Error in vsip_valldestroy_f() (1)\n"
                "Could not create VSIP block\n");
      ret |= 1<<1;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    vview = vsip_vbind_f (block, 0, 1, BLOCK_SIZE);
    if (!vview)
    {
      vtprintf ("Error in vsip_valldestoroy_f() (2)\n"
                "Could not create view to VSIP block\n");
      ret |= 1<<2;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    vsip_valldestroy_f (vview);

    vview = vsip_vcreate_f (BLOCK_SIZE, 0);
    if (!vview)
    {
      vtprintf ("Error in vsip_valldestroy_f() (3)\n"
                "Could not create VSIP view and block\n");
      ret |= 1<<3;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    vsip_valldestroy_f (vview);

    array = (vsip_scalar_f *)malloc(sizeof(vsip_scalar_f)*BLOCK_SIZE);
    block = vsip_blockbind_f (array, BLOCK_SIZE, 0);
    if (!block)
    {
      vtprintf ("Error in vsip_valldestroy_f() (4)\n"
                "Could not create user block\n");
      ret |= 1<<4;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }

    vview = vsip_vbind_f (block, 0, 1, BLOCK_SIZE);
    if (!vview)
    {
      vtprintf ("Error in vsip_valldestoroy_f() (5)\n"
                "Could not create view to user block\n");
      ret |= 1<<5;
      report (ret); 
      vsip_finalize ((void *)0);
      return (ret != 0);
    }
    vsip_valldestroy_f (vview);
    free (array);
  }
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
