/* vsiptest_vbind_i.c Version 0.000 June 17, 1999
// 
// This file contains the vsiptest_vbind_i test functionality.
// 
// Functional Description:
//   vsip_vbind_i binds a vector view to a block object
//  
//
// Copyright June 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: This test suite will create real blocks of data with
//
// Revision History:
// June 17, 1999   Created
*/

#include "vsiptest_view.h"
#include "vsip.h"

void vput_i (vsip_vview_i *view, int pos, vsip_scalar_i val);
vsip_scalar_i vget_i (vsip_vview_i *view, int pos);

int main()
{
  int            i,j;
  vsip_block_i  *block;
  vsip_block_i  *vsipl_block = NULL;
  vsip_vview_i  *view;
  int            have_malloc_failer = vthas_fail_mech();
  int            BLOCK_SIZE = VSIPTEST_VBIND_I_BLOCK_SIZE;
  int            ret = 0;
  vsip_scalar_i *array;
  vsip_vattr_i   attrib;
  vsip_offset    offset;
  vsip_stride    stride;
  vsip_length    length;

  vsip_init ((void *)0);

  if (have_malloc_failer)
  {
      block = vsip_blockcreate_i (10, 0);
      vtfail_next_malloc(1);
      view = vsip_vbind_i (block, 0, 1, 1);
      if (view != NULL)
      {
          vtprintf ("Error in vsip_vbind_i() (1)\n"
                    "Failed bind did not return NULL\n");
          ret |= 1<<1;
      }
  }

  array = (vsip_scalar_i *)malloc(BLOCK_SIZE*sizeof(vsip_scalar_i));
  if (array)
  {
    offset = 0; 
    stride = 1;
    length = BLOCK_SIZE;

    for (i=0; i<BLOCK_SIZE; i++)
    {
      array[i] = i;
    }
    block = vsip_blockbind_i (array, BLOCK_SIZE, 0);
    view = vsip_vbind_i (block, 0, 1, BLOCK_SIZE);
    if (!view)
    {
      vtprintf ("Error in vsip_vbind_i() (2)\n"
                "Failed to bind view at offset 0, stride 1, length %i\n",
                BLOCK_SIZE);
      ret |= 2;
    }
    else
    {
      for (i=0; i<BLOCK_SIZE; i++)
      {
        if (vget_i(view, i) != i)
        {
          vtprintf ("Error in vsip_vbind_i (3)\n"
                    "Element %i does not match data array for offset 0"
                    " stride 1, length %i\n", i, BLOCK_SIZE);
          ret |= 3;
          break;
        }
      }
    }
    vsip_vgetattrib_i (view, &attrib);
    if (attrib.offset != offset ||
        attrib.stride != stride || 
        attrib.length != length)
    {
      vtprintf ("Error in vsip_vbind_i (4)\n"
                "View of offset %i stride %i legnth %i did not match "
                "attributes\n", offset, stride, length);
      ret |= 4;
    }

    vsip_vdestroy_i (view);
    vsip_blockdestroy_i (block);

    for (i=0; i<BLOCK_SIZE; i++)
    {
      array[i] = i * 3;
    }

    offset = BLOCK_SIZE/2 - 1;
    stride = 1;
    length = BLOCK_SIZE - BLOCK_SIZE/2 + 1;

    block = vsip_blockbind_i (array, BLOCK_SIZE, 0);
    view = vsip_vbind_i (block, offset, stride, length);
    if (!view)
    {
      vtprintf ("Error in vsip_vbind_i() (5)\n"
                "Failed to bind view at offset %i, stride 1, length %i\n",
                offset, length);
      ret |= 5;
    }
    else
    {
      for (i=0; i<length; i++)
      {
        if (vget_i(view, i) != (i*stride+offset)*3)
        {
          vtprintf ("Error in vsip_vbind_i (6)\n"
                    "Element %i does not match data array for offset %i"
                    " stride 1, length %i\n", i, offset, stride);
          ret |= 6;
          break;
        }
      }
    }
    vsip_vgetattrib_i (view, &attrib);
    if (attrib.offset != offset ||
        attrib.stride != stride || 
        attrib.length != length)
    {
      vtprintf ("Error in vsip_vbind_i (7)\n"
                "View of offset %i stride %i legnth %i did not match attributes\n",
                offset, stride, length);
      ret |= 7;
    }
    vsip_vdestroy_i (view);
    vsip_blockdestroy_i (block);

    for (i=0; i<BLOCK_SIZE; i++)
    {
      array[i] = 5*i;
    }

    offset = BLOCK_SIZE - 1;
    stride = 1;
    length = BLOCK_SIZE - offset;

    block = vsip_blockbind_i (array, BLOCK_SIZE, 0);
    view = vsip_vbind_i (block, offset, stride, length);
    if (!view)
    {
      vtprintf ("Error in vsip_vbind_i() (8)\n"
                "Failed to bind view at offset %i, stride 1, length %i\n",
                offset, length);
      ret |= 8;
    }
    else
    {
      for (i=0; i<length; i++)
      {
        if (vget_i(view, i) != (i*stride+offset)*5)
        {
          vtprintf ("Error in vsip_vbind_i (9)\n"
                    "Element %i does not match data array for offset %i"
                    " stride 1, length %i\n", i, offset, stride);
          ret |= 9;
          break;
        }
      }
    }
    vsip_vgetattrib_i (view, &attrib);
    if (attrib.offset != offset ||
        attrib.stride != stride || 
        attrib.length != length)
    {
      vtprintf ("Error in vsip_vbind_i (9)\n"
                "View of offset %i stride %i legnth %i did not match attributes\n",
                offset, stride, length);
      ret |= 9;
    }
    vsip_vdestroy_i (view);
    vsip_blockdestroy_i (block);

        
    block = vsip_blockcreate_i (BLOCK_SIZE, 0);
    vsipl_block = block;
    view = vsip_vbind_i (block, 0, 1, BLOCK_SIZE);
    {
      for (i=0; i<BLOCK_SIZE; i++)
      {
        vput_i (view, i, i);
      }
    }
    vsip_vdestroy_i (view);

    offset = BLOCK_SIZE/2 - 1;
    stride = 1;
    length = BLOCK_SIZE - BLOCK_SIZE/2 + 1;

    view = vsip_vbind_i (block, offset, stride, length);
    if (!view)
    {
      vtprintf ("Error in vsip_vbind_i() (10)\n"
                "Failed to bind view at offset %i, stride 1, length %i for "
                "VSIP created block\n", offset, length);
      ret |= 10;
    }
    else
    {
      for (i=1; i<length; i++)
      {
        if (vget_i(view, i) != (i*stride+offset))
        {
          vtprintf ("Error in vsip_vbind_i (11)\n"
                    "Element %i does not match data array for offset %i "
                    "stride 1, length %i (%f)in VSIP created block\n", 
                    i, offset, stride, vget_i (view, i));
          ret |= 11;
          break;
        }
      }
    }
    vsip_vgetattrib_i (view, &attrib);
    if (attrib.offset != offset ||
        attrib.stride != stride || 
        attrib.length != length)
    {
      vtprintf ("Error in vsip_vbind_i (12)\n"
                "View of offset %i stride %i legnth %i did not match "
                "attributes\n", offset, stride, length);
      ret |= 12;
    }
    vsip_vdestroy_i (view);

    offset = BLOCK_SIZE - 1;
    stride = 1;
    length = BLOCK_SIZE - offset;

    view = vsip_vbind_i (block, offset, stride, length);
    if (!view)
    {
      vtprintf ("Error in vsip_vbind_i() (13)\n"
                "Failed to bind view at offset %i, stride 1, length %i\n",
                offset, length);
      ret |= 13;
    }
    else
    {
      for (i=0; i<length; i++)
      {
        if (vget_i(view, i) !=  (i*stride+offset))
        {
          vtprintf ("Error in vsip_vbind_i (14)\n"
                    "Element %i does not match data array for offset %i"
                    " stride 1, length %i in VSIP created block\n", 
                    i, offset, stride);
          ret |= 14;
          break;
        }
      }
    }
    vsip_vgetattrib_i (view, &attrib);
    if (attrib.offset != offset ||
        attrib.stride != stride || 
        attrib.length != length)
    {
      vtprintf ("Error in vsip_vbind_i (15)\n"
                "View of offset %i stride %i length %i did not match attributes\n",
                offset, stride, length);
      ret |= 15;
    }

    vsip_vdestroy_i (view);

    for (i=0; i<BLOCK_SIZE; i++)
    {
      array[i] = i;
    }

    for (i=0; i<elements(VSIPTEST_VBIND_I_OFFSET); i++)
    {
      offset = VSIPTEST_VBIND_I_OFFSET[i];
      stride = VSIPTEST_VBIND_I_STRIDE[i];
      length = VSIPTEST_VBIND_I_LENGTH[i];

      block = vsipl_block;
      view = vsip_vbind_i (block, offset, stride, length);
      if (!view)
      {
        vtprintf ("Error in vsip_vbind_i (16)\n"
                  "View of offset %i stride %i length %i could not be bound"
                  " to VSIP created block\n", offset, stride, length);
        ret |= 16;
      }
       
      for (j=0; j<length; j++)
      {
        if (vget_i (view, j) != (offset + stride * j))
        {
          vtprintf ("Error in vsip_vbind_j (17)\n"
                    "Element %i of view of offset %i stride %i length %i did "
                    "not match data in VSIP created block\n",
                    j, offset, stride, length);
          ret |= 17;
          break;
        }
      }

      vsip_vgetattrib_i (view, &attrib);
      if (attrib.offset != offset ||
          attrib.stride != stride || 
          attrib.length != length)
      {
        vtprintf ("Error in vsip_vbind_i (18)\n"
                  "View of offset %i stride %i length %i did not match "
                  "attributes\n", offset, stride, length);
        ret |= 18;
      }

      vsip_vdestroy_i (view);

      block = vsip_blockbind_i (array, BLOCK_SIZE, 0);
      view = vsip_vbind_i (block, offset, stride, length);
      if (!view)
      {
        vtprintf ("Error in vsip_vbind_i (19)\n"
                  "View of offset %i stride %i length %i could not be bound\n",
                  offset, stride, length);
        ret |= 19;
      }
       
      for (j=0; j<length; j++)
      {
        if (vget_i (view, j) != (offset + stride * j))
        {
          vtprintf ("Error in vsip_vbind_i (20)\n"
                    "Element %i of view of offset %i stride %i length %i did "
                    "not match data\n", j, offset, stride, length);
          ret |= 20;
        }
      }

      vsip_vgetattrib_i (view, &attrib);
      if (attrib.offset != offset ||
          attrib.stride != stride || 
          attrib.length != length)
      {
        vtprintf ("Error in vsip_vbind_i (21)\n"
                  "View of offset %i stride %i length %i did not match attributes\n",
                  offset, stride, length);
        ret |= 21;
      }

      vsip_vdestroy_i (view);
      vsip_blockdestroy_i (block);
    }
  }
  vsip_blockdestroy_i (vsipl_block);
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}



