/* vsiptest_vcreate_f.c Version 0.000 May 24, 1997
// 
// This file contains the vsiptest_vbind_d test functionality.
// 
// Functional Description:
//   vsip_vreate_f creates a block, and a view bound to that block
//  
//
// Copyright May 1997, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: This test suite will create real blocks of data with
//
// Revision History:
// May 24, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i,j;
  vsip_block_f  *block;
  vsip_vview_f  *view;
  int            have_malloc_failer = vthas_fail_mech();
  int            BLOCK_SIZE;
  int            ret = 0;
  vsip_vattr_f   attrib;
  vsip_scalar_f  val;
  int result;

  vsip_init ((void *)0);
  if (have_malloc_failer)
  {
    vtfail_next_malloc(1);
    view = vsip_vcreate_f (1, 0);
    if (view != NULL)
    {
        vtprintf ("Error in vsip_vcreate_f() (1)\n"
                  "Failed create did not return NULL\n");
        ret |= 1<<1;
    }
    vsip_valldestroy_f (view);
  }

  for (i=0; i<elements(VSIPTEST_VCREATE_F_BLOCK_SIZE); i++)
  {
    BLOCK_SIZE = VSIPTEST_VCREATE_F_BLOCK_SIZE[i];
    view = vsip_vcreate_f (BLOCK_SIZE, 0);
    block = vsip_vgetblock_f (view);

    if (view == NULL || block == NULL)
    {
      vtprintf ("Error in vsip_vcreate_f() (2)\n"
                "Failed to create %i element block and view\n", BLOCK_SIZE);
      ret |= 1<<2;
    }

    result = vtcheck_f_block(block, BLOCK_SIZE);
    if (result)
    {
      vtprintf ("Error in vsip_vcreate_f() (3)\n"
                "Block created returned error code 0x%x from validy check\n", 
                result);
      ret |= 1<<3;
    }

    for (j=0; j<BLOCK_SIZE; j++)
    {
      vsip_vput_f (view, j, (vsip_scalar_f)1.1*j);
    }

    vsip_vgetattrib_f (view, &attrib);
    if (attrib.length != BLOCK_SIZE ||
        attrib.offset != 0          ||
        attrib.stride != 1            )
    {
      vtprintf ("Error in vsip_vcreate_f() (4)\n"
                "Created view did not have correct attributes: %i %i %i\n",
                attrib.offset, attrib.stride, attrib.length);
      ret |= 1<<4;
    }

    for (j=0; j<BLOCK_SIZE; j++)
    {
      val = vsip_vget_f (view, j);
      if (vtfcomp (val, 1.1*j))
      {
        vtprintf ("Error in vsip_vcreate_f() (5)\n"
                  "Element %i in view of %i element block did not match"
                  " inserted data\n", j, BLOCK_SIZE);
        ret |= 1<<5;
        break;
      }
    }
    vsip_vdestroy_f (view);
    vsip_blockdestroy_f (block);
  }
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
