/* vsiptest_vdestroy_f.c Version 0.000 May 21, 1999
//
// This file contains the vsiptest_vdestroy_f test functionality.
//
// Functional Description:
//   vsip_vdestroy_f destroys a view to a vector object
//
//
// Copyright May 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Randy Janka
//         randy.janka@gtri.gatech.edu
//
// Test description: This test suite will create user blocks of data
//      (including null blocks), then bind a block object and vbind a view object.
//      The vector view object will then be destroyed using the vsip_vdestroy_f
//      function.  The returned block pointer should be the same as the original
//      block pointer.
//
//      Check some of the parameters of this function
//        1)
//        2)
//        3)
//        4)
//        5)
//        6)
//
// VSIP functions called: vsip_blockbind_f, vsip_blockadmit_f, vsip_blockrelease_f,
//                        vsip_vbind_f, vsip_vdestroy_f
//
// Revision History:
//
// */

#include "vsiptest_view.h"

int main ()
{
  size_t             block_size=0, i, j;
  vsip_scalar_f     *public_block;
  vsip_memory_hint   hint = 0;
  vsip_block_f      *bind_block, *return_bind_block;
  vsip_vview_f      *bind_view;
  vsip_scalar_bl    update = 0;

  int ret = 0;

  vsip_init ((void *)0);

  public_block = NULL;

  /* Check various public block sizes from 2 to block_length_max */
  for (j=0; j<elements (VSIPTEST_VDESTROY_F_SIZE_ARRAY); j++)
  {
    block_size = VSIPTEST_VDESTROY_F_SIZE_ARRAY[j];
    public_block = (vsip_scalar_f *)malloc(block_size*sizeof(vsip_scalar_f));

    /* Fill the block with data values equal to its index value */
    for(i=0; i<block_size; i++)
    {
      *(public_block + i) = (vsip_scalar_f)i;
    }

    /********** VSIP_BLOCKBIND_F VSIP_BLOCKADMIT_F ************
      Bind to the public block & admit it for vector binding. */

    bind_block = vsip_blockbind_f(public_block, block_size, hint);
    if (bind_block == NULL)
    {
      vtprintf("Error in vsip_vdestroy_f(); (3)\n"
               "block bind to %d-el size block returns failure.\n", block_size);
      ret |= 1<<3;
      break;
    }
    if (vsip_blockadmit_f(bind_block, update))
    {
      vtprintf ("Error in vsip_vdestroy_f(); (4)\n"
               "Failed admit of %d-el block.\n",block_size);
      ret |= 1<<4;
      break;
    }

    /****************************************************************/

    /********************* VSIP_VBIND_F *****************************
      Create a view on the bound block */

    bind_view = vsip_vbind_f(bind_block, 0, 1, (vsip_length)block_size);
    if (bind_view == NULL)
    {
      vtprintf ("Error in vsip_vdestroy_f(); (5)\n"
               "Failed vector bind to %d-el block.\n",block_size);
      ret |= 1<<5;
      break;
    }

    /********************* VSIP_VDESTROY_F ***************************
      Destroy the view on the block and check the block pointer. */
    return_bind_block = vsip_vdestroy_f(bind_view);
    if (return_bind_block == NULL)
    {
      vtprintf("Error in vsip_vdestroy_f(); (6) \n"
               "vector destroy of valid vector returns failure.\n");
      ret |= 1<<6;
      break;
    }
    if (return_bind_block != bind_block)
    {
      vtprintf("Error in vsip_vdestroy_f(); (7) \n"
               "block pointer has changed.\n");
      ret |= 1<<7;
      break;
    }

    /********************* VSIP_BLOCKRELEASE_F *******************
      Release the block free the memory. */
    vsip_blockdestroy_f(bind_block);
    free(public_block);
  }
  report (ret); ;
  vsip_finalize ((void *)0);
  return (ret != 0);
}

