/* vsiptest_vget_f.c Version 0.000 June 9, 1999
// 
// This file contains the vsiptest_vget_f test functionality.
// 
// Functional Description:
//   vsip_vget_f 
//
// Copyright June 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//   1) create user array with ramp values
//   2) bind and admit array
//   2) create VSIPL block with ramp values
//   3)  for each of several offset, stride, length
//     d) bind a view to user, vsip blocks
//     g) compare all values in each view sequentially
/      h) compare all values in each view randomly
//
// Revision History:
// June 9, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i,j, k;
  vsip_block_f  *ublock, *vblock;
  vsip_vview_f  *vall_view, *vview, *uview;
  int            ret = 0;
  vsip_scalar_f  val;
  vsip_offset    offset;
  vsip_stride    stride;
  vsip_length    length;
  vsip_scalar_f *array;

  int            *order;

  vsip_init ((void *)0);
  order = (int *)malloc(VSIPTEST_VGET_F_BLOCK_SIZE*sizeof(int));

  vtshuffle (order, VSIPTEST_VGET_F_BLOCK_SIZE);

  array = (vsip_scalar_f *)
           malloc(VSIPTEST_VGET_F_BLOCK_SIZE*sizeof(vsip_scalar_f));

  if (!array)
  {
    vtprintf ("Error in vsip_vget_f() (1)\n"
              "Could not allocate data array\n");
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  for (i=0; i<VSIPTEST_VGET_F_BLOCK_SIZE; i++)
  {
    array[i] = (vsip_scalar_f)i;
  }
  ublock = vsip_blockbind_f (array, VSIPTEST_VGET_F_BLOCK_SIZE, 0);
  if (!ublock)
  {
    vtprintf ("Error in vsip_vget_f() (2)\n"
              "Could not bind block to user array\n");
    ret |= 1<<2;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  vsip_blockadmit_f (ublock, 1);

  vblock = vsip_blockcreate_f (VSIPTEST_VGET_F_BLOCK_SIZE, 0);
  if (!vblock)
  {
    vtprintf ("Error in vsip_vget_f() (3)\n"
              "Could not create VSIPL block\n");
    ret |= 1<<3;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  vall_view = vsip_vbind_f (vblock, 0, 1, VSIPTEST_VGET_F_BLOCK_SIZE);
  if (!vall_view)
  {
    vtprintf ("Error in vsip_vget_f() (4)\n"
              "Could not create view to all of VSIPL block\n");
    ret |= 1<<4;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  vsip_vramp_f (0.0, 1.0, vall_view);
  vsip_vdestroy_f (vall_view);

  for (i=0; i<elements(VSIPTEST_VGET_F_OFFSET); i++)
  {
    offset = VSIPTEST_VGET_F_OFFSET[i];
    stride = VSIPTEST_VGET_F_STRIDE[i];
    length = VSIPTEST_VGET_F_LENGTH[i];

    vtshuffle (order, length);

    /* create a view to each block */
    vview = vsip_vbind_f (vblock, offset, stride, length);
    uview = vsip_vbind_f (ublock, offset, stride, length);

    if (!vview)
    {
      vtprintf ("Error in vsip_vget_f (5)\n"
                "Could not create view to VSIPL block o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<5;
    }

    if (!uview)
    {
      vtprintf ("Error in vsip_vget_f (6)\n"
                "Could not create view to user block o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<6;
    }

    /* check each datum */

    for (j=0; j<length; j++)
    {
      val = vsip_vget_f (vview, j);
      if (vtfcomp (val, (vsip_scalar_f)(offset + j * stride)))
      {
        vtprintf ("Error in vsip_vget_f (7)\n"
                  "Element %i of VSIPL view did not match expected\n"
                  "for sequential access o:%i s:%i l:%i expected:%f got:%f\n", 
                  offset, stride, length, (float)(offset + j * stride), val);
        ret |= 1<<7;
        break;
      }
    }

    for (j=0; j<length; j++)
    {
      val = vsip_vget_f (uview, j);
      if (vtfcomp (val, (vsip_scalar_f)(offset + j * stride)))
      {
        vtprintf ("Error in vsip_vget_f (8)\n"
                  "Element %i of user view did not match expected\n"
                  "for sequential access o:%i s:%i l:%i expected:%f got:%f\n",
                  offset, stride, length, (float)(offset + j * stride), val);
        ret |= 1<<8;
        break;
      }
    }
    for (k=0; k<length; k++)
    {
      j = order[k];
      val = vsip_vget_f (vview, j);
      if (vtfcomp (val, (vsip_scalar_f)(offset + j * stride)))
      {
        vtprintf ("Error in vsip_vget_f (9)\n"
                  "Element %i of VSIPL view did not match expected\n"
                  "for sequential access o:%i s:%i l:%i expected:%f got:%f\n", 
                  offset, stride, length, (float)(offset + j * stride), val);
        ret |= 1<<9;
        break;
      }
    }

    for (k=0; k<length; k++)
    {
      j = order[k];
      val = vsip_vget_f (uview, j);
      if (vtfcomp (val, (vsip_scalar_f)(offset + j * stride)))
      {
        vtprintf ("Error in vsip_vget_f (10)\n"
                  "Element %i of user view did not match expected\n"
                  "for random access o:%i s:%i l:%i expected:%f got:%f\n",
                  offset, stride, length, (float)(offset + j * stride), val);
        ret |= 1<<10;
        break;
      }
    }
    vsip_vdestroy_f (uview);
    vsip_vdestroy_f (vview);
  }
  vsip_blockdestroy_f (vblock);
  vsip_blockdestroy_f (ublock);
  
  report (ret); 
  vsip_finalize ((void *)0);
  return (ret != 0);
}
