/* vsiptest_vputattrib_i.c Version 0.000 August 2, 1999
// 
// This file contains the vsiptest_vputattrib_i test functionality.
// 
// Functional Description:
//   vsip_vputattrib_i sets the attributes for an integer view
//
// Copyright August 1999, Georgia Institute of Technology.
// This is an unpublished work. All rights reserved.
//
// Author: Dan Campbell 
//         dan.campbell@gtri.gatech.edu
//
// Test description: 
//   1) Create a user array filled with ramp values
//   2) Bind a block to this array, and admit it
//   3) Create a VSIPL block, fill it with ramp values
//   4) For each of several offset, stride, length
//      a) bind a view to each block
//      b) putattribs on each block
//      c) check attribs with getattrib
//      d) check the created view to make sure the values match up as 
//         expected for this SOL
//      e) destroy the views
//
// Revision History:
// August 2, 1999   Created
*/

#include"vsiptest_view.h"

int main()
{
  int            i,j;
  vsip_block_i  *block, *vblock;
  vsip_vview_i  *vall_view, *vview, *uview;
  int            ret = 0;
  vsip_offset    offset;
  vsip_stride    stride;
  vsip_length    length;
  vsip_scalar_i  *array;
  vsip_scalar_i  val;
  vsip_vattr_i   attrib, attrib2;
  
  vsip_init ((void *)0);

  array = (vsip_scalar_i *)
           malloc(VSIPTEST_VPUTATTRIB_I_BLOCK_LENGTH*sizeof(vsip_scalar_i));

  if (!array)
  {
    vtprintf ("Error in vsip_vputattrib_i() (1)\n"
              "Could not allocate data arrays\n");
    ret |= 1<<1;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  for (i=0; i<VSIPTEST_VPUTATTRIB_I_BLOCK_LENGTH; i++)
  {
    array[i]     = (vsip_scalar_i)    i;
  }

  block = vsip_blockbind_i (array, VSIPTEST_VPUTATTRIB_I_BLOCK_LENGTH, 0);
  if (!block)
  {
    vtprintf ("Error in vsip_vputattrib_i() (2)\n"
              "Could not bind block to user array\n");
    ret |= 1<<2;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  vsip_blockadmit_i (block, 1);

  vblock = vsip_blockcreate_i (VSIPTEST_VPUTATTRIB_I_BLOCK_LENGTH, 0);
  if (!vblock)
  {
    vtprintf ("Error in vsip_vputattrib_i() (3)\n"
              "Could not create VSIPL block\n");
    ret |= 1<<3;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }

  vall_view = vsip_vbind_i (vblock, 0, 1, VSIPTEST_VPUTATTRIB_I_BLOCK_LENGTH);
  if (!vall_view)
  {
    vtprintf ("Error in vsip_vputattrib_i() (4)\n"
              "Could not create view to all of VSIPL block\n");
    ret |= 1<<5;
    report (ret); 
    vsip_finalize ((void *)0);
    return (ret != 0);
  }
  for (i=0; i<VSIPTEST_VPUTATTRIB_I_BLOCK_LENGTH; i++)
  {
    vput_i (vall_view, i, (vsip_scalar_i)i);
  }
  vsip_vdestroy_i (vall_view);

  for (i=0; i<elements(VSIPTEST_VPUTATTRIB_I_OFFSET); i++)
  {
    offset = VSIPTEST_VPUTATTRIB_I_OFFSET[i];
    stride = VSIPTEST_VPUTATTRIB_I_STRIDE[i];
    length = VSIPTEST_VPUTATTRIB_I_LENGTH[i];

    /* create a view to each block */
    vview = vsip_vbind_i (vblock, 0, 1, 1);
    uview = vsip_vbind_i (block, 0, 1, 1);

    if (!vview)
    {
      vtprintf ("Error in vsip_vputattrib_i (5)\n"
                "Could not create view to VSIPL block o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<5;
    }

    if (!uview)
    {
      vtprintf ("Error in vsip_vputattrib_i (6)\n"
                "Could not create view to interleaved user block "
                "o:%i s:%i l:%i\n",
                offset, stride, length);
      ret |= 1<<6;
    }

    attrib.stride = stride;
    attrib.offset = offset;
    attrib.length = length;

    vsip_vputattrib_i (vview, &attrib);
    vsip_vputattrib_i (uview, &attrib);

    vsip_vgetattrib_i (uview, &attrib2);
    if (attrib2.offset != offset || 
        attrib2.stride != stride || 
        attrib2.length != length)
    {
      vtprintf ("Error in vsip_vputattribf_i (7)\n"
                "Actual attributes (%i, %i, %i) did not match set attributes "
                "(%i, %i, %i) for user block \n", attrib2.offset, attrib2.stride,
                attrib2.length, offset, stride, length);
     ret |= 1<<7;
    }

    vsip_vgetattrib_i (vview, &attrib2);
    if (attrib2.offset != offset || 
        attrib2.stride != stride || 
        attrib2.length != length)
    {
      vtprintf ("Error in vsip_vputattribf_i (8)\n"
                "Actual attributes (%i, %i, %i) did not match set attributes "
                "(%i, %i, %i) for VSIPL block \n", attrib2.offset, attrib2.stride,
                attrib2.length, offset, stride, length);
     ret |= 1<<8;
    }
    /* check each datum */

    for (j=0; j<length; j++)
    {
      val = vget_i (vview, j);
      if (vtfcomp (val, (vsip_scalar_i)(offset + j * stride))) 
      {
        vtprintf ("Error in vsip_vputattrib_i (9)\n"
                  "Element %i of VSIPL view did not match expected " 
                  "for o:%i s:%i l:%i expected:%f got:%f\n", 
                  j,
                  offset, stride, length, (float)(offset + j * stride), val);
        ret |= 1<<9;
        break;
      }
      val = vget_i (uview, j);
      if (vtfcomp (val, (vsip_scalar_i)(offset + j * stride)))
      {
        vtprintf ("Error in vsip_vputattrib_i (10)\n"
                  "Element %i of VSIPL view did not match expected " 
                  "for o:%i s:%i l:%i expected:%f got:%f\n", 
                  j,
                  offset, stride, length, (float)(offset + j * stride), val);
        ret |= 1<<10;
        break;
      }
    }
    vsip_vdestroy_i (uview);
    vsip_vdestroy_i (vview);
  }

  vsip_blockdestroy_i (vblock);
  vsip_blockdestroy_i (block);
  free (array);
  
  report (ret); 
    vsip_finalize ((void *)0);
  return (ret != 0);
}
